@@ -1,54 +1,58 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
+from __future__ import division
+
 import copy
 import inspect
-from itertools import chain
 import os
 import re
 import sys
 import traceback
-import xml.dom
 
+import vistrails
 from vistrails.core import debug
 from vistrails.core import get_vistrails_application
+from vistrails.core.configuration import get_vistrails_configuration
 from vistrails.core.modules.module_descriptor import ModuleDescriptor
 from vistrails.core.utils import versions_increasing, VistrailsInternalError
-from vistrails.core.utils.uxml import (named_elements, enter_named_element)
-
 from vistrails.db.domain import DBPackage
 
+
+vistrails_dir = os.path.dirname(os.path.realpath(vistrails.__file__))
+
 ##############################################################################
 
 class Package(DBPackage):
@@ -138,6 +142,8 @@ class Package(DBPackage):
             self.prefix = None
             self.py_dependencies = set()
             self.old_identifiers = []
+            self._default_configuration = None
+            self._persistent_configuration = None
         else:
             self._module = other._module
             self._init_module = other._init_module
@@ -148,6 +154,11 @@ class Package(DBPackage):
             self.prefix = other.prefix
             self.py_dependencies = copy.copy(other.py_dependencies)
             self.old_identifiers = [i for i in self.old_identifiers]
+            self._default_configuration = \
+                                        copy.copy(other._default_configuration)
+            self._persistent_configuration = \
+                                    copy.copy(other._persistent_configuration)
+
         # FIXME decide whether we want None or ''
         if self.version is None:
             self.version = ''
@@ -251,9 +262,9 @@ class Package(DBPackage):
     ##########################################################################
     # Methods
 
-    _python_lib_regex = re.compile(r'python[0-9.]+[a-z]?/lib/',
+    _python_lib_regex = re.compile(r'python[0-9.]+[a-z]?/lib/(?!vistrails)',
                                    re.IGNORECASE)
-    _lib_python_regex = re.compile(r'lib/python[0-9.]+[a-z]?/',
+    _lib_python_regex = re.compile(r'lib/python[0-9.]+[a-z]?/(?!vistrails)',
                                    re.IGNORECASE)
     def import_override(self, orig_import,
                         name, globals, locals, fromlist, level,
@@ -271,6 +282,8 @@ class Package(DBPackage):
                 pkg_fname = pkg.__file__
             except AttributeError:
                 return True
+            if os.path.realpath(pkg_fname).startswith(vistrails_dir):
+                return False
             if "site-packages" in pkg_fname:
                 return True
             if os.sep != '/':
@@ -340,9 +353,6 @@ class Package(DBPackage):
     def get_py_deps(self):
         return self.py_dependencies
 
-    def remove_py_deps(self, deps):
-        self.py_dependencies.difference_update(deps)
-
     def load(self, prefix=None):
         """load(module=None). Loads package's module.
 
@@ -399,18 +409,8 @@ class Package(DBPackage):
         if r:
             raise self.InitializationFailed(self, errors)
 
-        # Sometimes we don't want to change startup.xml, for example
-        # when peeking at a package that's on the available package list
-        # on edit -> preferences. That's what the load_configuration field
-        # is for
-        if self.load_configuration:
-            if hasattr(self._module, 'configuration'):
-                # hold a copy of the initial configuration so it can be reset
-                self._initial_configuration = \
-                    copy.copy(self._module.configuration)
-            self.load_persistent_configuration()
-
         self.set_properties()
+        self.do_load_configuration()
 
     def initialize(self):
         if not self._loaded:
@@ -448,17 +448,19 @@ class Package(DBPackage):
 
             if hasattr(self._init_module, 'initialize'):
                 self._init_module.initialize()
-        except Exception:
+        except Exception, e:
+            debug.unexpected_exception(e)
             self.unload()
             raise
 
     def unload(self):
         for path in self.py_dependencies:
             if path not in sys.modules:
-                # print "skipping %s" % path
                 pass
-            else:
-                # print 'deleting path:', path, path in sys.modules
+            elif not getattr(get_vistrails_configuration(),
+                             'dontUnloadModules',
+                             False):
+                debug.debug("deleting from sys.modules: %s" % path)
                 del sys.modules[path]
         self.py_dependencies.clear()
         self._loaded = False
@@ -623,18 +625,19 @@ class Package(DBPackage):
                 debug.critical("Couldn't finalize %s: %s: %s" % (
                                self.name, type(e).__name__, ', '.join(e.args)))
         # Save configuration
-        if self.configuration:
-            self.set_persistent_configuration()
+        if self.load_configuration and self.configuration is not None:
+            self.persist_configuration()
         self.unload()
         self._module = None
         self._init_module = None
         self._initialized = False
 
     def dependencies(self):
+        deps = []
         try:
             callable_ = self._module.package_dependencies
         except AttributeError:
-            deps = []
+            pass
         else:
             try:
                 deps = callable_()
@@ -647,119 +650,54 @@ class Package(DBPackage):
             deps.extend(self._module._dependencies)
         return deps
 
-    def loaded(self):
-        return self._loaded
-
     def initialized(self):
         return self._initialized
 
     ##########################################################################
     # Configuration
 
-    def _get_package_node(self, dom, create):
-        doc = dom.documentElement
-        packages = enter_named_element(doc, 'packages')
-        for package_node in named_elements(packages, 'package'):
-            if package_node.attributes['name'].value == self.codepath:
-                return package_node, 'enabled'
-        oldpackages = enter_named_element(doc, 'disabledpackages')
-        for package_node in named_elements(oldpackages, 'package'):
-            if package_node.attributes['name'].value == self.codepath:
-                return package_node, 'disabled'
+    def _get_persistent_configuration(self):
+        return self._persistent_configuration
+    def _set_persistent_configuration(self, config):
+        self._persistent_configuration = config
+    persistent_configuration = property(_get_persistent_configuration,
+                                        _set_persistent_configuration)
 
-        if create is None:
-            return None, None
-        else:
-            package_node = dom.createElement('package')
-            package_node.setAttribute('name', self.codepath)
-            if create == 'enabled':
-                packages.appendChild(package_node)
-            elif create == 'disabled':
-                oldpackages.appendChild(package_node)
-            else:
-                raise ValueError
-            get_vistrails_application().vistrailsStartup.write_startup_dom(dom)
-            return package_node, create
+    def do_load_configuration(self):
+        # Sometimes we don't want to change startup.xml, for example
+        # when peeking at a package that's on the available package list
+        # on edit -> preferences. That's what the load_configuration field
+        # is for
+        if self.load_configuration:
+            if self.configuration is not None:
+                # hold a copy of the initial configuration so it can be reset
+                self._default_configuration = copy.copy(self.configuration)
 
-    def _move_package_node(self, dom, where, node):
-        doc = dom.documentElement
-        packages = enter_named_element(doc, 'packages')
-        oldpackages = enter_named_element(doc, 'disabledpackages')
-        if where == 'enabled':
-            oldpackages.removeChild(node)
-            packages.appendChild(node)
-        elif where == 'disabled':
-            packages.removeChild(node)
-            oldpackages.appendChild(node)
-        else:
-            raise ValueError
-        get_vistrails_application().vistrailsStartup.write_startup_dom(dom)
+                # now we update the actual configuration in place so it is
+                # available to the package itself
+                if self.persistent_configuration is not None:
+                    self.configuration.update(self.persistent_configuration)
+                else:
+                    # we don't have a persisted configuration so we
+                    # should create one
+                    self.persistent_configuration = \
+                                                copy.copy(self.configuration)
+                    self.persist_configuration(True)
 
-    def remove_own_dom_element(self):
-        """Moves the node to the <disabledpackages> section.
-        """
-        dom = get_vistrails_application().vistrailsStartup.startup_dom()
-
-        node, section = self._get_package_node(dom, create='disabled')
-        if section == 'enabled':
-            self._move_package_node(dom, 'disabled', node)
+        
+    def persist_configuration(self, no_update=False):
+        if self.load_configuration:
+            if not no_update:
+                self.persistent_configuration.update(self.configuration)
+            # make sure startup is updated to reflect changes
+            get_vistrails_application().startup.persist_pkg_configuration(
+                self.codepath, self.persistent_configuration)
 
     def reset_configuration(self):
         """Reset package configuration to original package settings.
+
         """
-
-        (dom, element) = self.find_own_dom_element()
-        doc = dom.documentElement
-        configuration = enter_named_element(element, 'configuration')
-        if configuration:
-            element.removeChild(configuration)
-        self.configuration = copy.copy(self._initial_configuration)
-
-        startup = get_vistrails_application().vistrailsStartup
-        startup.write_startup_dom(dom)
-
-    def find_own_dom_element(self):
-        """find_own_dom_element() -> (DOM, Node)
-
-        Opens the startup DOM, looks for the element that belongs to the package,
-        and returns DOM and node. Creates a new one in the <disabledpackages>
-        section if none is found.
-        """
-        dom = get_vistrails_application().vistrailsStartup.startup_dom()
-
-        node, section = self._get_package_node(dom, create='disabled')
-        return (dom, node)
-
-    def load_persistent_configuration(self):
-        (dom, element) = self.find_own_dom_element()
-
-        configuration = enter_named_element(element, 'configuration')
-        if configuration:
-            self.configuration.set_from_dom_node(configuration)
-        dom.unlink()
-
-    def set_persistent_configuration(self):
-        (dom, element) = self.find_own_dom_element()
-        child = enter_named_element(element, 'configuration')
-        if child:
-            element.removeChild(child)
-        self.configuration.write_to_dom(dom, element)
-        get_vistrails_application().vistrailsStartup.write_startup_dom(dom)
-        dom.unlink()
-
-    def create_startup_package_node(self):
-        """Writes the node to the <packages> section.
-
-        If it was in the <disabledpackages> section, move it, else, create it.
-        """
-        dom = get_vistrails_application().vistrailsStartup.startup_dom()
-
-        node, section = self._get_package_node(dom, create='enabled')
-        if section == 'disabled':
-            self._move_package_node(dom, 'enabled', node)
-
-        configuration = enter_named_element(node, 'configuration')
-        if configuration:
-            self.configuration.set_from_dom_node(configuration)
-
-        dom.unlink()
+        self.configuration = copy.copy(self._default_configuration)
+        if self.load_configuration:
+            self.persistent_configuration = copy.copy(self.configuration)
+            self.persist_configuration(True)
