@@ -1,34 +1,35 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
@@ -138,6 +139,8 @@
 #             result = result.union(query.run(pipeline, module_ids))
 
 ################################################################################
+from __future__ import division
+
 import xml.sax.saxutils
 
 from vistrails.core.utils import memo_method
@@ -265,7 +268,7 @@ class Query1c(Query):
            vistrails.vistrails_name = %s and
            vistrails.vistrails_id = wf_exec.vistrails_id""", name)
         lst = list(c.fetchall())
-        versions = set([x[1] for x in lst])
+        versions = set(x[1] for x in lst)
         executions = set(lst)
         result = []
         for version in versions:
@@ -292,55 +295,6 @@ class Query1c(Query):
         return result
     
 
-class Query2(Query):
-
-    def run(self, vistrail, name):
-        import vistrails.gui.vis_application
-        c = vistrails.gui.vis_application.logger.db.cursor()
-        c.execute("""
-        select distinct module_id, wf_version from
-        wf_exec, exec, vistrails
-        where
-           wf_exec.wf_exec_id = exec.wf_exec_id and
-           vistrails.vistrails_name = %s and
-           vistrails.vistrails_id = wf_exec.vistrails_id""", name)
-        lst = list(c.fetchall())
-        versions = set([x[1] for x in lst])
-        executions = set(lst)
-        result = []
-        for version in versions:
-            p = vistrail.getPipeline(int(version))
-            inv_graph = p.graph.inverse()
-
-            # s = upstream(x) union x where x.name = filesink blablabla
-            s = set()
-            for module_id, module in p.modules.iteritems():
-                if (module_id, version) not in executions:
-                    continue
-                if module.name == 'FileSink':
-                    for f in module.functions:
-                        if (f.name == 'outputName' and
-                            len(f.params) == 1 and
-                            f.params[0].value() == 'atlas-x.gif'):
-                            s = s.union(set(self.upstream(inv_graph, module_id) + [module_id]))
-                            break
-
-            # s2 = upstream(y) where y.name = softmean
-            s2 = set()
-            for module_id, module in p.modules.iteritems():
-                if module.name == 'SoftMean':
-                    s2 = s2.union(set(self.upstream(inv_graph, module_id) + [module_id]))
-
-            qresult = s - s2
-            
-            for m in qresult:
-                result.append((int(version), m))
-        self.queryResult = result
-        self.tupleLength = 2
-        self.computeIndices()
-        return result
-
-
 class Query3(Query):
 
     def run(self, vistrail, name):
@@ -354,7 +308,7 @@ class Query3(Query):
            vistrails.vistrails_name = %s and
            vistrails.vistrails_id = wf_exec.vistrails_id""", name)
         lst = list(c.fetchall())
-        versions = set([x[1] for x in lst])
+        versions = set(x[1] for x in lst)
         executions = set(lst)
         result = []
         for version in versions:
