@@ -1,53 +1,54 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
+from __future__ import division
+
 from itertools import izip
 import operator
 
 from vistrails.core.data_structures.bijectivedict import Bidict
 from vistrails.core.modules.utils import create_port_spec_string, parse_port_spec_string
-from vistrails.core.system import get_vistrails_basic_pkg_id
+from vistrails.core.system import get_vistrails_basic_pkg_id, \
+    get_module_registry
 from vistrails.core.utils import enum, VistrailsInternalError
 from vistrails.core.vistrail.port_spec_item import PortSpecItem
-from vistrails.db.domain import DBPortSpec
+from vistrails.db.domain import DBPortSpec, IdScope
 
+from ast import literal_eval
 import unittest
 import copy
-from vistrails.db.domain import IdScope
-import vistrails.core
-
-################################################################################
 
 PortEndPoint = enum('PortEndPoint',
                     ['Invalid', 'Source', 'Destination'])
@@ -122,6 +123,8 @@ class PortSpec(DBPortSpec):
             
         if 'sort_key' not in kwargs:
             kwargs['sort_key'] = -1
+        if 'depth' not in kwargs:
+            kwargs['depth'] = 0
         if 'id' not in kwargs:
             kwargs['id'] = -1
         if 'tooltip' in kwargs:
@@ -185,8 +188,6 @@ class PortSpec(DBPortSpec):
 
     @staticmethod
     def convert(_port_spec):
-        from vistrails.core.modules.module_registry import module_registry_loaded, \
-            ModuleRegistryException
         if _port_spec.__class__ == PortSpec:
             return
         _port_spec.__class__ = PortSpec
@@ -219,6 +220,7 @@ class PortSpec(DBPortSpec):
     sort_key = DBPortSpec.db_sort_key
     min_conns = DBPortSpec.db_min_conns
     max_conns = DBPortSpec.db_max_conns
+    _depth = DBPortSpec.db_depth
     port_spec_items = DBPortSpec.db_portSpecItems
     items = DBPortSpec.db_portSpecItems
 
@@ -251,6 +253,12 @@ class PortSpec(DBPortSpec):
         return signature
     signature = property(_get_signature)
 
+    def _get_depth(self):
+        return self._depth or 0
+    def _set_depth(self, depth):
+        self._depth = depth
+    depth = property(_get_depth, _set_depth)
+
     def toolTip(self):
         if self._tooltip is None:
             self.create_tooltip()
@@ -290,19 +298,19 @@ class PortSpec(DBPortSpec):
         if defaults is None:
             defaults = []
         elif isinstance(defaults, basestring):
-            defaults = eval(defaults)
+            defaults = literal_eval(defaults)
         if labels is None:
             labels = []
         elif isinstance(labels, basestring):
-            labels = eval(labels)
+            labels = literal_eval(labels)
         if values is None:
             values = []
         elif isinstance(values, basestring):
-            values = eval(values)
+            values = literal_eval(values)
         if entry_types is None:
             entry_types = []
         elif isinstance(entry_types, basestring):
-            entry_types = eval(entry_types)
+            entry_types = literal_eval(entry_types)
         attrs = [defaults, labels, values, entry_types]
         if items:
             self.set_items(items, *attrs)
@@ -330,7 +338,6 @@ class PortSpec(DBPortSpec):
         # multiple parameters, where each parameter can be either of the above:
         # add_input_port(_, _, [Float, (Integer, 'count')])
 
-        from vistrails.core.modules.module_registry import get_module_registry
         registry = get_module_registry()
         entries = []
         def canonicalize(sig_item):
@@ -398,9 +405,11 @@ class PortSpec(DBPortSpec):
             port_string = self.type.capitalize()
         else:
             port_string = 'Invalid'
-        self._tooltip = "%s port %s\n%s" % (port_string,
+        _depth = " (depth %s)" % self.depth if self.depth else ''
+        self._tooltip = "%s port %s\n%s%s" % (port_string,
                                             self.name,
-                                            self._short_sigstring)
+                                            self._short_sigstring,
+                                            _depth)
         
     ##########################################################################
     # Operators
@@ -410,9 +419,9 @@ class PortSpec(DBPortSpec):
         object. 
 
         """
-        rep = "<portSpec id=%s name=%s type=%s signature=%s />"
+        rep = "<portSpec id=%s name=%s type=%s signature=%s depth=%s />"
         return  rep % (str(self.id), str(self.name), 
-                       str(self.type), str(self.sigstring))
+                       str(self.type), str(self.sigstring), str(self.depth))
 
     def __eq__(self, other):
         """ __eq__(other: PortSpec) -> boolean
