@@ -1,41 +1,44 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
+from __future__ import division
+
+from ast import literal_eval
 import copy
 import sys
-from vistrails.core.system import get_vistrails_default_pkg_prefix, \
-    get_vistrails_basic_pkg_id
+from vistrails.core.system import get_vistrails_basic_pkg_id
 import vistrails.db.services.io
 from vistrails.db.domain import DBOpmProcess, DBOpmArtifact, DBOpmUsed, \
     DBOpmWasGeneratedBy, DBOpmProcessIdCause, DBOpmProcessIdEffect, \
@@ -47,8 +50,6 @@ from vistrails.db.domain import DBOpmProcess, DBOpmArtifact, DBOpmUsed, \
     DBParameter
 from vistrails.db.services.vistrail import materializeWorkflow
 
-sys.path.append('/vistrails/src/trunk/vistrails')
-
 def create_process(item_exec, account, id_scope):
     return DBOpmProcess(id='p' + str(id_scope.getNewId(DBOpmProcess.vtType)),
                         value=DBOpmProcessValue(item_exec),
@@ -267,7 +268,7 @@ def create_opm(workflow, version, log, reg):
             input_list_artifact = found_input_ports['InputList']
             result_artifact = found_output_ports.get('Result', None)
             input_port_list = \
-                eval(found_input_ports['InputPort'].db_parameters[0].db_val)
+                literal_eval(found_input_ports['InputPort'].db_parameters[0].db_val)
             output_port = \
                 found_input_ports['OutputPort'].db_parameters[0].db_val
 
@@ -413,7 +414,7 @@ def create_opm(workflow, version, log, reg):
             out_downstream_artifacts = {}
 
 
-        ctrl_flow_pkg = '%s.control_flow' % get_vistrails_default_pkg_prefix()
+        ctrl_flow_pkg = 'org.vistrails.vistrails.control_flow'
         basic_pkg = get_vistrails_basic_pkg_id()
         all_special_ports = {'%s:Map' % ctrl_flow_pkg:
                                  [{'InputPort': False, 
@@ -469,7 +470,7 @@ def create_opm(workflow, version, log, reg):
                 return artifact
 
             if annotation.db_key == 'used_files':
-                used_files = eval(annotation.db_value)
+                used_files = literal_eval(annotation.db_value)
                 for fname in used_files:
                     if fname not in file_artifacts:
                         artifact = create_artifact_from_filename(fname,
@@ -485,7 +486,7 @@ def create_opm(workflow, version, log, reg):
                     dependencies.append(create_used(process, artifact,
                                                     account, id_scope))
             elif annotation.db_key == 'generated_tables':
-                generated_tables = eval(annotation.db_value)
+                generated_tables = literal_eval(annotation.db_value)
                 for db_tuple in generated_tables:
                     artifact = process_db_tuple(db_tuple)
                     dependencies.append(create_was_generated_by(artifact,
@@ -493,7 +494,7 @@ def create_opm(workflow, version, log, reg):
                                                                 account,
                                                                 id_scope))
             elif annotation.db_key == 'used_tables':
-                used_tables = eval(annotation.db_value)
+                used_tables = literal_eval(annotation.db_value)
                 for db_tuple in used_tables:
                     artifact = process_db_tuple(db_tuple)
                     dependencies.append(create_used(process, artifact,
