@@ -1,43 +1,46 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
 """ This modules builds a widget to visualize workflow execution logs """
+from __future__ import division
+
 from PyQt4 import QtCore, QtGui
 from vistrails.core.vistrail.pipeline import Pipeline
 from vistrails.core.log.module_exec import ModuleExec
 from vistrails.core.log.group_exec import GroupExec
-from vistrails.core.log.loop_exec import LoopExec
+from vistrails.core.log.loop_exec import LoopExec, LoopIteration
 from vistrails.core.log.workflow_exec import WorkflowExec
 from vistrails.gui.pipeline_view import QPipelineView
 from vistrails.gui.theme import CurrentTheme
@@ -52,44 +55,38 @@ import vistrails.core.db.io
 
 class QExecutionItem(QtGui.QTreeWidgetItem):
     """
-    QExecutionListWidget is a widget containing a list of workflow executions.
-    
+    QExecutionItem represents a workflow or module execution.
+
     """
-    def __init__(self, execution, parent=None):
+    def __init__(self, execution, parent=None, prev=None):
         QtGui.QTreeWidgetItem.__init__(self, parent)
         self.execution = execution
         execution.item = self
-
-        # find parent workflow or group
-        if parent is not None:
-            while (parent.parent() is not None and
-                   not isinstance(parent.execution, GroupExec)):
-                parent = parent.parent()
-            self.wf_execution = parent.execution
-        else:
-            self.wf_execution = execution
+        self.modules = []
+        self.wf_item = prev or self
 
         if isinstance(execution, WorkflowExec):
             for item_exec in execution.item_execs:
-                QExecutionItem(item_exec, self)
+                QExecutionItem(item_exec, self, self)
             if execution.completed == -2:
                 brush = CurrentTheme.SUSPENDED_MODULE_BRUSH
             elif execution.completed == 1:
                 brush = CurrentTheme.SUCCESS_MODULE_BRUSH
             else:
                 brush = CurrentTheme.ERROR_MODULE_BRUSH
-                
+
             if execution.db_name:
                 self.setText(0, execution.db_name)
             else:
                 self.setText(0, 'Version #%s' % execution.parent_version )
-        if isinstance(execution, ModuleExec):
+        elif isinstance(execution, ModuleExec):
+            prev.modules.append(self)
             for loop_exec in execution.loop_execs:
-                QExecutionItem(loop_exec, self)
+                QExecutionItem(loop_exec, self, prev)
             if execution.completed == 1:
                 if execution.error:
                     brush = CurrentTheme.ERROR_MODULE_BRUSH
-                    self.wf_execution.completed = -1
+                    self.wf_item.execution.completed = -1
                 elif execution.cached:
                     brush = CurrentTheme.NOT_EXECUTED_MODULE_BRUSH
                 else:
@@ -99,12 +96,16 @@ class QExecutionItem(QtGui.QTreeWidgetItem):
             else:
                 brush = CurrentTheme.ERROR_MODULE_BRUSH
             self.setText(0, '%s' % execution.module_name)
-        if isinstance(execution, GroupExec):
+        elif isinstance(execution, GroupExec):
+            prev.modules.append(self)
             for item_exec in execution.item_execs:
-                QExecutionItem(item_exec, self)
+                if isinstance(item_exec, LoopExec):
+                    QExecutionItem(item_exec, self, prev)
+                else:
+                    QExecutionItem(item_exec, self, self)
             if execution.completed == 1:
                 if execution.error:
-                    self.wf_execution.completed = -1
+                    self.wf_item.execution.completed = -1
                     brush = CurrentTheme.ERROR_MODULE_BRUSH
                 elif execution.cached:
                     brush = CurrentTheme.NOT_EXECUTED_MODULE_BRUSH
@@ -115,12 +116,17 @@ class QExecutionItem(QtGui.QTreeWidgetItem):
             else:
                 brush = CurrentTheme.ERROR_MODULE_BRUSH
             self.setText(0, 'Group')
-        if isinstance(execution, LoopExec):
+        elif isinstance(execution, LoopExec):
+            for iteration in execution.loop_iterations:
+                QExecutionItem(iteration, self, prev)
+            brush = CurrentTheme.MODULE_BRUSH
+            self.setText(0, 'Loop')
+        elif isinstance(execution, LoopIteration):
             for item_exec in execution.item_execs:
-                QExecutionItem(item_exec, self)
+                QExecutionItem(item_exec, self, prev)
             if execution.completed == 1:
                 if execution.error:
-                    self.wf_execution.completed = -1
+                    self.wf_item.execution.completed = -1
                     brush = CurrentTheme.ERROR_MODULE_BRUSH
                 else:
                     brush = CurrentTheme.SUCCESS_MODULE_BRUSH
@@ -128,14 +134,31 @@ class QExecutionItem(QtGui.QTreeWidgetItem):
                 brush = CurrentTheme.SUSPENDED_MODULE_BRUSH
             else:
                 brush = CurrentTheme.ERROR_MODULE_BRUSH
-            self.setText(0, 'Loop #%s' % execution.db_iteration)
-        self.setText(1, '%s' % execution.ts_start)
-        self.setText(2, '%s' % execution.ts_end)
+            self.setText(0, 'Iteration #%s' % execution.iteration)
+
+        self.setText(1, execution.ts_start.strftime(
+                    '%H:%M %d/%m').replace(' 0', ' ').replace('/0', '/'))
+        self.setData(1, QtCore.Qt.UserRole, str(execution.ts_start))
+        #self.setText(2, '%s' % execution.ts_end) end is hidden
         pixmap = QtGui.QPixmap(10,10)
         pixmap.fill(brush.color())
         icon = QtGui.QIcon(pixmap)
         self.setIcon(0, icon)
 
+
+    def __lt__( self, other ):
+
+        tree = self.treeWidget()
+        if ( not tree ):
+            column = 0
+        else:
+            column = tree.sortColumn()
+
+        if column != 1: # only use special sorting for date
+            return super(QExecutionItem, self).__lt__(other)
+
+        return self.data(1, QtCore.Qt.UserRole) < other.data(1, QtCore.Qt.UserRole)
+
 class QExecutionListWidget(QtGui.QTreeWidget):
     """
     QExecutionListWidget is a widget containing a list of workflow executions.
@@ -144,15 +167,17 @@ class QExecutionListWidget(QtGui.QTreeWidget):
     def __init__(self, parent=None):
         QtGui.QTreeWidget.__init__(self, parent)
         self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
-        self.setColumnCount(3)
-        self.setHeaderLabels(['Pipeline', 'Start', 'End'])
+        self.setColumnCount(2)
+        self.setHeaderLabels(['Pipeline', 'Start']) # end is hidden
+        self.header().setDefaultSectionSize(200)
         self.sortByColumn(1, QtCore.Qt.AscendingOrder)
         self.setSortingEnabled(True)
 
     def set_log(self, log=None):
         self.clear()
-        for execution in log:
-            self.addTopLevelItem(QExecutionItem(execution))
+        if log is not None:
+            for execution in log:
+                self.addTopLevelItem(QExecutionItem(execution))
 
     def add_workflow_exec(self, workflow_exec):
         # mark as recent
@@ -218,8 +243,9 @@ class QLegendWidget(QtGui.QWidget):
 class QLogDetails(QtGui.QWidget, QVistrailsPaletteInterface):
     def __init__(self, parent=None):
         QtGui.QWidget.__init__(self, parent)
+        self.controller = None
         self.execution = None
-        self.parentExecution = None
+        self.parentItem = None
         self.set_title("Log Details")
         self.legend = QLegendWidget()
         self.executionList = QExecutionListWidget()
@@ -261,9 +287,15 @@ class QLogDetails(QtGui.QWidget, QVistrailsPaletteInterface):
                                                self.openVersionAction)
 
     def openVersion(self):
-        if not hasattr(self.parentExecution, 'item'):
+
+        parent = self.executionList.selectedItems()
+        if len(parent) != 1:
             return
-        version = self.parentExecution.item.wf_execution.parent_version
+        parent = parent[0]
+        while not isinstance(parent.execution, WorkflowExec):
+            parent = parent.parent()
+
+        version = parent.execution.parent_version
         from vistrails.gui.vistrails_window import _app
         _app.get_current_view().version_selected(version, True)
         self.controller.recompute_terse_graph()
@@ -289,39 +321,45 @@ class QLogDetails(QtGui.QWidget, QVistrailsPaletteInterface):
         if self.isDoubling:
             self.isDoubling = False
             return
-        if isinstance(item.wf_execution, GroupExec):
+        if isinstance(item.wf_item, GroupExec):
             self.backButton.show()
         else:
             self.backButton.hide()
-        self.notify_app(item.wf_execution, item.execution)
+        self.notify_app(item.wf_item, item.execution)
 
-    def notify_app(self, wf_execution, execution):
+    def notify_app(self, wf_item, execution):
         # make sure it is only called once
         if self.isUpdating:
             return
         self.isUpdating = True
         from vistrails.gui.vistrails_window import _app
-        _app.notify("execution_changed", wf_execution, execution)
+        _app.notify("execution_changed", wf_item, execution)
         self.isUpdating = False
 
     def set_controller(self, controller):
         #print '@@@@ QLogDetails calling set_controller'
+        if self.controller == controller:
+            return
+
         self.controller = controller
         self.executionList.controller = self.controller
-        if not hasattr(self.controller, 'loaded_workflow_execs'):
-            self.controller.loaded_workflow_execs = {}
-            for e in self.controller.read_log().workflow_execs:
-                # set workflow names
-                e.db_name = controller.get_pipeline_name(e.parent_version)
-                self.controller.loaded_workflow_execs[e] = e
-        self.log = self.controller.loaded_workflow_execs
+        if self.controller is not None:
+            if not hasattr(self.controller, 'loaded_workflow_execs'):
+                self.controller.loaded_workflow_execs = {}
+                for e in self.controller.read_log().workflow_execs:
+                    # set workflow names
+                    e.db_name = controller.get_pipeline_name(e.parent_version)
+                    self.controller.loaded_workflow_execs[e] = e
+            self.log = self.controller.loaded_workflow_execs
+        else:
+            self.log = None
         self.executionList.set_log(self.log)
 
-    def execution_changed(self, wf_execution, execution):
+    def execution_changed(self, wf_item, execution):
         if not execution:
             return
         self.execution = execution
-        self.parentExecution = wf_execution
+        self.parentItem = wf_item
         text = ''
         if hasattr(execution, 'item') and \
            not execution.item == self.executionList.currentItem():
@@ -334,8 +372,9 @@ class QLogDetails(QtGui.QWidget, QVistrailsPaletteInterface):
             text += 'User: %s\n' % execution.user
         if hasattr(execution, 'cached'):
             text += 'Cached: %s\n' % ("Yes" if execution.cached else 'No')
-        text += 'Completed: %s\n' % {'0':'No', '1':'Yes'}.get(
-                                    str(execution.completed), 'No')
+        if hasattr(execution, 'completed'):
+            text += 'Completed: %s\n' % {'0':'No', '1':'Yes'}.get(
+                                        str(execution.completed), 'No')
         if hasattr(execution, 'error') and execution.error:
             text += 'Error: %s\n' % execution.error
         annotations = execution.db_annotations \
@@ -351,16 +390,16 @@ class QLogDetails(QtGui.QWidget, QVistrailsPaletteInterface):
         if self.isDoubling:
             self.isDoubling = False
             return
-        if isinstance(item.wf_execution, GroupExec):
+        if isinstance(item.wf_item, GroupExec):
             self.backButton.show()
         else:
             self.backButton.hide()
-        self.notify_app(item.wf_execution, item.execution)
+        self.notify_app(item.wf_item, item.execution)
 
     def doubleClick(self, item, col):
         # only difference here is that we should show contents of GroupExecs 
         self.isDoubling = True
-        if isinstance(item.wf_execution, GroupExec):
+        if isinstance(item.wf_item, GroupExec):
             self.backButton.show()
         else:
             self.backButton.hide()
@@ -368,13 +407,13 @@ class QLogDetails(QtGui.QWidget, QVistrailsPaletteInterface):
             # use itself as the workflow
             self.notify_app(item.execution, item.execution)
         else:
-            self.notify_app(item.wf_execution, item.execution)
+            self.notify_app(item.wf_item, item.execution)
 
     def goBack(self):
-        if not isinstance(self.parentExecution, GroupExec):
+        if not isinstance(self.parentItem.execution, GroupExec):
             self.backButton.hide()
-        self.notify_app(self.parentExecution.item.wf_execution,
-                        self.parentExecution)
+        self.notify_app(self.parentItem.item.wf_item,
+                        self.parentItem)
 
     def update_selection(self):
         if hasattr(self.execution, 'item') and \
@@ -388,10 +427,8 @@ class QLogView(QPipelineView):
         self.set_title("Provenance")
         self.log = None
         self.execution = None
-        self.parentExecution = None
+        self.parentItem = None
         self.isUpdating = False
-        # self.exec_to_wf_map = {}
-        # self.workflow_execs = []
         # Hook shape selecting functions
         self.connect(self.scene(), QtCore.SIGNAL("moduleSelected"),
                      self.moduleSelected)
@@ -409,19 +446,18 @@ class QLogView(QPipelineView):
              'publishPaper': [('setEnabled', False, False)],
             })
 
-    def notify_app(self, wf_execution, execution):
+    def notify_app(self, wf_item, execution):
         # make sure it is only called once
         if self.isUpdating:
             return
         self.isUpdating = True
         from vistrails.gui.vistrails_window import _app
-        _app.notify("execution_changed", wf_execution, execution)
+        _app.notify("execution_changed", wf_item, execution)
         self.isUpdating = False
 
 
     def set_controller(self, controller):
         QPipelineView.set_controller(self, controller)
-        #print "@@@ set_controller called", id(self.controller), len(self.controller.vistrail.actions)
         if not hasattr(self.controller, 'loaded_workflow_execs'):
             self.controller.loaded_workflow_execs = {}
             for e in self.controller.read_log().workflow_execs:
@@ -440,18 +476,17 @@ class QLogView(QPipelineView):
         """ moduleSelected(id: int, selectedItems: [QGraphicsItem]) -> None
         """
         if len(selectedItems)!=1 or id==-1:
-            if self.execution != self.parentExecution:
-                self.notify_app(self.parentExecution, self.parentExecution)
-#            self.moduleUnselected()
+            if self.execution != self.parentItem.execution:
+                self.notify_app(self.parentItem, self.parentItem.execution)
             return
 
         item = selectedItems[0]
         if hasattr(item,'execution') and item.execution:
             if self.execution != item.execution:
                 item = self.scene().selectedItems()[0]
-                self.notify_app(self.parentExecution, item.execution)
-        elif self.execution != self.parentExecution:
-                self.notify_app(self.parentExecution, self.parentExecution)
+                self.notify_app(self.parentItem, item.execution)
+        elif self.execution != self.parentItem.execution:
+            self.notify_app(self.parentItem, self.parentItem.execution)
 
     def set_exec_by_id(self, exec_id):
         if not self.log:
@@ -462,7 +497,7 @@ class QLogView(QPipelineView):
         except ValueError:
             return False
         if len(workflow_execs):
-            self.notify_app(workflow_execs[0], workflow_execs[0])
+            self.notify_app(workflow_execs[0].item, workflow_execs[0])
             return True
         return False
 
@@ -472,7 +507,7 @@ class QLogView(QPipelineView):
         workflow_execs = [e for e in self.log
                           if str(e.ts_start) == str(exec_date)]
         if len(workflow_execs):
-            self.notify_app(workflow_execs[0], workflow_execs[0])
+            self.notify_app(workflow_execs[0].item, workflow_execs[0])
             return True
         return False
 
@@ -484,46 +519,33 @@ class QLogView(QPipelineView):
 
             return self.controller.vistrail.getPipeline(version)
         if isinstance(execution, GroupExec):
-            parent = execution.item.wf_execution
+            parent = execution.item.wf_item.execution
             parent_pipeline = self.get_execution_pipeline(parent)
             return parent_pipeline.db_get_module_by_id(
                                    execution.db_module_id).pipeline
 
-    def execution_changed(self, wf_execution, execution):
+    def execution_changed(self, wf_item, execution):
         self.execution = execution
-        if self.parentExecution != wf_execution:
-            self.parentExecution = wf_execution
-            self.pipeline = self.get_execution_pipeline(wf_execution)
+        if self.parentItem != wf_item:
+            self.parentItem = wf_item
+            self.pipeline = self.get_execution_pipeline(wf_item.execution)
             self.update_pipeline()
         self.update_selection()
 
-        # if idx < len(self.workflow_execs) and idx >= 0:
-        #     self.execution = self.workflow_execs[idx]
-        # else:
-        #     self.execution = None
-
-        # self.currentItem = self.workflow_execs[idx]
-        # self.execution = item.execution
-        # self.workflowExecution = item
-        # while self.workflowExecution.parent():
-        #     self.workflowExecution = self.workflowExecution.parent()
-        # self.workflowExecution = self.workflowExecution.execution
-        # self.parentExecution = item
-        # while self.parentExecution.execution.__class__ not in \
-        #         [WorkflowExec, LoopExec, GroupExec]:
-        #     self.parentExecution = self.parentExecution.parent()
-        # self.parentExecution = self.parentExecution.execution
-        # self.showExecution()
-
     def update_pipeline(self):
-        #print "ACTIONS!"
-        #print "#### controller", id(self.controller)
         scene = self.scene()
         scene.clearItems()
         self.pipeline.validate(False)
         
-        module_execs = dict([(e.module_id, e) 
-                             for e in self.parentExecution.item_execs])
+        modules = [(e.execution.module_id, e.execution) for e in self.parentItem.modules
+                                                   if hasattr(e.execution, 'module_id')]
+        modules.reverse()
+        module_execs = {}
+        for id, m in modules:
+            if id not in module_execs:
+                module_execs[id] = []
+            module_execs[id].append(m)
+
         # controller = DummyController(self.pipeline)
         scene.controller = self.controller
         self.moduleItems = {}
@@ -531,7 +553,7 @@ class QLogView(QPipelineView):
             module = self.pipeline.modules[m_id]
             brush = CurrentTheme.PERSISTENT_MODULE_BRUSH
             if m_id in module_execs:
-                e = module_execs[m_id]
+                e = module_execs[m_id][-1]
                 if e.completed == 1:
                     if e.error:
                         brush = CurrentTheme.ERROR_MODULE_BRUSH
@@ -548,9 +570,9 @@ class QLogView(QPipelineView):
             item.controller = self.controller
             self.moduleItems[m_id] = item
             if m_id in module_execs:
-                e = module_execs[m_id]
-                item.execution = e
-                e.module = item
+                for e in module_execs[m_id]:
+                    item.execution = e
+                    e.module = item
             else:
                 item.execution = None
         connectionItems = []
@@ -571,8 +593,8 @@ class QLogView(QPipelineView):
         self.isUpdating = True
         module = None
         if (isinstance(self.execution, ModuleExec) or \
-            (isinstance(self.execution, GroupExec) and
-             self.execution == self.parentExecution)) and \
+            isinstance(self.execution, GroupExec)) and \
+            hasattr(self.execution, 'module') and \
           not self.execution.module.isSelected():
             self.execution.module.setSelected(True)
             module = self.execution.module
