@@ -1,34 +1,35 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
@@ -39,6 +40,8 @@ QInteractiveGraphicsScene
 QInteractiveGraphicsView
 QPIPGraphicsView
 """
+from __future__ import division
+
 from vistrails.core import debug
 from PyQt4 import QtCore, QtGui
 from vistrails.gui.theme import CurrentTheme
@@ -93,7 +96,7 @@ class QInteractiveGraphicsScene(QtGui.QGraphicsScene):
         self.sceneBoundingRect = QtCore.QRectF()
         self.multiSelecting = False
         
-    def updateSceneBoundingRect(self, keep_square=True):
+    def updateSceneBoundingRect(self, keep_square=False):
         """ updateSceneBoundingRect() -> None        
         Compute the actual bounding rect of all shapes, then update
         the scene rect to be much wider for panning
@@ -123,10 +126,10 @@ class QInteractiveGraphicsScene(QtGui.QGraphicsScene):
             else:
                 self.sceneBoundingRect.adjust(0, -diff/2, 0, diff/2)
         panRect = self.sceneBoundingRect.adjusted(
-            -self.sceneBoundingRect.width()*100,
-            -self.sceneBoundingRect.height()*100,
-            self.sceneBoundingRect.width()*100,
-            self.sceneBoundingRect.height()*100)
+            -self.sceneBoundingRect.width()*2,
+            -self.sceneBoundingRect.height()*2,
+            self.sceneBoundingRect.width()*2,
+            self.sceneBoundingRect.height()*2)
         if panRect.width()<1e-6 and panRect.height()<1e-6:
             panRect = QtCore.QRectF(-1000,-1000,2000,2000)
         self.setSceneRect(panRect)
@@ -213,7 +216,7 @@ class QInteractiveGraphicsScene(QtGui.QGraphicsScene):
             pixmap.save(filename)
             self.setBackgroundBrush(brush)
         except Exception, e:
-            debug.critical("Exception: %s"%str(e))
+            debug.critical("Exception saving to PNG", e)
 
 class QInteractiveGraphicsView(QtGui.QGraphicsView):
     """
@@ -236,6 +239,7 @@ class QInteractiveGraphicsView(QtGui.QGraphicsView):
                              QtGui.QPainter.SmoothPixmapTransform)
         self.scaleMax = 2000
         self.scaleRatio = self.scaleMax/10
+        self.scaleOffset = 700
         self.currentScale = self.scaleMax/2
         self.startScroll = (0,0)
         self.lastPos = QtCore.QPoint(0,0)
@@ -273,9 +277,10 @@ class QInteractiveGraphicsView(QtGui.QGraphicsView):
                 changeFlags = pinch.changeFlags()
                 if changeFlags & QtGui.QPinchGesture.ScaleFactorChanged:
                     if self.gestureStartScale is None:
+                        self.computeScale()
                         self.gestureStartScale = self.currentScale
-                    newScale = self.gestureStartScale * \
-                        pinch.property("scaleFactor").toReal()[0]
+                    newScale = self.gestureStartScale + self.scaleMax * \
+                        math.log(pinch.property("scaleFactor"))/2
                     # Clamp the scale
                     if newScale<0: newScale = 0
                     if newScale>self.scaleMax: newScale = self.scaleMax
@@ -405,13 +410,14 @@ class QInteractiveGraphicsView(QtGui.QGraphicsView):
                 # super(QInteractiveGraphicsView, self).mousePressEvent(e)
         else:
             if buttons & QtCore.Qt.RightButton:
+                self.computeScale()
                 if item is None:
                     self.setCursorState(2)
-                    self.computeScale()
                 else:
                     QtGui.QGraphicsView.mousePressEvent(self, e)
             elif buttons & QtCore.Qt.MidButton:
                 self.setCursorState(3)
+                self.computeScale()
                 self.startScroll = (self.horizontalScrollBar().value(),
                                     self.verticalScrollBar().value())
             self.lastPos = QtCore.QPoint(QtGui.QCursor.pos())
@@ -517,9 +523,10 @@ class QInteractiveGraphicsView(QtGui.QGraphicsView):
         """ updateMatrix() -> None
         Update the view matrix with the current scale
         
-        """        
+        """
         matrix = QtGui.QMatrix()
-        power = float(self.currentScale-self.scaleMax/2)/self.scaleRatio
+        power = float(self.currentScale - self.scaleMax/2 - self.scaleOffset
+                      )/self.scaleRatio
         scale = pow(2.0, power)
         matrix.scale(scale, scale)
         self.setMatrix(matrix)
@@ -530,7 +537,8 @@ class QInteractiveGraphicsView(QtGui.QGraphicsView):
         
         """
         self.currentScale = (math.log(self.matrix().m11(), 2.0)*
-                             self.scaleRatio + self.scaleMax/2)
+                             self.scaleRatio + self.scaleMax/2 +
+                             self.scaleOffset)
 
     def setPIPScene(self, scene):
         """ setPIPScene(scene: QGraphicsScene) -> None        
@@ -594,6 +602,7 @@ class QInteractiveGraphicsView(QtGui.QGraphicsView):
 
     def zoomToFit(self):
         self.scene().fitToView(self, True)
+        self.computeScale()
 
     def zoomIn(self):
         self.setUpdatesEnabled(False)
