@@ -1,54 +1,57 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
+from __future__ import division
+
 from PyQt4 import QtCore, QtGui
 from PyQt4.QtCore import pyqtSignal
 
-from vistrails.gui.mashups.mashups_widgets import (QAliasSliderWidget, QDropDownWidget,
-                                         QAliasNumericStepperWidget)
+from vistrails.core import debug
+from vistrails.gui.mashups.mashups_widgets import QAliasNumericStepperWidget, \
+    QAliasSliderWidget, QDropDownWidget
 from vistrails.gui.utils import show_warning, TestVisTrailsGUI
 
-spreadsheet = __import__('vistrails.packages.spreadsheet', globals(), locals(), 
-                         ['spreadsheet_controller'], -1) 
-spreadsheetController = spreadsheet.spreadsheet_controller.spreadsheetController
+from vistrails.packages.spreadsheet.spreadsheet_controller import \
+    spreadsheetController
 
 
 class QMashupAppMainWindow(QtGui.QMainWindow):
     #signals
     appWasClosed = pyqtSignal(QtGui.QMainWindow)
-    
-    def __init__(self, parent=None, vistrail_view=None, dumpcells=False, 
+
+    def __init__(self, parent=None, vistrail_view=None, dumpcells=False,
                  controller=None, version=-1):
         """ QMashupAppMainWindow()
         Initialize an app window from a mashup.
@@ -58,7 +61,7 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         QtGui.QMainWindow.__init__(self, parent)
         self.vtkCells = []
         self.setStatusBar(QtGui.QStatusBar(self))
-    
+
         # Central widget
         centralWidget = QtGui.QWidget()
         self.mainLayout = QtGui.QVBoxLayout()
@@ -99,14 +102,14 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         # Show here to make sure XDisplay info is correct (for VTKCell)
         self.show()
 
-        spreadsheetController.setEchoMode(True)        
+        spreadsheetController.setEchoMode(True)
         #will run to get Spreadsheet Cell events
         (cellEvents, errors) = self.runAndGetCellEvents(useDefaultValues=True)
         if cellEvents:
             self.numberOfCells = len(cellEvents)
             self.initCells(cellEvents)
         if len(errors) > 0:
-            show_warning("VisTrails::Mashup Preview", 
+            show_warning("VisTrails::Mashup Preview",
                          "There was a problem executing the pipeline: %s." %
                          errors)
         # Construct the controllers for aliases
@@ -114,17 +117,17 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         self.cellControls = {}
         self.aliasWidgets = {}
         self.initControls()
-        
-        if self.currentMashup.layout != None:
+
+        if self.currentMashup.layout is not None:
             self.restoreState(QtCore.QByteArray.fromPercentEncoding(
                                 QtCore.QByteArray(self.currentMashup.layout)))
-        
-        if self.currentMashup.geometry != None:
+
+        if self.currentMashup.geometry is not None:
             self.restoreGeometry(QtCore.QByteArray.fromPercentEncoding(
                               QtCore.QByteArray(self.currentMashup.geometry)))
         else:
             self.resize(self.sizeHint())
-                    
+
         # Constructing buttons
         buttonDock = QCustomDockWidget('Control Buttons', self)
         buttonWidget = QtGui.QWidget(buttonDock)
@@ -182,7 +185,7 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
             sequenceLayout.addWidget(self.cb_loop_sequence)
             buttonLayout.addLayout(sequenceLayout, 0, 0, QtCore.Qt.AlignRight)
         buttonLayout.addWidget(self.cb_auto_update, 0, 1, QtCore.Qt.AlignLeft)
-        buttonLayout.addWidget(self.cb_keep_camera, 0, 2, 1, 2, QtCore.Qt.AlignLeft) 
+        buttonLayout.addWidget(self.cb_keep_camera, 0, 2, 1, 2, QtCore.Qt.AlignLeft)
         if self.sequenceOption:
             buttonLayout.addWidget(self.loopButton, 1, 1, QtCore.Qt.AlignRight)
             self.loopButton.setEnabled(False)
@@ -198,34 +201,34 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         buttonDock.setWidget(buttonWidget)
         self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, buttonDock)
         self.controlDocks["__buttons__"] = buttonDock
-        
-        self.saveAllAct = QtGui.QAction("S&ave Combined", self, 
+
+        self.saveAllAct = QtGui.QAction("S&ave Combined", self,
                                         shortcut=QtGui.QKeySequence.SelectAll,
-                                        statusTip="Save combined images to disk", 
+                                        statusTip="Save combined images to disk",
                                         triggered=self.saveAllEvent)
-        self.saveAct = QtGui.QAction("&Save Each", self, 
+        self.saveAct = QtGui.QAction("&Save Each", self,
                                      shortcut=QtGui.QKeySequence.Save,
-                                     statusTip="Save separate images to disk", 
+                                     statusTip="Save separate images to disk",
                                      triggered=self.saveEventAction)
         self.showBuilderAct = QtGui.QAction("VisTrails Main Window", self,
                                             statusTip="Show VisTrails Main Window",
                                             triggered=self.showBuilderWindow)
         self.createMenus()
         self.lastExportPath = ''
-                    
+
     def createMenus(self):
         self.fileMenu = self.menuBar().addMenu("&File")
         self.fileMenu.addAction(self.saveAct)
         self.fileMenu.addAction(self.saveAllAct)
-        
+
         self.viewMenu = self.menuBar().addMenu("&View")
         self.viewMenu.addAction(self.editingModeAct)
-        
+
         self.windowMenu = self.menuBar().addMenu("&Window")
         self.windowMenu.addAction(self.showBuilderAct)
-        
+
     def runAndGetCellEvents(self, useDefaultValues=False):
-        spreadsheetController.setEchoMode(True)        
+        spreadsheetController.setEchoMode(True)
         #will run to get Spreadsheet Cell events
         cellEvents = []
         errors = []
@@ -234,13 +237,13 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
             if res:
                 cellEvents = spreadsheetController.getEchoCellEvents()
         except Exception, e:
-            import traceback
-            print "Executing pipeline failed:", str(e), traceback.format_exc()
+            debug.unexpected_exception(e)
+            print "Executing pipeline failed:", debug.format_exc()
         finally:
             spreadsheetController.setEchoMode(False)
-            
+
         return (cellEvents, errors)
-    
+
     def updateCells(self, info=None):
         # check if we should create a sequence
         if self.cb_loop_sequence.isChecked():
@@ -258,7 +261,7 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         #self.SaveCamera()
         for i in xrange(self.numberOfCells):
             camera = []
-            if (hasattr(self.cellWidgets[i],"getRendererList") and 
+            if (hasattr(self.cellWidgets[i],"getRendererList") and
                 self.cb_keep_camera.isChecked()):
                 for ren in self.cellWidgets[i].getRendererList():
                     camera.append(ren.GetActiveCamera())
@@ -281,7 +284,7 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
                 for i in xrange(self.numberOfCells):
                     self.cellWidgets[i].setPlayerFrame(slider.value())
             return
-        
+
         if not interactive:
             for i in xrange(self.numberOfCells):
                 self.cellWidgets[i].clearHistory()
@@ -311,7 +314,7 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
             # show the result
             self.updateRenderedCells(value if interactive else 0)
             self.is_executing = True
-                
+
             if value >= slider.maximum():
                 break
             value += slider.singleStep()
@@ -319,14 +322,14 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         self.is_executing = False
         slider.setValue(old_value)
         self.loopButton.setEnabled(True)
-        
+
     def updateRenderedCells(self, value):
         """ Show the cell specified by slider info
         """
         self.is_executing = True
         for i in xrange(self.numberOfCells):
             camera = []
-            if (hasattr(self.cellWidgets[i],"getRendererList") and 
+            if (hasattr(self.cellWidgets[i],"getRendererList") and
                 self.cb_keep_camera.isChecked()):
                 for ren in self.cellWidgets[i].getRendererList():
                     camera.append(ren.GetActiveCamera())
@@ -361,7 +364,7 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
                     cell.startPlayer()
                 else:
                     cell.stopPlayer()
-            
+
 
     def timerEvent(self, event):
         if self.steps:
@@ -389,23 +392,23 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
             self.editing = not self.editing
         if not self.editing:
             self.saveSettings()
-               
+
     def saveSettings(self):
         layout = self.saveState().toPercentEncoding()
         geom = self.saveGeometry().toPercentEncoding()
-            
+
         self.currentMashup.layout = layout
         self.currentMashup.geometry = geom
-        
+
         self.controller.setChanged(True)
-        
+
         #self.controller.writeMashuptrail()
-   
+
     def closeEvent(self, event):
         self.saveSettings()
         self.appWasClosed.emit(self)
         event.accept()
-        
+
     def auto_update_changed(self, state):
         if state == QtCore.Qt.Unchecked:
             self.updateButton.setEnabled(True)
@@ -432,23 +435,23 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
                     cell._player.raise_()
                     cell._player.show()
                     cell.hide()
-        
+
     def loop_int_changed(self, state):
-            self.loopButton.setEnabled(False)
-            
+        self.loopButton.setEnabled(False)
+
     def saveAll(self):
         for w in self.widgets:
             w.saveAll(self.dumpcells)
-            
+
     def saveEach(self):
         for w in self.widgets:
             w.saveEach(self.dumpcells, self.frameNo)
-        
+
     def saveEventAction(self, checked):
         self.saveEvent()
-          
+
     def saveEvent(self, folder=None):
-        if folder == None:
+        if folder is None:
             folder = QtGui.QFileDialog.getExistingDirectory(self,
                                                         "Save images to...",
                                                         self.lastExportPath,
@@ -457,9 +460,9 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
             self.dumpcells = str(folder)
             self.saveEach()
             self.lastExportPath = str(folder)
-            
+
     def saveAllEvent(self, folder=None):
-        if folder == None:
+        if folder is None:
             folder = QtGui.QFileDialog.getExistingDirectory(self,
                                                         "Save images to...",
                                                         self.lastExportPath,
@@ -467,11 +470,10 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         if folder:
             self.dumpcells = str(folder)
             self.saveAll()
-            self.lastExportPath
-    
+
     def saveAndExport(self, clicked=True):
         self.saveAll()
-        
+
     def initCells(self, cellEvents):
         cellLayout = QtGui.QHBoxLayout()
         self.mainLayout.addLayout(cellLayout, self.numberOfCells * 2)
@@ -481,58 +483,57 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
             cellWidget = event.cellType(self.centralWidget())
             if event.cellType.__name__ == 'QVTKWidget':
                 vtkCells.append(cellWidget)
-            cellWidget.updateContents(event.inputPorts)
             cellWidget.show()
             cellWidget.setSizePolicy(QtGui.QSizePolicy.Expanding,
                                       QtGui.QSizePolicy.Expanding)
             cellWidget.setMinimumSize(300, 100)
             cellLayout.addWidget(cellWidget)
             self.cellWidgets.append(cellWidget)
+            cellWidget.updateContents(event.inputPorts)
         def getSelectedCellWidgets():
             return vtkCells
         for cellWidget in self.vtkCells:
             cellWidget.getSelectedCellWidgets = getSelectedCellWidgets
-      
+
     def initControls(self):
         if len(self.currentMashup.alias_list) == 0:
             return
-        
+
         #Constructing alias controls
         self.controlDocks = {}
         self.cellControls = {}
         self.toolbuttons = {}
-        
+
         row = 0
         for alias in self.currentMashup.alias_list:
             dock = QCustomDockWidget(alias.name, #"Control for '%s'" % aliasName,
                                       self)
             vtparam = self.controller.getVistrailParam(alias)
-            
+
             if alias.component.widget == 'slider':
                 aliasWidget = QAliasSliderWidget(alias, vtparam, dock)
-                # enables looping of 
+                # enables looping of
                 if alias.component.seq:
                     self.sequenceOption = aliasWidget
 
-
             elif alias.component.widget == 'numericstepper':
                 aliasWidget = QAliasNumericStepperWidget(alias, vtparam, dock)
             else:
                 aliasWidget = QDropDownWidget(alias, vtparam, dock)
-            
+
             aliasWidget.setSizePolicy(QtGui.QSizePolicy.Preferred,
                                           QtGui.QSizePolicy.Maximum)
             self.connect(aliasWidget,
                              QtCore.SIGNAL("contentsChanged"),
                              self.widget_changed)
-                
+
             dock.setWidget(aliasWidget)
             self.addDockWidget(QtCore.Qt.RightDockWidgetArea, dock)
             self.controlDocks[alias.name] = dock
             self.cellControls[alias.name] = aliasWidget.value
             row += 1
             self.aliasWidgets[alias.name] = aliasWidget
-        
+
         # Added a stretch space
         stretchDock = QCustomDockWidget('Stretch Space', self)
         stretch = QtGui.QWidget()
@@ -541,14 +542,14 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
         stretchDock.setWidget(stretch)
         self.addDockWidget(QtCore.Qt.RightDockWidgetArea, stretchDock)
         self.controlDocks["_stretch_"] = stretchDock
-            
+
     def widget_changed(self, info):
         if self.cb_auto_update.isChecked() and not self.is_executing:
             self.updateCells(info)
-        
-            
+
+
     def run(self, useDefaultValues=False):
-        
+
         # Building the list of parameter values
         params = []
         if useDefaultValues:
@@ -563,7 +564,7 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
                 else:
                     val =str(edit.text())
                 params.append((alias.component.vttype, alias.component.vtid,
-                              val))    
+                              val))
         results = self.controller.execute(params)[0]
         result = results[0]
         (objs, errors, executed) = (result.objects, result.errors,
@@ -573,16 +574,16 @@ class QMashupAppMainWindow(QtGui.QMainWindow):
             print errors
             return (False, errors)
         return (True, [])
-    
+
     def showBuilderWindow(self):
         from vistrails.gui.vistrails_window import _app
         _app.show()
-            
+
 class QCustomDockWidget(QtGui.QDockWidget):
     def __init__(self, title, parent=None):
         QtGui.QDockWidget.__init__(self, title, parent)
         self.setObjectName(title)
-        self.setFeatures(QtGui.QDockWidget.DockWidgetClosable | 
+        self.setFeatures(QtGui.QDockWidget.DockWidgetClosable |
                          QtGui.QDockWidget.DockWidgetMovable)
         self.emptyTitleBar = QtGui.QWidget()
         self.titleBarVisible = True
@@ -590,7 +591,7 @@ class QCustomDockWidget(QtGui.QDockWidget):
 
     def showTitleBar(self):
         self.titleBarVisible = True
-        self.setFeatures(QtGui.QDockWidget.DockWidgetClosable | 
+        self.setFeatures(QtGui.QDockWidget.DockWidgetClosable |
                          QtGui.QDockWidget.DockWidgetMovable)
         self.setMaximumHeight(524287)
         self.setTitleBarWidget(None)
@@ -611,14 +612,28 @@ class QCustomDockWidget(QtGui.QDockWidget):
 
 
 class TestMashupApp(TestVisTrailsGUI):
+    def setUp(self):
+        super(TestMashupApp, self).setUp()
+        try:
+            import vtk
+        except ImportError:
+            self.skipTest("VTK is not available")
+        from vistrails.tests.utils import enable_package
+        enable_package('org.vistrails.vistrails.vtk')
 
     def test_load_mashup(self):
+        import vistrails.api
         import vistrails.core.system
-        filename = (vistrails.core.system.vistrails_root_directory() + 
+        filename = (vistrails.core.system.vistrails_root_directory() +
                     '/tests/resources/spx_loop.vt')
         view = vistrails.api.open_vistrail_from_file(filename)
+        # Execute workflow to trigger upgrades
+        view.controller.execute_current_workflow()
+        view.controller.flush_delayed_actions()
         id = "d5026457-de6c-11e2-b074-3c07543dba07"
         mashup = view.get_mashup_from_mashuptrail_id(id, "loop")
+        self.assert_(mashup)
         view.open_mashup(mashup)
         mashup = view.get_mashup_from_mashuptrail_id(id, "no loop")
+        self.assert_(mashup)
         view.open_mashup(mashup)
