@@ -1,34 +1,35 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
@@ -37,34 +38,24 @@ exploration. This allows user-defined constants to be used as dimensions
 in parameter exploration, provided the user implements the appropriate
 API in the classes.
 """
+from __future__ import division
+
 from PyQt4 import QtCore, QtGui
 from vistrails.core.modules.basic_modules import Color
+from vistrails.core import debug
 from vistrails.core.modules.paramexplore import IntegerLinearInterpolator, \
    FloatLinearInterpolator, RGBColorInterpolator, HSVColorInterpolator
 
 from vistrails.gui.common_widgets import QStringEdit
 from vistrails.gui.modules.constant_configuration import ColorChooserButton
 from vistrails.gui.modules.python_source_configure import PythonEditor
+from vistrails.gui.modules.utils import get_param_explore_widget_list
 from vistrails.gui.theme import CurrentTheme
 from vistrails.gui.utils import show_warning
 from vistrails.core.utils import all, unimplemented
 
 ##############################################################################
 
-def get_param_explore_widget_list(module_klass):
-    widget_list = []
-    klass_list = module_klass.get_param_explore_widget_list()
-    for klass in klass_list:
-        if klass is None:
-            pass
-        elif isinstance(klass, tuple):
-            (path, klass_name) = klass
-            module = __import__(path, globals(), locals(), [klass_name])
-            widget_list.append(getattr(module, klass_name))
-        else:
-            widget_list.append(klass)
-    return widget_list
-
 class QParameterEditor(QtGui.QWidget):
     """
     QParameterEditor specifies the method used for interpolating
@@ -88,7 +79,7 @@ class QParameterEditor(QtGui.QWidget):
         hLayout.setSpacing(0)
         self.setLayout(hLayout)
 
-        module = param_info.spec.descriptor.module
+        descriptor = param_info.spec.descriptor
 
         self.stackedEditors = QtGui.QStackedWidget()
         self.stackedEditors.setSizePolicy(QtGui.QSizePolicy.Expanding,
@@ -99,7 +90,7 @@ class QParameterEditor(QtGui.QWidget):
             self._exploration_widgets.append(wd)
             self.stackedEditors.addWidget(wd)
 
-        for widget_class in get_param_explore_widget_list(module):
+        for widget_class in get_param_explore_widget_list(descriptor):
             new_widget = widget_class(param_info, size)
             add_exploration_widget(new_widget)
 
@@ -677,7 +668,6 @@ class QUserFunctionEditor(QtGui.QFrame):
         param_info = self._param_info
         module = param_info.spec.descriptor.module
         def get():
-            import code
             values = []
             d = {}
             try:
@@ -687,11 +677,12 @@ class QUserFunctionEditor(QtGui.QFrame):
             def evaluate(i):
                 try:
                     v = d['value'](i)
-                    if v == None:
+                    if v is None:
                         return module.default_value
                     return v
                 except Exception, e:
-                    return str(e)
+                    debug.unexpected_exception(e)
+                    return debug.format_exception(e)
             return [evaluate(i) for i in xrange(self.size)]
         result = get()
         
