@@ -1,34 +1,35 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
@@ -36,6 +37,8 @@
 Dialog for web repository options
 Includes login and upload tabs
 """
+from __future__ import division
+
 from PyQt4 import QtGui, QtCore
 from vistrails.core.configuration import get_vistrails_configuration, get_vistrails_persistent_configuration
 from vistrails.core.repository.poster.encode import multipart_encode
@@ -228,7 +231,7 @@ class QRepositoryPushWidget(QtGui.QWidget):
                             ("Error checking user projects (server side issues)")
                     debug.critical("Error checking user projects (server side issues)")
                 else:
-                    debug.critical(str(e))
+                    debug.critical("Exception checking user projects", e)
 
                 self._push_button.setEnabled(False)
                 self.update_push_information()
@@ -277,7 +280,7 @@ class QRepositoryPushWidget(QtGui.QWidget):
                             ("Error when checking dependencies (server side issues)")
                     debug.critical("Error when checking dependencies (server side issues)")
                 else:
-                    debug.critical(str(e))
+                    debug.critical("Exception checking dependencies", e)
 
                 self._push_button.setEnabled(False)
                 self.update_push_information()
@@ -305,7 +308,7 @@ class QRepositoryPushWidget(QtGui.QWidget):
                     if module.name[-6:] == 'Reader' or \
                        module.name in self.local_data_modules:
                         for edge in pipeline.graph.edges_to(module.id):
-                            if pipeline.modules[edge[0]].name in ['HTTPFile',
+                            if pipeline.modules[edge[0]].name in ['DownloadFile',
                                                                   'RepoSync']:
                                 on_repo = True
 
@@ -424,13 +427,10 @@ class QRepositoryPushWidget(QtGui.QWidget):
             (fd, filename) = tempfile.mkstemp(suffix='.vt', prefix='vt_tmp')
             os.close(fd)
 
-            # writing tmp vt and switching back to orginal vt
+            # writing tmp vt and switching back to original vt
             locator = ZIPFileLocator(filename)
             controller = vistrails.api.get_current_controller()
-            tmp_controller = VistrailController(controller.vistrail.do_copy(), 
-                                                locator)
-            tmp_controller.changed = True
-            tmp_controller.write_vistrail(locator)
+            controller.write_vistrail(locator, export=True)
 
             # check if this vt is from the repository
             if controller.vistrail.get_annotation('repository_vt_id'):
@@ -515,10 +515,11 @@ class QRepositoryPushWidget(QtGui.QWidget):
                             load_vistrail(updated_locator)
 
                     # FIXME need to figure out what to do with this !!!
+                    current_version = controller.current_version
                     controller.set_vistrail(up_vistrail,
                                             controller.vistrail.locator,
                                             abstractions, thumbnails, mashups)
-
+                    controller.change_selected_version(current_version)
                     # update version tree drawing
                     controller.recompute_terse_graph()
                     controller.invalidate_version_tree()
@@ -530,7 +531,7 @@ class QRepositoryPushWidget(QtGui.QWidget):
                             "Update to repository was successful"
 
         except Exception, e:
-            debug.critical("An error occurred", str(e))
+            debug.critical("An error occurred", e)
             self._repository_status['details'] = "An error occurred"
         self.update_push_information()
 
@@ -560,8 +561,8 @@ class QRepositoryLoginPopup(QtGui.QDialog):
         l2 = QtGui.QLabel("Username:", self)
         grid_layout.addWidget(l2, 1, 0)
 
-        if self.config.check('webRepositoryLogin'):
-            self.loginUser = QtGui.QLineEdit(self.config.webRepositoryLogin, self)
+        if self.config.check('webRepositoryUser'):
+            self.loginUser = QtGui.QLineEdit(self.config.webRepositoryUser, self)
         else:
             self.loginUser = QtGui.QLineEdit("", self)
 
@@ -581,7 +582,7 @@ class QRepositoryLoginPopup(QtGui.QDialog):
         grid_layout.addWidget(self.loginPassword, 2, 1)
 
         self.saveLogin = QtGui.QCheckBox("Save username", self)
-        if self.config.check('webRepositoryLogin'):
+        if self.config.check('webRepositoryUser'):
             self.saveLogin.setChecked(True)
         grid_layout.addWidget(self.saveLogin, 3, 0)
 
@@ -670,18 +671,18 @@ class QRepositoryLoginPopup(QtGui.QDialog):
 
             # add association between VisTrails user and web repository user
             if self.saveLogin.checkState():
-                if not (self.config.check('webRepositoryLogin') and self.config.webRepositoryLogin == self.loginUser.text()):
-                    self.config.webRepositoryLogin = str(self.loginUser.text())
+                if not (self.config.check('webRepositoryUser') and self.config.webRepositoryUser == self.loginUser.text()):
+                    self.config.webRepositoryUser = str(self.loginUser.text())
                     pers_config = get_vistrails_persistent_configuration()
-                    pers_config.webRepositoryLogin = self.config.webRepositoryLogin
+                    pers_config.webRepositoryUser = self.config.webRepositoryUser
                     get_vistrails_application().save_configuration()
 
             # remove association between VisTrails user and web repository user
             else:
-                if self.config.check('webRepositoryLogin') and self.config.webRepositoryLogin:
-                    self.config.webRepositoryLogin = ""
+                if self.config.check('webRepositoryUser') and self.config.webRepositoryUser:
+                    self.config.webRepositoryUser = ""
                     pers_config = get_vistrails_persistent_configuration()
-                    pers_config.webRepositoryLogin = ""
+                    pers_config.webRepositoryUser = ""
                     get_vistrails_application().save_configuration()
             self.close_dialog(0)
 
