@@ -1,34 +1,35 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
@@ -44,21 +45,25 @@ QGraphicsVersionItem
 QVersionTreeScene
 QVersionTreeView
 """
+from __future__ import division
+
 from PyQt4 import QtCore, QtGui
+from vistrails.core.configuration import get_vistrails_configuration
+from vistrails.core import debug
 from vistrails.core.system import systemType
 from vistrails.core.thumbnails import ThumbnailCache
+from vistrails.core.vistrail.controller import custom_color_key, \
+    parse_custom_color
+from vistrails.core.vistrail.vistrail import Vistrail
 from vistrails.gui.base_view import BaseView
 from vistrails.gui.graphics_view import (QInteractiveGraphicsScene,
                                QInteractiveGraphicsView,
-                               QGraphicsItemInterface,
-                               QGraphicsRubberBandItem)
+                               QGraphicsItemInterface)
 from vistrails.gui.qt import qt_super
 from vistrails.gui.theme import CurrentTheme
 from vistrails.gui.version_prop import QVersionPropOverlay
-from vistrails.gui.vis_diff import QVisualDiff
 from vistrails.gui.collection.workspace import QParamExplorationEntityItem
 import vistrails.gui.utils
-import math
 
 
 ################################################################################
@@ -130,8 +135,8 @@ class QGraphicsLinkItem(QGraphicsItemInterface, QtGui.QGraphicsPolygonItem):
 
         # check if it is the same geometry 
         # improves performance on updates
-        if self.c1 != None and self.c2 != None and \
-                self.expand != None and self.collapse !=None:
+        if self.c1 is not None and self.c2 is not None and \
+                self.expand is not None and self.collapse !=None:
             isTheSame = self.c1 == c1 and \
                 self.c2 == c2 and \
                 self.expand == expand and \
@@ -477,7 +482,8 @@ class QGraphicsVersionItem(QGraphicsItemInterface, QtGui.QGraphicsEllipseItem):
                 self._versionPen = self._versionPenNormal
             self.updatePainterState()
 
-    def update_color(self, isThisUs, new_rank, new_max_rank, new_ghosted):
+    def update_color(self, isThisUs, new_rank, new_max_rank, new_ghosted,
+                     new_customcolor):
         """ update_color(isThisUs: bool,
                          new_rank, new_max_rank: int) -> None
 
@@ -487,22 +493,35 @@ class QGraphicsVersionItem(QGraphicsItemInterface, QtGui.QGraphicsEllipseItem):
         NOTE: if username changes during execution, this might break.
         """
         if (new_rank == self.rank and new_max_rank == self.max_rank and
-            new_ghosted == self.ghosted):
+            new_ghosted == self.ghosted and
+            new_customcolor == self.custom_color):
             # nothing changed
             return
         self.setGhosted(new_ghosted)
+        self.custom_color = new_customcolor
         self.rank = new_rank
         self.max_rank = new_max_rank
         if not self.ghosted:
-            if isThisUs:
-                brush = CurrentTheme.VERSION_USER_BRUSH
+            if self.custom_color is not None:
+                configuration = get_vistrails_configuration()
+                sat_from_rank = not configuration.check(
+                        'fixedCustomVersionColorSaturation')
+                brush = QtGui.QBrush(QtGui.QColor.fromRgb(*self.custom_color))
             else:
-                brush = CurrentTheme.VERSION_OTHER_BRUSH
-            sat = float(new_rank+1) / new_max_rank
-            (h, s, v, a) = brush.color().getHsvF()
-            newHsv = (h, s*sat, v+(1.0-v)*(1-sat), a)
-            self.versionBrush = QtGui.QBrush(QtGui.QColor.fromHsvF(*newHsv))
-                
+                if isThisUs:
+                    brush = CurrentTheme.VERSION_USER_BRUSH
+                else:
+                    brush = CurrentTheme.VERSION_OTHER_BRUSH
+                sat_from_rank = True
+            if sat_from_rank:
+                sat = float(new_rank+1) / new_max_rank
+                (h, s, v, a) = brush.color().getHsvF()
+                newHsv = (h, s*sat, v+(1.0-v)*(1-sat), a)
+                brush = QtGui.QBrush(brush)
+                brush.setColor(QtGui.QColor.fromHsvF(*newHsv))
+            self.versionBrush = brush
+        self.update()
+
     def setSaturation(self, isThisUser, sat):
         """ setSaturation(isThisUser: bool, sat: float) -> None        
         Set the color of this version depending on whose is the user
@@ -573,7 +592,7 @@ class QGraphicsVersionItem(QGraphicsItemInterface, QtGui.QGraphicsEllipseItem):
             textToDraw=self.label
         else:
             textToDraw=self.descriptionLabel
-        
+
         if (ThumbnailCache.getInstance().conf.mouseHover and
             action and action.thumbnail is not None):
             fname = ThumbnailCache.getInstance().get_abs_name_entry(action.thumbnail)
@@ -768,19 +787,17 @@ class QVersionTreeScene(QInteractiveGraphicsScene):
         self.edges = {}     # (sourceVersion, targetVersion) -> edge gui object
         self.controller = None
         self.fullGraph = None
-        self.timer = QtCore.QBasicTimer()
-        self.animation_step = 1
-        self.num_animation_steps = 1
         self.emit_selection = True
         self.select_by_click = True
         self.connect(self, QtCore.SIGNAL("selectionChanged()"),
                      self.selectionChanged)
-   
+
     def addVersion(self, node, action, tag, description):
-        """ addModule(node, action: DBAction, tag: str, description: str) 
+        """ addModule(node, action: DBAction, tag: str, description: str,
+                custom_color: (int, int, int))
                 -> None
         Add a module to the scene.
-        
+
         """
         versionShape = QGraphicsVersionItem(None)
         versionShape.setupVersion(node, action, tag, description)
@@ -860,9 +877,24 @@ class QVersionTreeScene(QInteractiveGraphicsScene):
             else:
                 max_rank = otherMaxRank
 #             max_rank = ourMaxRank if nodeUser==currentUser else otherMaxRank
+            configuration = get_vistrails_configuration()
+            if configuration.check('enableCustomVersionColors'):
+                custom_color = controller.vistrail.get_action_annotation(
+                    nodeId,
+                    custom_color_key)
+                if custom_color is not None:
+                    try:
+                        custom_color = parse_custom_color(custom_color.value)
+                    except ValueError, e:
+                        debug.warning("Version %r has invalid color annotation "
+                                      "(%s)" % (nodeId, e))
+                        custom_color = None
+            else:
+                custom_color = None
+            ####
             item.update_color(nodeUser==currentUser,
                               ranks[nodeId],
-                              max_rank, ghosted)
+                              max_rank, ghosted, custom_color)
         for (version_from, version_to), link in self.edges.iteritems():
             if self.versions[version_from].ghosted and \
                     self.versions[version_to].ghosted:
@@ -900,21 +932,21 @@ class QVersionTreeScene(QInteractiveGraphicsScene):
 
         # update link items
         dst = controller._current_terse_graph.edges_from(new_version)
-        for eto, _ in dst:
+        for eto, (expand, collapse) in dst:
             edge = self.edges[(old_version, eto)]
             edge.setupLink(self.versions[new_version],
                            self.versions[eto],
-                           self.fullGraph.parent(eto) != new_version,
+                           expand,
                            False) # We shouldn't ever need a collapse here
             self.edges[(new_version, eto)] = edge
             del self.edges[(old_version, eto)]
 
         src = controller._current_terse_graph.edges_to(new_version)
-        for efrom, _ in src:
+        for efrom, (expand, collapse) in src:
             edge = self.edges[(efrom, old_version)]
             edge.setupLink(self.versions[efrom],
                            self.versions[new_version],
-                           self.fullGraph.parent(new_version) != efrom,
+                           expand,
                            False) # We shouldn't ever need a collapse here
             self.edges[(efrom, new_version)] = edge
             del self.edges[(efrom, old_version)]
@@ -924,23 +956,11 @@ class QVersionTreeScene(QInteractiveGraphicsScene):
         Construct the scene to view a version tree
         
         """
-        import time
-        t = time.clock()
-
-        tClearRefine = time.clock()
-
-        # Clean the previous scene
-        # self.clear()
-
         self.select_by_click = False        
         self.controller = controller
 
         # perform graph layout
-        (tree, self.fullGraph, layout) = \
-            controller.refine_graph(float(self.animation_step)/
-                                    float(self.num_animation_steps))
-
-        tClearRefine = time.clock() - tClearRefine
+        (tree, self.fullGraph, layout) = controller.refine_graph()
 
         # compute nodes that should be removed
         # O(n  * (hashmap query key time)) on 
@@ -960,18 +980,16 @@ class QVersionTreeScene(QInteractiveGraphicsScene):
 
         # loop on the nodes of the tree
         vistrail = controller.vistrail
-        tm = vistrail.get_tagMap()
         am = vistrail.actionMap
         last_n = vistrail.getLastActions(controller.num_versions_always_shown)
 
         self.emit_selection = False
         for node in layout.nodes.itervalues():
-
             # version id
             v = node.id
 
             # version tag
-            tag = tm.get(v, None)
+            tag = tree.vertices.get(v, None)
             action = am.get(v, None)
             description = vistrail.get_description(v)
 
@@ -982,12 +1000,11 @@ class QVersionTreeScene(QInteractiveGraphicsScene):
             else:
                 self.addVersion(node, action, tag, description)
             if select_node:
-                self.versions[v].setSelected(v == controller.current_version)
+                self.versions[v].setSelected(v == controller.current_base_version)
 
         self.emit_selection = True
         self.selectionChanged()
 
-
         # remove gui edges from scene
         for (v1, v2) in removeEdgeSet:
             self.removeLink(v1,v2)
@@ -996,73 +1013,28 @@ class QVersionTreeScene(QInteractiveGraphicsScene):
         for v in removeNodeSet:
             self.removeVersion(v)
 
-        tCreate = time.clock()
-
         # adjust the colors
         self.adjust_version_colors(controller)
 
         # Add or update links
-        for source in tree.vertices.iterkeys():
+        for source, source_tag in tree.vertices.iteritems():
             eFrom = tree.edges_from(source)
-            for (target, aux) in eFrom:
+            for target, (expand, collapse) in eFrom:
                 guiSource = self.versions[source]
                 guiTarget = self.versions[target]
-                sourceChildren = [to for (to, _) in 
-                                  self.fullGraph.adjacency_list[source]
-                                  if (to in am) and not vistrail.is_pruned(to)]
-                targetChildren = [to for (to, _) in
-                                  self.fullGraph.adjacency_list[target]
-                                  if (to in am) and not vistrail.is_pruned(to)]
-                expand = self.fullGraph.parent(target)!=source
-                collapse = (self.fullGraph.parent(target)==source and # No in betweens
-                            len(targetChildren) == 1 and # target is not a leaf or branch
-                            target != controller.current_version and # target is not selected
-                            target not in tm and # target has no tag
-                            target not in last_n and # not one of the last n modules
-                            (source in tm or # source has a tag
-                             source == 0 or # source is root node
-                             len(sourceChildren) > 1 or # source is branching node 
-                             source == controller.current_version)) # source is selected
                 if self.edges.has_key((source,target)):
                     linkShape = self.edges[(source,target)]
                     linkShape.setupLink(guiSource, guiTarget,
                                         expand, collapse)
                 else:
-                    #print "add link %d %d" % (source, target)
                     self.addLink(guiSource, guiTarget, 
                                  expand, collapse)
 
-        tCreate = time.clock() - tCreate
-
         # Update bounding rects and fit to all view
-        tUpdate = time.clock()
-        if not self.controller.animate_layout:
-            self.updateSceneBoundingRect()
-        elif not self.timer.isActive():
-            self.timer.start(0, self)
-        tUpdate = time.clock() - tUpdate
+        self.updateSceneBoundingRect()
 
         self.select_by_click = True
 
-        t = time.clock() - t
-        # print "time in msec to setupScene total: %f  refine %f  layout %f  create %f" % (t, tClearRefine, tCreate)
-
-    def timerEvent(self, event):
-        """ timerEvent(event: QTimerEvent) -> None
-        
-        Start up a timer for animating tree drawing events
-        """
-        if event.timerId() == self.timer.timerId():
-            self.animation_step += 1
-            if self.animation_step >= self.num_animation_steps:
-                self.animation_step = 1
-                self.timer.stop()
-                self.controller.animate_layout = False
-            self.setupScene(self.controller)
-            self.update()
-        else:
-            qt_super(QVersionTreeScene, self).timerEvent(event)
-
     def keyPressEvent(self, event):
         """ keyPressEvent(event: QKeyEvent) -> None
         Capture 'Del', 'Backspace' for pruning versions when not editing a tag
@@ -1183,7 +1155,7 @@ class QVersionTreeView(QInteractiveGraphicsView, BaseView):
         return pipeline is not None and len(pipeline.modules) > 0
     
     def execute(self):
-        res = self.controller.execute_current_workflow()
+        res = self.controller.execute_user_workflow()
         from vistrails.gui.vistrails_window import _app
         if len(res[0][0].errors) > 0:
             _app.qactions['pipeline'].trigger()
@@ -1214,7 +1186,7 @@ class QVersionTreeView(QInteractiveGraphicsView, BaseView):
         items = self.scene().items(br)
         if len(items)==0 or items==[self.selectionBox]:
             for item in self.scene().selectedItems():
-                if isinstance(item, vistrails.gui.version_view.QGraphicsVersionItem):
+                if isinstance(item, QGraphicsVersionItem):
                     item.text.clearFocus()
         qt_super(QVersionTreeView, self).selectModules()
                 
@@ -1225,7 +1197,7 @@ class QVersionTreeView(QInteractiveGraphicsView, BaseView):
     def set_controller(self, controller):
         oldController = self.controller
         if oldController != controller:
-            if oldController != None:
+            if oldController is not None:
                 self.disconnect(oldController,
                                 QtCore.SIGNAL('vistrailChanged()'),
                                 self.vistrailChanged)
@@ -1296,4 +1268,3 @@ class QVersionTreeView(QInteractiveGraphicsView, BaseView):
 
     def select_current_version(self):
         self.scene().setupScene(self.controller)
-################################################################################
