@@ -1,41 +1,45 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
 """ This modules builds a widget to interact with vistrail diff
 operation """
+from __future__ import division
+
 from PyQt4 import QtCore, QtGui
 from vistrails.core.system import get_vistrails_basic_pkg_id
+from vistrails.core.utils import VistrailsInternalError
 from vistrails.core.utils.color import ColorByName
 from vistrails.core.vistrail.abstraction import Abstraction
 from vistrails.core.vistrail.pipeline import Pipeline
@@ -291,7 +295,8 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
         g_layout.addWidget(self.legend)
         legend_group.setLayout(g_layout)
         layout.addWidget(legend_group)
-
+        layout.setStretch(0,0)
+        layout.addStretch(1)
         self.params = QParamTable()
         params_group = QtGui.QGroupBox("Parameter Changes")
         g_layout = QtGui.QVBoxLayout()
@@ -300,6 +305,28 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
         g_layout.addWidget(self.params)
         params_group.setLayout(g_layout)
         layout.addWidget(params_group)
+        layout.setStretch(2,1000)
+
+        self.cparams = QParamTable()
+        params_group = QtGui.QGroupBox("Control Parameter Changes")
+        g_layout = QtGui.QVBoxLayout()
+        g_layout.setMargin(0)
+        g_layout.setSpacing(0)
+        g_layout.addWidget(self.cparams)
+        params_group.setLayout(g_layout)
+        layout.addWidget(params_group)
+        layout.setStretch(3,1000)
+        
+        self.annotations = QParamTable()
+        params_group = QtGui.QGroupBox("Annotation Changes")
+        g_layout = QtGui.QVBoxLayout()
+        g_layout.setMargin(0)
+        g_layout.setSpacing(0)
+        g_layout.addWidget(self.annotations)
+        params_group.setLayout(g_layout)
+        layout.addWidget(params_group)
+        layout.setStretch(3,1000)
+        
         self.setLayout(layout)
         self.addButtonsToToolbar()
 
@@ -332,7 +359,7 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
         result = str(result)
         try:
             self.controller.add_analogy(result, version_a, version_b)
-        except:
+        except VistrailsInternalError:
             debug.critical("Analogy name already exists")
 
     def set_diff(self):
@@ -340,8 +367,6 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
             return
         ((vistrail_a, version_a), (vistrail_b, version_b)) = \
             self.controller.current_diff_versions
-        (p1, p2, v1Andv2, heuristicMatch, v1Only, v2Only, paramChanged) = \
-            self.controller.current_diff
 
         # Set up the version name correctly
         v1_name = vistrail_a.getVersionName(version_a)
@@ -366,10 +391,14 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
         
         self.legend.set_names(v1_name, v2_name)
         self.params.set_names(v1_name, v2_name)
+        self.cparams.set_names(v1_name, v2_name)
+        self.annotations.set_names(v1_name, v2_name)
+        self.update_module()
 
     def set_controller(self, controller=None):
         self.controller = controller
-        self.set_diff()
+        if self.controller is not None:
+            self.set_diff()
         
     def update_module(self, module=None):
         """ moduleSelected(id: int, selectedItems: [QGraphicsItem]) -> None
@@ -379,11 +408,16 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
         """
         if module is None or not hasattr(self.controller, 'current_diff'):
             self.params.model().clearList()
+            self.params.parent().setVisible(False)
+            self.cparams.model().clearList()
+            self.cparams.parent().setVisible(False)
+            self.annotations.model().clearList()
+            self.annotations.parent().setVisible(False)
             return
         
-        # Interprete the diff result and setup item models
-        (p1, p2, v1Andv2, heuristicMatch, v1Only, v2Only, paramChanged) = \
-            self.controller.current_diff
+        # Interpret the diff result and setup item models
+        (p1, p2, v1Andv2, heuristicMatch, v1Only, v2Only, paramChanged,
+         cparamChanged, annotChanged) = self.controller.current_diff
 
         # # Set the window title
         # if id>self.maxId1:
@@ -395,15 +429,18 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
 
         # FIXME set the module name/package info?
             
-        # Clear the old inspector
-        param_model = self.params.model()
-        # annotations = self.inspector.annotationsTab.model()
-        param_model.clearList()
-        # annotations.clearList()
+        to_text = lambda x:'%s(%s)' % (x[0], ','.join(v[1] for v in x[1]))
+        self.setTable(module, paramChanged, self.params, to_text)
+        to_text = lambda x:'%s(%s)' % (x[0], x[1])
+        self.setTable(module, cparamChanged, self.cparams, to_text)
+        self.setTable(module, annotChanged, self.annotations, to_text)
 
+    def setTable(self, module, changed, table, to_text):
         # Find the parameter changed module
+        model = table.model()
+        model.clearList()
         matching = None
-        for ((m1id, m2id), paramMatching) in paramChanged:
+        for ((m1id, m2id), paramMatching) in changed:
             if m1id == module.id:
                 #print "found match"
                 matching = paramMatching
@@ -412,26 +449,24 @@ class QDiffProperties(QtGui.QWidget, QVistrailsPaletteInterface):
         #print "matching:", matching
         # If the module has no parameter changed, just display nothing
         if not matching:          
+            table.parent().setVisible(False)
             return
         
+        table.parent().setVisible(True)
+
         # Else just layout the diff on a table
-        param_model.insertRows(0,len(matching))
+        model.insertRows(0,len(matching))
         currentRow = 0
         for (f1, f2) in matching:
             if f1[0]!=None:
-                param_model.setData(
-                    param_model.index(currentRow, 0),
-                    '%s(%s)' % (f1[0], ','.join(v[1] for v in f1[1])))
+                model.setData(model.index(currentRow, 0), to_text(f1))
             if f2[0]!=None:
-                param_model.setData(
-                    param_model.index(currentRow, 1),
-                    '%s(%s)' % (f2[0], ','.join(v[1] for v in f2[1])))
+                model.setData(model.index(currentRow, 1), to_text(f2))
             if f1==f2:
-                param_model.disableRow(currentRow)
+                model.disableRow(currentRow)
             currentRow += 1
 
-        self.params.resizeRowsToContents()
-        # self.inspector.annotationsTab.resizeRowsToContents()
+        table.resizeRowsToContents()
 
 class QDiffView(QPipelineView):
     def __init__(self, parent=None):
@@ -521,8 +556,8 @@ class QDiffView(QPipelineView):
         self.set_diff_version_names()
         self.diff = vistrails.core.db.io.get_workflow_diff(*self.diff_versions)
             # self.controller.vistrail.get_pipeline_diff(version_a, version_b)
-        (p1, p2, v1Andv2, heuristicMatch, v1Only, v2Only, paramChanged) = \
-            self.diff
+        (p1, p2, v1Andv2, heuristicMatch, v1Only, v2Only, paramChanged,
+         cparamChanged, annotChanged) = self.diff
         # print "  $$$ v1Andv2:", v1Andv2
         # print "  $$$ heuristicMatch:", heuristicMatch
         # print "  $$$ v1Only", v1Only
@@ -635,10 +670,11 @@ class QDiffView(QPipelineView):
             p_both.add_module(copy.copy(p1.modules[m1id]))
 
         # Then add parameter changed version
-        for ((m1id, m2id), matching) in paramChanged:
+        inChanged = set(m for (m, matching)
+                        in chain(paramChanged, cparamChanged, annotChanged))
+        for (m1id, m2id) in inChanged:
             m1 = p1.modules[m1id]
             m2 = p2.modules[m2id]
-            
             sum1_x += p1.modules[m1id].location.x
             sum1_y += p1.modules[m1id].location.y
             sum2_x += p2.modules[m2id].location.x
@@ -721,7 +757,7 @@ class QDiffView(QPipelineView):
         for (m1id, m2id) in heuristicMatch:
             v1Tov2[m1id] = m2id
             v2Tov1[m2id] = m1id
-        for ((m1id, m2id), matching) in paramChanged:
+        for (m1id, m2id) in inChanged:
             v1Tov2[m1id] = m2id
             v2Tov1[m2id] = m1id
 
@@ -900,7 +936,7 @@ class QVisualDiff(QtGui.QMainWindow):
         result = str(result)
         try:
             self.controller.add_analogy(result, self.v1, self.v2)
-        except:
+        except VistrailsInternalError:
             debug.critical("Analogy name already exists")
         
     def createToolWindows(self, v1Name, v2Name):
