@@ -1,38 +1,41 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
 
+from __future__ import division
+
 if __name__ == '__main__':
     import sip
     sip.setapi('QString', 2)
@@ -97,7 +100,7 @@ class Command(object):
                 self.process = subprocess.Popen(self.command, **kwargs)
                 self.output, self.error = self.process.communicate()
                 self.status = self.process.returncode
-            except:
+            except Exception:
                 import traceback
                 self.error = traceback.format_exc()
                 self.status = -1
@@ -216,6 +219,12 @@ class QCLToolsWizard(QtGui.QWidget):
         self.stdAsFiles.setCheckable(True)
         self.toolBar.addAction(self.stdAsFiles)
 
+        self.failWithCmd = QtGui.QAction('fail execution if return != 0', self)
+        self.failWithCmd.setToolTip('If selected, VisTrails will check the exitcode, and abort the execution if not 0')
+        self.failWithCmd.setCheckable(True)
+        self.failWithCmd.setChecked(True)
+        self.toolBar.addAction(self.failWithCmd)
+
         self.toolBar.addSeparator()
 
         self.previewPorts = QtGui.QAction('preview', self)
@@ -360,6 +369,7 @@ class QCLToolsWizard(QtGui.QWidget):
         self.argList = QtGui.QListWidget()
         self.layout().addWidget(self.argList)
         self.stdAsFiles.setChecked(False)
+        self.failWithCmd.setChecked(True)
         self.setTitle()
         self.generate_preview()
     
@@ -403,6 +413,8 @@ class QCLToolsWizard(QtGui.QWidget):
                                 'env_port' in conf['options'])
         self.stdAsFiles.setChecked('options' in conf and
                                    'std_using_files' in conf['options'])
+        self.failWithCmd.setChecked('options' in conf and
+                                    'fail_with_cmd' in conf['options'])
         self.envOption = conf['options']['env'] \
                  if ('options' in conf and 'env' in conf['options']) else None
         self.conf = conf
@@ -430,6 +442,8 @@ class QCLToolsWizard(QtGui.QWidget):
         options = {}
         if self.stdAsFiles.isChecked():
             options['std_using_files'] = ''
+        if self.failWithCmd.isChecked():
+            options['fail_with_cmd'] = ''
         if self.envPort.isChecked():
             options['env_port'] = ''
         if self.envOption:
@@ -683,7 +697,7 @@ class QCLToolsWizard(QtGui.QWidget):
             for a, b in encode_list:
                 text = text.replace(a, b)
             return text
-        except:
+        except Exception:
             return None
     
     def generateFromManPage(self):
@@ -788,7 +802,8 @@ class QCLToolsWizard(QtGui.QWidget):
 class QArgWidget(QtGui.QWidget):
     """ Widget for configuring an argument """
     KLASSES = {
-            'input': ['flag', 'file', 'string', 'integer', 'float', 'list'],
+            'input': ['flag', 'file', 'path', 'directory',
+                      'string', 'integer', 'float', 'list'],
             'output': ['file', 'string'],
             'inputoutput': ['file'],
             'stdin': ['file', 'string'],
@@ -801,6 +816,8 @@ class QArgWidget(QtGui.QWidget):
             'integer': 'Integer',
             'float': 'Float',
             'file': 'File',
+            'path': 'Path',
+            'directory': 'Directory',
             'list': 'List',
         }
 
@@ -866,7 +883,7 @@ class QArgWidget(QtGui.QWidget):
             self.klassDict[n] = i
         self.klassList.setCurrentIndex(self.klassDict.get(self.klass, 0))
         #label = QtGui.QLabel('Class:')
-        tt = 'Port Type. Can be String, Integer, Float, File or Boolean Flag. List means an input list of one of the other types. Only File and String should be used for output ports.'
+        tt = 'Port Type. Can be String, Integer, Float, File/Directory/Path or Boolean Flag. List means an input list of one of the other types. Only File and String should be used for output ports.'
         self.klassList.setToolTip(tt)
         #label.setToolTip(tt)
         #layout1.addWidget(label)
@@ -911,7 +928,7 @@ class QArgWidget(QtGui.QWidget):
         layout2.addWidget(self.required)
         
         # subtype
-        self.subList = ['String', 'Integer', 'Float', 'File']
+        self.subList = ['String', 'Integer', 'Float', 'File', 'Directory', 'Path']
         self.subDict = dict(zip(self.subList, xrange(len(self.subList))))
         self.subDict.update(dict(zip([s.lower() for s in self.subList], xrange(len(self.subList)))))
         self.subtype = QtGui.QComboBox()
@@ -998,25 +1015,27 @@ class QArgWidget(QtGui.QWidget):
             self.suffix.setText(self.options.get('suffix', ''))
         self.typeChanged()
         self.klassChanged()
-            
+
     def toList(self):
         self.getValues()
         if self.argtype not in self.stdTypes:
-            return [self.argtype, self.name, self.klass, self.options]
+            return [self.argtype, self.name, self.klass, dict(self.options)]
         else:
-            return [self.name, self.klass, self.options]
-            
+            return [self.name, self.klass, dict(self.options)]
+
     def fromList(self, arg):
         if self.argtype not in self.stdTypes:
-            self.argtype, self.name, self.klass, self.options = arg
+            self.argtype, self.name, klass, self.options = arg
         else:
-            self.name, self.klass, self.options = arg
+            self.name, klass, self.options = arg
+        self.klass = klass.lower()
         self.setValues()
 
     def klassChanged(self, index=None):
         if self.argtype in self.stdTypes:
             return
         klass = self.klassList.itemData(self.klassList.currentIndex())
+        type = self.typeList.itemData(self.typeList.currentIndex())
         self.listLabel.setVisible(klass == "list" and type == 'input')
         self.subtype.setVisible(klass == "list" and type == 'input')
 
@@ -1043,9 +1062,15 @@ class QArgWidget(QtGui.QWidget):
 
     def guess(self, name, count=0):
         """ add argument by guessing what the arg might be """
-        if '.' in name or '/' in name or '\\' in name: # guess file
-            self.fromList(['Input', 'file%s' % count, 'File',
-                           {'desc':'"%s" guessed to be an Input file' % name}])
+        if '.' in name or '/' in name or '\\' in name: # guess path
+            if os.path.isfile(name):
+                guessed, type_ = 'file', 'File'
+            elif os.path.isdir(name):
+                guessed, type_ = 'directory', 'Directory'
+            else:
+                guessed, type_ = 'path', 'Path'
+            self.fromList(['Input', '%s%d' % (guessed, count), type_,
+                           {'desc':'"%s" guessed to be an Input %s' % (name, guessed)}])
         elif name.startswith('-'): # guess flag
             self.fromList(['Input', 'flag%s' % name, 'Flag',
                            {'desc':'"%s" guessed to be a flag' % name,
