@@ -1,3 +1,41 @@
+###############################################################################
+##
+## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2011-2014, NYU-Poly.
+## Copyright (C) 2006-2011, University of Utah.
+## All rights reserved.
+## Contact: contact@vistrails.org
+##
+## This file is part of VisTrails.
+##
+## "Redistribution and use in source and binary forms, with or without
+## modification, are permitted provided that the following conditions are met:
+##
+##  - Redistributions of source code must retain the above copyright notice,
+##    this list of conditions and the following disclaimer.
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
+##    documentation and/or other materials provided with the distribution.
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
+##    this software without specific prior written permission.
+##
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
+##
+###############################################################################
+
+from __future__ import division
+
 import inspect
 import mixins
 from xml.etree import ElementTree as ET
@@ -58,12 +96,6 @@ class SpecList(object):
             elif elt.tag == "customCode":
                 custom_code = elt.text
         retval = SpecList(module_specs, custom_code)
-        # for spec in retval.module_specs:
-        #     print "==", spec.name, "=="
-        #     for ps in spec.port_specs:
-        #         print " ", ps.arg, ps.name
-        #         for alt_ps in ps.alternate_specs:
-        #             print "  !!!", ps.arg, ps.name, alt_ps.name
         return retval
 
 class ModuleSpec(object):
@@ -179,7 +211,7 @@ class PortSpec(object):
              "required": (False, False, True),
              "show_port": (False, False, True),
              "hide": (False, False, True),
-             "property_type": ""}
+             "property_type": "",}
 
     def __init__(self, arg, **kwargs):
         self.arg = arg
@@ -225,20 +257,6 @@ class PortSpec(object):
                     elt.set(attr, str(attr_val))
         return elt
 
-        # if self.name != "":
-        #     elt.set("name", self.name)
-        # if self.port_type is not None:
-        #     elt.set("port_type", self.port_type)
-        # else:
-        #     elt.set("port_type", "__unknown__")
-        # if self.port_type == "__property__":
-        #     elt.set("property_type", self.property_type)
-        # if self.required != False:
-        #     elt.set("required", str(self.required))
-        # if self.
-        # elt.set("hide", str(self.hide))
-        # elt.set("show_port", str(self.show_port))
-
     @classmethod
     def internal_from_xml(cls, elt, obj=None):
         arg = elt.get("arg", "")
@@ -249,8 +267,6 @@ class PortSpec(object):
 
         child_elts = {}
         for child in elt.getchildren():
-            # if child.tag not in obj.attrs:
-            #     raise RuntimeError('Cannot deal with tag "%s"' % child.tag)
             if child.tag not in child_elts:
                 child_elts[child.tag] = []
             child_elts[child.tag].append(child)
@@ -308,27 +324,6 @@ class PortSpec(object):
                         elt.tag)
 
 
-    # @staticmethod
-    # def from_xml(elt, obj=None):
-    #     arg = elt.get("arg", "")
-    #     if obj is None:
-    #         obj = PortSpec(arg)
-    #     else:
-    #         obj.arg = arg
-    #     obj.port_type = elt.get("port_type", "")
-    #     if obj.port_type == "__unknown__":
-    #         obj.port_type = None
-
-    #     if obj.port_type is not None and \
-    #             obj.port_type.lower() == "__property__":
-    #         obj.name = elt.get("name", obj.arg + "Properties")
-    #     else:
-    #         obj.name = elt.get("name", obj.arg)
-    #     obj.required = eval(elt.get("required", "False"))
-    #     obj.hide = eval(elt.get("hide", "False"))
-    #     obj.show_port = eval(elt.get("show_port", "False"))
-    #     return obj
-
     def is_property(self):
         return self.port_type == "__property__"
 
@@ -350,6 +345,7 @@ class InputPortSpec(PortSpec):
              "in_kwargs": (True, False, True),
              "in_args": (False, False, True),
              "constructor_arg": (False, False, True),
+             "not_setp": (False, False, True),
              "arg_pos": (-1, False, True),
              }
     attrs.update(PortSpec.attrs)
@@ -403,14 +399,7 @@ class InputPortSpec(PortSpec):
     def has_alternate_versions(self):
         return len(self.alternate_specs) > 0
 
-    # def is_property_input(self):
-    #     return self.get_port_type().lower() == "__property__"
-
 class AlternatePortSpec(InputPortSpec):
-    # attrs = ["name", "port_type", "docstring", "required", "hide", 
-    #          "entry_types", "values", "defaults", "translations", 
-    #          "property_type"]
-
     xml_name = "alternateSpec"
     def __init__(self, *args, **kwargs):
         if len(args) < 1:
@@ -432,11 +421,6 @@ class AlternatePortSpec(InputPortSpec):
             else:
                 self.name = base_name + "Scalar"
         self.arg = self._parent.arg
-            
-    # def to_xml(self, elt=None):
-    #     if elt is None:
-    #         elt = ET.Element("alternateSpec")
-    #     return PortSpec.to_xml(self, elt)
 
     def get_port_attr_dict(self):
         print "CALLING AlternatePortSpec.get_port_attr_dict", self.arg
