@@ -1,51 +1,50 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
+from __future__ import division
+
+from ast import literal_eval
 import os
 import sys
-import tempfile
 import urllib
 import rpy2.robjects as robjects
 
-from vistrails.core.modules.basic_modules import File, Constant, \
-    new_constant
+from vistrails.core.modules.basic_modules import PathObject, new_constant
+from vistrails.core.modules.vistrails_module import Module, ModuleError
+from .widgets import RSourceConfigurationWidget, RFigureConfigurationWidget
 
-from vistrails.core.modules.vistrails_module import Module, ModuleError, \
-    ModuleConnector, NotCacheable
-from vistrails.core.modules.basic_modules import new_constant
-import vistrails.core.modules.module_registry
-from widgets import RSourceConfigurationWidget, RFigureConfigurationWidget
 
 # FIXME when rpy2 is installed on the path, we won't need this
 old_sys_path = sys.path
@@ -96,7 +95,7 @@ def create_vector(v_list, desired_type=None):
     return robjects.RVector(v_list)
 
 def vector_conv(v, desired_type=None):
-    v_list = eval(v)
+    v_list = literal_eval(v)
     return create_vector(v_list, desired_type)
 
 RVector = new_constant('RVector', staticmethod(vector_conv),
@@ -159,14 +158,14 @@ def create_matrix(v_list):
     
 def matrix_conv(v):
     # should be a double list
-    v_list = eval(v)
+    v_list = literal_eval(v)
     create_matrix(v_list)
 
 def matrix_compute(self):
-    if self.hasInputFromPort('rvector'):
-        rvector = self.getInputFromPort('rvector')
-        nrows = self.getInputFromPort('nrows')
-        self.setResult('value', robjects.r.matrix(rvector, nrow=nrows))
+    if self.has_input('rvector'):
+        rvector = self.get_input('rvector')
+        nrows = self.get_input('nrows')
+        self.set_output('value', robjects.r.matrix(rvector, nrow=nrows))
     else:
         RArray.compute(self)
 
@@ -190,7 +189,7 @@ def create_list(v_dict):
     return robjects.r['list'](**data_dict)
 
 def list_conv(v):
-    v_dict = eval(v)
+    v_dict = literal_eval(v)
     return create_list(v_dict)
 
 RList = new_constant('RList', staticmethod(list_conv),
@@ -211,7 +210,7 @@ def create_data_frame(v_dict):
     return robjects.r['data.frame'](**data_dict)
 
 def data_frame_conv(v):
-    v_dict = eval(v)
+    v_dict = literal_eval(v)
     return create_data_frame(v_dict)
 
 RDataFrame = new_constant('RDataFrame', staticmethod(data_frame_conv),
@@ -224,63 +223,63 @@ class RVectorFromList(Module):
     _output_ports = [('rvector', '(Types|RVector)')]
 
     def compute(self):
-        ilist = self.getInputFromPort('list')
+        ilist = self.get_input('list')
         rvector = create_vector(ilist)
-        self.setResult('rvector', rvector)
+        self.set_output('rvector', rvector)
 
 class ListFromRVector(Module):
     _input_ports = [('rvector', '(Types|RVector)')]
     _output_ports = [('list', '(basic:List)')]
 
     def compute(self):
-        rvector = self.getInputFromPort('rvector')
+        rvector = self.get_input('rvector')
         olist = list(rvector)
-        self.setResult('list', olist)
+        self.set_output('list', olist)
 
 class RMatrixFromNestedList(Module):
     _input_ports = [('list', '(basic:List)')]
     _output_ports = [('rmatrix', '(Types|RMatrix)')]
 
     def compute(self):
-        ilist = self.getInputFromPort('list')
+        ilist = self.get_input('list')
         rmatrix = create_matrix(ilist)
-        self.setResult('rmatrix', rmatrix)
+        self.set_output('rmatrix', rmatrix)
 
 class NestedListFromRMatrix(Module):
     _input_ports = [('rmatrix', '(Types|RMatrix)')]
     _output_ports = [('list', '(basic:List)')]
     
     def compute(self):
-        rmatrix = self.getInputFromPort('rmatrix')
+        rmatrix = self.get_input('rmatrix')
         mlist = list(rmatrix)
         nrows = rmatrix.nrow
-        ncols = len(mlist) / nrows
+        ncols = len(mlist) // nrows
         olist = [] 
         for row in xrange(nrows):
             olist.append(mlist[row*ncols:(row+1)*ncols])
-        self.setResult('list', olist)
+        self.set_output('list', olist)
 
 class RDataFrameFromDict(Module):
     _input_ports = [('dict', '(basic:Dictionary)')]
     _output_ports = [('rdataframe', '(Types|RDataFrame)')]
     
     def compute(self):
-        idict = self.getInputFromPort('dict')
+        idict = self.get_input('dict')
         rdataframe = create_data_frame(idict)
-        self.setResult('rdataframe', rdataframe)
+        self.set_output('rdataframe', rdataframe)
 
 class DictFromRDataFrame(Module):
     _input_ports = [('rdataframe','(Types|RDataFrame)')]
     _output_ports = [('dict', '(basic:Dictionary)')]
 
     def compute(self):
-        rdataframe = self.getInputFromPort('rdataframe')
+        rdataframe = self.get_input('rdataframe')
         colnames = list(rdataframe.colnames())
         odict = {}
         for i in xrange(len(rdataframe)):
             # FIXME !!! just assume that each row can be converted to a list!!!
             odict[colnames[i]] = list(rdataframe[i])
-        self.setResult('dict', odict)
+        self.set_output('dict', odict)
 
 class RListFromDict(Module):
     # _input_ports = [('dict', '(basic:Dictionary)')]
@@ -288,9 +287,9 @@ class RListFromDict(Module):
     _output_ports = [('rlist', '(Types|RList)')]
     
     def compute(self):
-        idict = self.getInputFromPort('dict')
+        idict = self.get_input('dict')
         rlist = create_list(idict)
-        self.setResult('rlist', rlist)
+        self.set_output('rlist', rlist)
 
 class DictFromRList(Module):
     _input_ports = [('rlist', '(Types|RList)')]
@@ -298,14 +297,14 @@ class DictFromRList(Module):
     _output_ports = [('dict', '(basic:Module)')]
 
     def compute(self):
-        rlist = self.getInputFromPort('rlist')
+        rlist = self.get_input('rlist')
         colnames = list(rlist.names)
         odict = {}
         for i in xrange(len(rlist)):
             # FIXME !!! just assume that each row can be converted to a list!!!
             # FIXME this may need to be a list of lists
             odict[colnames[i]] = list(rlist[i])
-        self.setResult('dict', odict)
+        self.set_output('dict', odict)
 
 class RRead(Module):
     _input_ports = [('file', '(basic:File)'),
@@ -318,13 +317,13 @@ class RRead(Module):
     _output_ports = [('rdataframe', '(Types|RDataFrame)')]
 
     def do_read(self, read_cmd):
-        fname = self.getInputFromPort('file').name
+        fname = self.get_input('file').name
         options_dict = {}
         for port in RRead._input_ports:
-            if port[0] != 'file' and self.hasInputFromPort(port):
-                options_dict[port] = self.getInputFromPort(port)
+            if port[0] != 'file' and self.has_input(port):
+                options_dict[port] = self.get_input(port)
         rdataframe = robjects.r[read_cmd](fname, **options_dict)
-        self.setResult('rdataframe', rdataframe)
+        self.set_output('rdataframe', rdataframe)
 
 class RReadTable(RRead):
     def compute(self):
@@ -364,7 +363,7 @@ class RSource(Module):
         def cache_this():
             self.is_cacheable = lambda *args, **kwargs: True
         if use_input:
-            inputDict = dict([(k, self.getInputFromPort(k))
+            inputDict = dict([(k, self.get_input(k))
                               for k in self.inputPorts
                               if k not in excluded_inputs])
             for k,v in inputDict.iteritems():
@@ -373,7 +372,7 @@ class RSource(Module):
         if use_output:
             for k in self.outputPorts:
                 if k not in excluded_outputs and k in robjects.globalEnv:
-                    self.setResult(k, robjects.globalEnv[k])
+                    self.set_output(k, robjects.globalEnv[k])
 
     def run_file(self, fname, excluded_inputs=set(['source']), 
                  excluded_outputs=set()):
@@ -395,7 +394,7 @@ class RSource(Module):
         robjects.r('setwd("%s")' % dir)
 
     def compute(self):
-        code_str = urllib.unquote(str(self.forceGetInputFromPort('source', '')))
+        code_str = urllib.unquote(str(self.force_get_input('source', '')))
         self.run_code(code_str, use_input=True, use_output=True,
                       excluded_inputs=set(['source']))
 
@@ -409,10 +408,8 @@ class RFigure(RSource):
         self.run_code(code_str, use_input=True, 
                       excluded_inputs=excluded_inputs)
         robjects.r['dev.off']()
-        image_file = File()
-        image_file.name = fname
-        image_file.upToDate = True
-        self.setResult('imageFile', image_file)
+        image_file = PathObject(fname)
+        self.set_output('imageFile', image_file)
 
     def run_figure_file(self, fname, graphics_dev, width, height, 
                         excluded_inputs=set(['source'])):
@@ -424,19 +421,19 @@ class RFigure(RSource):
 class RSVGFigure(RFigure):
     def compute(self):
         code_str = \
-            urllib.unquote(str(self.forceGetInputFromPort('source', '')))
+            urllib.unquote(str(self.force_get_input('source', '')))
         RFigure.run_figure(self, code_str, 'svg', 4, 3)
 
 class RPNGFigure(RFigure):
     def compute(self):
         code_str = \
-            urllib.unquote(str(self.forceGetInputFromPort('source', '')))
+            urllib.unquote(str(self.force_get_input('source', '')))
         RFigure.run_figure(self, code_str, 'png', 640, 480)
 
 class RPDFFigure(RFigure):
     def compute(self):
         code_str = \
-            urllib.unquote(str(self.forceGetInputFromPort('source', '')))
+            urllib.unquote(str(self.force_get_input('source', '')))
         RFigure.run_figure(self, code_str, 'pdf', 4, 3)
 
 class RFactor(Module):
