@@ -1,48 +1,52 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
-################################################################################
-# This file contains a set of internal Spreadsheet basic classes used
-# by others:
-#   StandardWidgetHeaderView
-#   StandardWidgetItemDelegate
-#   StandardWidgetSheet
-################################################################################
-from PyQt4 import QtCore, QtGui
-from spreadsheet_helpers import CellHelpers, CellResizer
 
-################################################################################
+"""This file contains a set of internal Spreadsheet basic classes used by
+others:
+  StandardWidgetHeaderView
+  StandardWidgetItemDelegate
+  StandardWidgetSheet
+"""
+
+from __future__ import division
+
+from PyQt4 import QtCore, QtGui
+
+from .spreadsheet_helpers import CellHelpers, CellResizer
+
 
 class StandardWidgetHeaderView(QtGui.QHeaderView):
     """
@@ -50,16 +54,19 @@ class StandardWidgetHeaderView(QtGui.QHeaderView):
     column/row labels) inheriting from QHeaderView. The main
     difference between this class and the original one is that it
     allows resizing and stretching at the same time
-    
+
     """
-    minimumSize = 50
+    THICKNESS = 30
+    MINIMUM_SIZE = 50
+
+    fitToWindow = True
 
     def __init__(self, orientation, parent=None):
         """ StandardWidgetHeaderView(orientation: QtCore.Qt.Align...,
                                      parent: QWidget)
                                      -> StandardWidgetHeaderView
         Initialize the header view to be like the one in the spreadsheet table
-        
+
         """
         QtGui.QHeaderView.__init__(self, orientation, parent)
         self.setMovable(True)
@@ -67,30 +74,118 @@ class StandardWidgetHeaderView(QtGui.QHeaderView):
         self.resizeSections(QtGui.QHeaderView.Stretch)
         self.setClickable(True)
         self.setHighlightSections(True)
-        self.fitToViewport = False
         if orientation==QtCore.Qt.Vertical:
             self.setDefaultAlignment(QtCore.Qt.AlignHCenter |
                                      QtCore.Qt.AlignVCenter)
 
-    def setFitToViewport(self, fit=True):
-        """ setFitToViewport(fit: boolean) -> None        
-        Set fit to viewport for have all the sections always stretch
-        to the whole viewport
+        self.connect(self, QtCore.SIGNAL('sectionResized(int, int, int)'),
+                     self.section_resized)
+        self._target_size = None
 
+    section_sizes = None
+    def read_section_sizes(self):
+        if (self.section_sizes is None or
+                len(self.section_sizes) != self.count()):
+            self.section_sizes = [float(self.sectionSize(self.logicalIndex(i)))
+                                  for i in xrange(self.count())]
+
+    _resizing = False
+
+    def section_resized(self, log_index, old_size, new_size):
+        if self._resizing:
+            return
+        else:
+            self._resizing = True
+            try:
+                self._section_resized(log_index, old_size, new_size)
+            finally:
+                self._resizing = False
+
+    def resize_right_rows(self, vis_index):
+        if self._resizing:
+            return
+        else:
+            self._resizing = True
+            try:
+                self._resize_right_rows(vis_index)
+            finally:
+                self._resizing = False
+
+    def _section_resized(self, log_index, old_size, new_size):
+        """ section_resized(horizontal: bool, log_index: int,
+                old_size: int, new_size: int) -> None
+        Called when a section of of the header is resized
         """
-        self.fitToViewport = fit
+        if not self.fitToWindow or self._target_size is None:
+            return
+
+        vis_index = self.visualIndex(log_index)
+        if vis_index == self.count() - 1:
+            self.resizeSection(log_index, old_size)
+            return
+
+        orig_new_size = new_size
+
+        # Can't shrink below minimum size
+        if new_size < old_size and new_size < self.MINIMUM_SIZE:
+            new_size = self.MINIMUM_SIZE
+
+        if self._target_size is None:
+            return
+
+        # Can't take other cells below minimum size
+        if new_size > old_size:
+            min_right = 0
+            for i in xrange(vis_index + 1, self.count()):
+                if not self.isSectionHidden(self.logicalIndex(i)):
+                    min_right += self.MINIMUM_SIZE
+            pos = self.sectionPosition(log_index)
+            total_right = self._target_size - pos - new_size
+            if total_right < min_right:
+                new_size = self._target_size - pos - min_right
+
+        if new_size != orig_new_size:
+            self.resizeSection(log_index, new_size)
+
+        # Resize the rows to the right
+        self.read_section_sizes()
+        self.section_sizes[vis_index] = float(new_size)
+        self._resize_right_rows(vis_index + 1)
+
+    def _resize_right_rows(self, vis_index):
+        self.read_section_sizes()
+
+        previous_space = sum(self.section_sizes[vis_index:])
+        new_space = self._target_size - sum(self.section_sizes[:vis_index])
+
+        # If we are growing the sections
+        if new_space > previous_space:
+            allocated_space = new_space - previous_space
+            for i, size in enumerate(self.section_sizes[vis_index:], vis_index):
+                size += allocated_space * (size / previous_space)
+                self.section_sizes[i] = size
+                self.resizeSection(self.logicalIndex(i), size)
+        # If we are shrinking the sections
+        else:
+            reclaimed_space = previous_space - new_space
+            for i, size in enumerate(self.section_sizes[vis_index:], vis_index):
+                size -= reclaimed_space * (
+                        (size - self.MINIMUM_SIZE)/previous_space)
+                self.section_sizes[i] = size
+                self.resizeSection(self.logicalIndex(i), size)
 
     def sizeHint(self):
         """ sizeHint() -> QSize
         Set a default thickness of the bar to 30
-        
+
         """
         size = QtGui.QHeaderView.sizeHint(self)
         if self.orientation()==QtCore.Qt.Vertical:
-            size.setWidth(30)
+            size.setWidth(self.THICKNESS)
         else:
-            size.setHeight(30)
-        return size        
+            size.setHeight(self.THICKNESS)
+        return size
+
 
 class StandardWidgetItemDelegate(QtGui.QItemDelegate):
     """
@@ -102,7 +197,7 @@ class StandardWidgetItemDelegate(QtGui.QItemDelegate):
         """ StandardWidgetItemDelegate(table: QTableWidget)
                                        -> StandardWidgetItemDelegate
         Initialize to store a table and padding
-        
+
         """
         self.table = table
         self.padding = 4
@@ -111,7 +206,7 @@ class StandardWidgetItemDelegate(QtGui.QItemDelegate):
     def setPadding(self, padding):
         """ setPadding(padding: int) -> None
         Re-set padding to a different value
-        
+
         """
         if self.padding!=padding:
             self.padding = padding
@@ -121,7 +216,7 @@ class StandardWidgetItemDelegate(QtGui.QItemDelegate):
                                  option: QStyleOptionViewItem,
                                  index: QModelIndex) -> None
         Make sure the widget only occupied inside the padded area
-    
+
         """
         rect = self.table.visualRect(index)
         rect.adjust(self.padding,self.padding,-self.padding,-self.padding)
@@ -130,9 +225,9 @@ class StandardWidgetItemDelegate(QtGui.QItemDelegate):
 
     def paint(self, painter, option, index):
         """ paint(painter: QPainter, option: QStyleOptionViewItem,
-                  index: QModelIndex) -> None                  
+                  index: QModelIndex) -> None
         Paint the current cell with a ring outside
-        
+
         """
         QtGui.QItemDelegate.paint(self, painter, option, index)
         if ((index.row(), index.column())==self.table.activeCell):
@@ -141,11 +236,12 @@ class StandardWidgetItemDelegate(QtGui.QItemDelegate):
                 QtGui.QColor(0.8549*255, 0.6971*255, 0.2255*255)), self.padding))
             r = self.table.visualRect(index)
             painter.setClipRegion(QtGui.QRegion(r))
-            r.adjust(self.padding/2,self.padding/2,-self.padding/2,-self.padding/2)
+            r.adjust(self.padding//2, self.padding//2,
+                     -self.padding//2, -self.padding//2)
             painter.drawRoundedRect(r, self.padding, self.padding)
             painter.restore()
-            
-            
+
+
 class StandardWidgetSheet(QtGui.QTableWidget):
     """
     StandardWidgetSheet is a standard sheet that can contain any type
@@ -159,7 +255,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
         """ StandardWidgetSheet(rows: int, cols: int, parent: QWidget)
                                 -> StandardWidgetSheet
         Construct a sheet with rows x cols cells
-        
+
         """
         QtGui.QTableWidget.__init__(self, 0, 0, parent)
         self.setSelectionMode(QtGui.QAbstractItemView.NoSelection)
@@ -176,7 +272,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
                      self.columnMoved)
         self.connect(self.horizontalHeader(),
                      QtCore.SIGNAL('sectionPressed(int)'),
-                     self.forceColumnMultiSelect) 
+                     self.forceColumnMultiSelect)
         self.setVerticalHeader(StandardWidgetHeaderView(QtCore.Qt.Vertical,
                                                         self))
         self.verticalHeader().setSelectionModel(self.selectionModel())
@@ -196,7 +292,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
             self.connect(cornerButton,
                          QtCore.SIGNAL('clicked()'),
                          self.forceSheetSelect)
-        
+
         self.delegate = StandardWidgetItemDelegate(self)
         self.setItemDelegate(self.delegate)
         self.helpers = CellHelpers(parent, CellResizer(self))
@@ -209,11 +305,11 @@ class StandardWidgetSheet(QtGui.QTableWidget):
         self.activeCell = (-1,-1)
 
     def forceColumnMultiSelect(self, logicalIndex):
-        """ forceColumnMultiSelect(logicalIndex: int) -> None        
-        Make sure we always toggle the headerview in the right way        
+        """ forceColumnMultiSelect(logicalIndex: int) -> None
+        Make sure we always toggle the headerview in the right way
         NOTE: the MultiSelection type of SelectionMode does not work
         correctly for overlapping columns and rows selection
-        
+
         """
         if (self.selectionModel().isColumnSelected(logicalIndex, QtCore.QModelIndex())):
             self.selectionModel().select(self.model().index(0, logicalIndex),
@@ -225,11 +321,11 @@ class StandardWidgetSheet(QtGui.QTableWidget):
                                          QtGui.QItemSelectionModel.Columns)
 
     def forceRowMultiSelect(self, logicalIndex):
-        """ forceRowMultiSelect(logicalIndex: int) -> None        
-        Make sure we always toggle the headerview in the right way        
+        """ forceRowMultiSelect(logicalIndex: int) -> None
+        Make sure we always toggle the headerview in the right way
         NOTE: the MultiSelection type of SelectionMode does not work
         correctly for overlapping columns and rows selection
-        
+
         """
         if (self.selectionModel().isRowSelected(logicalIndex, QtCore.QModelIndex())):
             self.selectionModel().select(self.model().index(logicalIndex, 0),
@@ -241,16 +337,16 @@ class StandardWidgetSheet(QtGui.QTableWidget):
                                          QtGui.QItemSelectionModel.Rows)
 
     def forceSheetSelect(self):
-        """ forceSheetSelect() -> None        
+        """ forceSheetSelect() -> None
         Make sure we can toggle the whole sheet selection
-        
+
         """
         totalCells = self.rowCount()*self.columnCount()
         if (len(self.selectionModel().selectedIndexes())<totalCells):
             self.selectionModel().select(
                 QtGui.QItemSelection(self.model().index(0,0),
                                      self.model().index(self.rowCount()-1,
-                                                        self.columnCount()-1)),                
+                                                        self.columnCount()-1)),
                 QtGui.QItemSelectionModel.Select)
         else:
             self.selectionModel().clearSelection()
@@ -258,7 +354,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
     def updateHeaderStatus(self):
         """ updateHeaderStatus() -> None
         Update the visibility of the row and column header
-        
+
         """
         return
         self.horizontalHeader().setVisible(self.columnCount() > 1 or
@@ -269,7 +365,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
     def updateRowLabels(self, oldCount, newCount):
         """ updateRowLabels(oldCount: int, newCount: int) -> None
         Update vertical labels when the number of row changed
-        
+
         """
         vLabels = []
         vIdx = self.verticalHeader().visualIndex
@@ -281,14 +377,14 @@ class StandardWidgetSheet(QtGui.QTableWidget):
     def rowMoved(self, row, old, new):
         """ rowMove(row: int, old: int, new: int) -> None
         Renumber the vertical header labels when rows moved
-        
+
         """
         self.updateRowLabels(self.rowCount(), self.rowCount())
-        
+
     def updateColumnLabels(self, oldCount, newCount):
         """ updateColumnLabels(oldCount: int, newCount: int) -> None
         Update horizontal labels when the number of column changed
-        
+
         """
         hLabels = []
         vIdx = self.horizontalHeader().visualIndex
@@ -296,107 +392,60 @@ class StandardWidgetSheet(QtGui.QTableWidget):
             hLabels.append(chr(vIdx(i)+ord('A')))
         self.setHorizontalHeaderLabels(hLabels)
         self.updateHeaderStatus()
-        
+
     def columnMoved(self, row, old, new):
         """ columnMoved(row: int, old: int, new: int) -> None
         Renumber the horizontal header labels when columns moved
-        
+
         """
         self.updateColumnLabels(self.columnCount(), self.columnCount())
-        
+
     def setFitToWindow(self, fit=True):
         """ setFitToWindow(fit: boolean) -> None
         Force to fit all cells into the visible area. Set fit=False
         for the scroll mode where hidden cell can be viewed by scrolling
         the scrollbars.
-        
+
         """
         if fit!=self.fitToWindow:
             self.fitToWindow = fit
-            self.horizontalHeader().setFitToViewport(fit)
-            self.verticalHeader().setFitToViewport(fit)
+            self.horizontalHeader().fitToWindow = fit
+            self.horizontalHeader()._target_size = None
+            self.verticalHeader().fitToWindow = fit
+            self.verticalHeader()._target_size = None
             if not fit:
                 width = self.columnWidth(self.columnCount()-1)
                 height = self.rowHeight(self.rowCount()-1)
 
                 self.setColumnWidth(self.columnCount()-1, width)
                 self.setRowHeight(self.rowCount()-1, height)
-            self.horizontalHeader().setStretchLastSection(fit)
-            self.verticalHeader().setStretchLastSection(fit)
             self.stretchCells()
+            policy = (QtCore.Qt.ScrollBarAlwaysOff if fit
+                      else QtCore.Qt.ScrollBarAlwaysOn)
+            self.setHorizontalScrollBarPolicy(policy)
+            self.setVerticalScrollBarPolicy(policy)
 
     def showEvent(self, event):
         """ showEvent(event: QShowEvent) -> None
         Make sure to stretch the sheet on the first appearance
-        
+
         """
         self.stretchCells()
 
     def stretchCells(self):
         """ stretchCells() -> None
         Stretch all the cells with equally spaces to fit in the viewport
-        
+
         """
         if self.fitToWindow:
-            self.horizontalHeader().setFitToViewport(False)
             self.horizontalHeader().resizeSections(QtGui.QHeaderView.Stretch)
-            self.horizontalHeader().setFitToViewport(True)
-            self.verticalHeader().setFitToViewport(False)
             self.verticalHeader().resizeSections(QtGui.QHeaderView.Stretch)
-            self.verticalHeader().setFitToViewport(True)
-            
-    def resizeEvent(self, e):
-        """ resizeEvent(e: QResizeEvent) -> None
-        Resizes each row/column keeping the size ratios between them
-
-        """
-        if self.fitToWindow:
-            for min_size, getter, setter, count, final_size in [
-                    (self.horizontalHeader().minimumSize,
-                     self.columnWidth, self.setColumnWidth, self.columnCount(),
-                     e.size().width()),
-                    (self.verticalHeader().minimumSize,
-                     self.rowHeight, self.setRowHeight, self.rowCount(),
-                     e.size().height())]:
-                # Computes the total size of the columns
-                initial_size = 0
-                for i in xrange(count):
-                    size = getter(i)
-                    if size < min_size:
-                        initial_size += min_size
-                    else:
-                        initial_size += size
-
-                if initial_size == 0:
-                    continue
-
-                # Computes the resize ratio
-                ratio = float(final_size)/initial_size
-
-                i_total = 0
-                f_total = 0
-                for i in xrange(count - 1):
-                    initial = getter(i)
-                    if initial < min_size:
-                        initial = min_size
-                    final = int((initial + i_total) * ratio - f_total)
-                    if final < min_size:
-                        final = min_size
-                    setter(i, final)
-                    i_total += initial
-                    f_total += final
-                final = final_size - f_total
-                if final < min_size:
-                    final = min_size
-                setter(count - 1, final)
-
-        QtGui.QTableWidget.resizeEvent(self, e)
 
     def showHelpers(self, show, row, col):
-        """ showHelpers(show: boolean, row: int, col: int) -> None        
+        """ showHelpers(show: boolean, row: int, col: int) -> None
         Show/hide the helpers (resizer, toolbar) on the current cell
         depending on the value of show
-        
+
         """
         if self.helpers.isInteracting():
             return
@@ -410,10 +459,20 @@ class StandardWidgetSheet(QtGui.QTableWidget):
         else:
             self.helpers.hide()
 
+    def resizeEvent(self, e):
+        if not self.fitToWindow:
+            return
+
+        thickness = StandardWidgetHeaderView.THICKNESS
+        self.horizontalHeader()._target_size = self.size().width() - thickness
+        self.horizontalHeader().resize_right_rows(0)
+        self.verticalHeader()._target_size = self.size().height() - thickness
+        self.verticalHeader().resize_right_rows(0)
+
     def getRealLocation(self, vRow, vCol, visual=False):
         """ getRealLocation(vRow: int, vCol: int, visual: bool) -> (int, int)
         Return the actual location even if there is spanning at (vRow, vCol)
-        
+
         """
         # Qt doesn't provide a mechanism to map from a cell to its
         # span region, so we have to scan the whole spreadsheet N^2
@@ -440,12 +499,12 @@ class StandardWidgetSheet(QtGui.QTableWidget):
                             return (r, c)
                         if (r+rs, c+cs) in cellSet:
                             cellSet.remove((r+rs, c+cs))
-        return (-1, -1)        
+        return (-1, -1)
 
     def getCell(self, row, col):
         """ getCell(row: int, col: int) -> QWidget
         Get cell at a specific row and column
-        
+
         """
         return self.cellWidget(*self.getRealLocation(row, col))
 
@@ -453,7 +512,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
         """ getCellRect(row: int, col: int) -> QRect
         Return the rectangle surrounding the cell at location (row, col)
         in parent coordinates
-        
+
         """
         idx = self.model().index(*self.getRealLocation(row, col))
         return self.visualRect(idx)
@@ -462,7 +521,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
         """ getCellGlobalRect(row: int, col: int) -> QRect
         Return the rectangle surrounding the cell at location (row, col)
         in global coordinates
-        
+
         """
         rect = self.getCellRect(row, col)
         rect.moveTo(self.viewport().mapToGlobal(rect.topLeft()))
@@ -470,10 +529,10 @@ class StandardWidgetSheet(QtGui.QTableWidget):
 
     def setCellByWidget(self, row, col, cellWidget):
         """ setCellByWidget(row: int,
-                            col: int,                            
+                            col: int,
                             cellWidget: QWidget) -> None
         Replace the current location (row, col) with a cell widget
-        
+
         """
         if cellWidget:
             # Relax the size constraint of the widget
@@ -489,7 +548,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
     def selectCell(self, row, col, toggling):
         """ selectCell(row: int, col: int, toggling: bool) -> None
         Select a cell based on its current selection
-        
+
         """
         (row, col) = self.getRealLocation(row, col, visual=True)
         if toggling:
@@ -520,7 +579,7 @@ class StandardWidgetSheet(QtGui.QTableWidget):
         self.parent().toolBar.setCellToolBar(toolBar)
 
     def adjustWidgetGeometry(self, row, col):
-        """ setActiveCell(row: int, col: int) -> None        
+        """ setActiveCell(row: int, col: int) -> None
         Adjust the widget at cell (row, col) to fit inside the cell
 
         """
@@ -528,4 +587,3 @@ class StandardWidgetSheet(QtGui.QTableWidget):
         if cellWidget:
             index = self.model().index(*self.getRealLocation(row, col))
             self.delegate.updateEditorGeometry(cellWidget, None, index)
-        
