@@ -1,70 +1,75 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
-################################################################################
-# This file contains classes controlling tabs in the spreadsheets. A tab is
-# a container of a sheet:
-#   SizeSpinBox
-#   StandardTabDockWidget
-#   StandardWidgetSheetTab
-#   StandardWidgetTabBar
-#   StandardWidgetTabBarEditor
-#   StandardWidgetToolBar
-################################################################################
-from PyQt4 import QtCore, QtGui
+
+"""This file contains classes controlling tabs in the spreadsheets. A tab is
+a container of a sheet:
+  SizeSpinBox
+  StandardTabDockWidget
+  StandardWidgetSheetTab
+  StandardWidgetTabBar
+  StandardWidgetTabBarEditor
+  StandardWidgetToolBar
+"""
+
+from __future__ import division
+
 import os.path
-from spreadsheet_registry import spreadsheetRegistry
-from spreadsheet_sheet import StandardWidgetSheet
-from spreadsheet_cell import QCellPresenter, QCellContainer, QCellToolBar
-from spreadsheet_execute import assignPipelineCellLocations, \
-     executePipelineWithProgress
-from spreadsheet_config import configuration
+from PyQt4 import QtCore, QtGui
+
 from vistrails.core.inspector import PipelineInspector
+
+from .spreadsheet_registry import spreadsheetRegistry
+from .spreadsheet_sheet import StandardWidgetSheet
+from .spreadsheet_cell import QCellPresenter, QCellContainer, QCellToolBar
+from .spreadsheet_execute import assignPipelineCellLocations, \
+     executePipelineWithProgress
+from .spreadsheet_config import configuration
 import spreadsheet_rc
 
-################################################################################
 
 class SizeSpinBox(QtGui.QSpinBox):
     """
     SizeSpinBox is just an overrided spin box that will also emit
     'editingFinished()' signal when the user interact with mouse
-    
-    """    
+
+    """
     def __init__(self, initValue=0, parent=None):
         """ SizeSpinBox(initValue: int, parent: QWidget) -> SizeSpinBox
         Initialize with a default width of 50 and a value of 0
-        
+
         """
         QtGui.QSpinBox.__init__(self, parent)
         self.setMinimum(1)
@@ -75,22 +80,23 @@ class SizeSpinBox(QtGui.QSpinBox):
     def mouseReleaseEvent(self, event):
         """ mouseReleaseEvent(event: QMouseEvent) -> None
         Emit 'editingFinished()' signal when the user release a mouse button
-        
+
         """
         QtGui.QSpinBox.mouseReleaseEvent(self, event)
-        self.emit(QtCore.SIGNAL("editingFinished()"))        
+        self.emit(QtCore.SIGNAL("editingFinished()"))
+
 
 class StandardWidgetToolBar(QtGui.QToolBar):
     """
     StandardWidgetToolBar: The default toolbar for each sheet
     container. By default, only FitToWindow and Table resizing are
     included
-    
+
     """
     def __init__(self, parent=None):
         """ StandardWidgetToolBar(parent: QWidget) -> StandardWidgetToolBar
         Init the toolbar with default actions
-        
+
         """
         QtGui.QToolBar.__init__(self, parent)
         self.sheetTab = parent
@@ -103,11 +109,11 @@ class StandardWidgetToolBar(QtGui.QToolBar):
         self.addSeparator()
         self.layout().setSpacing(2)
         self.currentToolBarAction = None
-    
+
     def rowCountSpinBox(self):
         """ rowCountSpinBox() -> SizeSpinBox
         Return the row spin box widget:
-        
+
         """
         if not hasattr(self, 'rowSpinBox'):
             self.rowSpinBox = SizeSpinBox(self.sheetTab.sheet.rowCount())
@@ -122,7 +128,7 @@ class StandardWidgetToolBar(QtGui.QToolBar):
     def colCountSpinBox(self):
         """ colCountSpinBox() -> SizeSpinBox
         Return the column spin box widget:
-        
+
         """
         if not hasattr(self, 'colSpinBox'):
             self.colSpinBox = SizeSpinBox(self.sheetTab.sheet.columnCount())
@@ -138,7 +144,7 @@ class StandardWidgetToolBar(QtGui.QToolBar):
         """ setCellToolBar(cellToolBar: QToolBar) -> None
         Set the current cell toolbar on this toolbar. Use None to
         remove the cell toolbar
-        
+
         """
         if (not self.currentToolBarAction or
             self.widgetForAction(self.currentToolBarAction)!=cellToolBar):
@@ -151,11 +157,12 @@ class StandardWidgetToolBar(QtGui.QToolBar):
             else:
                 self.currentToolBarAction = None
 
+
 class StandardWidgetSheetTabInterface(object):
     """
     StandardWidgetSheetTabInterface is the interface for tab
     controller to call for manipulating a tab
-    
+
     """
     ### Belows are API Wrappers to connect to self.sheet
 
@@ -166,30 +173,30 @@ class StandardWidgetSheetTabInterface(object):
     def isSheetTabWidget(self):
         """ isSheetTabWidget() -> boolean
         Return True if this is a sheet tab widget
-        
+
         """
         return True
 
     def getDimension(self):
         """ getDimension() -> tuple
         Get the sheet dimensions
-        
+
         """
         return (0,0)
-            
+
     def setDimension(self, rc, cc):
         """ setDimension(rc: int, cc: int) -> None
         Set the sheet dimensions
-        
+
         """
         pass
-            
+
     def getCell(self, row, col):
-        """ getCell(row: int, col: int) -> QWidget        
+        """ getCell(row: int, col: int) -> QWidget
         Get cell at a specific row and column. In reality, this cell
         widget is inside a QCellContainer and the cell container is
         the actual widget under the cell
-        
+
         """
         cellWidget = self.getCellWidget(row, col)
         if isinstance(cellWidget, QCellContainer):
@@ -197,27 +204,27 @@ class StandardWidgetSheetTabInterface(object):
         return cellWidget
 
     def getCellWidget(self, row, col):
-        """ getCellWidget(row: int, col: int) -> QWidget        
+        """ getCellWidget(row: int, col: int) -> QWidget
         Get actual cell at a specific row and column. This will in
         fact return the container widget of a cell
-        
+
         """
         return None
 
     def setCellWidget(self, row, col, cellWidget):
         """ setCellWidget(row: int,
-                          col: int,                            
-                          cellWidget: QWidget) -> None                            
-        Replace the current location (row, col) with a 
+                          col: int,
+                          cellWidget: QWidget) -> None
+        Replace the current location (row, col) with a
         widget. The widget will be put into a container to be
         protected from being destroyed when taken out.
-        
+
         """
         pass
 
     def setCellByWidget(self, row, col, cellWidget):
         """ setCellByWidget(row: int,
-                            col: int,                            
+                            col: int,
                             cellWidget: QWidget) -> None
         Put the cellWidget inside a container and place it on the sheet
 
@@ -232,7 +239,7 @@ class StandardWidgetSheetTabInterface(object):
     def getCellToolBar(self, row, col):
         """ getCellToolBar(row: int, col: int) -> QWidget
         Return the toolbar widget at cell location (row, col)
-        
+
         """
         cell = self.getCell(row, col)
         if cell:
@@ -254,7 +261,7 @@ class StandardWidgetSheetTabInterface(object):
         """ getCellRect(row: int, col: int) -> QRect
         Return the rectangle surrounding the cell at location (row, col)
         in parent coordinates
-        
+
         """
         return QtCore.QRect()
 
@@ -262,12 +269,12 @@ class StandardWidgetSheetTabInterface(object):
         """ getCellGlobalRect(row: int, col: int) -> QRect
         Return the rectangle surrounding the cell at location (row, col)
         in global coordinates
-        
+
         """
         return QtCore.QRect()
 
     def getFreeCell(self):
-        """ getFreeCell() -> tuple        
+        """ getFreeCell() -> tuple
         Get a free cell location (row, col) on the spreadsheet
 
         """
@@ -280,17 +287,17 @@ class StandardWidgetSheetTabInterface(object):
         (r, c) = self.lastCellLocation
         (rs, cs) = self.getSpan(r, c)
         index = (colCount * r + c + cs) % (rowCount*colCount)
-        return (index/colCount, index%colCount)
+        return (index//colCount, index%colCount)
 
     def setCellByType(self, row, col, cellType, inputPorts):
         """ setCellByType(row: int,
                           col: int,
                           cellType: a type inherits from QWidget,
-                          inpurPorts: tuple) -> None                          
+                          inpurPorts: tuple) -> None
         Replace the current location (row, col) with a cell of
         cellType. If the current type of that cell is the same as
         cellType, only the contents is updated with inputPorts.
-        
+
         """
         oldCell = self.getCell(row, col)
         if cellType is None or not isinstance(oldCell, cellType):
@@ -308,29 +315,29 @@ class StandardWidgetSheetTabInterface(object):
         self.lastCellLocation = (row, col)
 
     def showHelpers(self, show, globalPos):
-        """ showHelpers(show: boolean, globalPos: QPoint) -> None    
+        """ showHelpers(show: boolean, globalPos: QPoint) -> None
         Show/hide the helpers (toolbar, resizer when the mouse is at
         globalPos
-        
+
         """
         pass
 
     def setCellPipelineInfo(self, row, col, info):
-        """ setCellPipelineInfo(row: int, col: int, info: any type) -> None        
+        """ setCellPipelineInfo(row: int, col: int, info: any type) -> None
         Provide a way for the spreadsheet to store vistrail
         information, info, for the cell (row, col)
-        
+
         """
         if not (row,col) in self.pipelineInfo:
             self.pipelineInfo[(row,col)] = {}
         self.pipelineInfo[(row,col)] = info
 
     def getCellPipelineInfo(self, row, col):
-        """ getCellPipelineInfo(row: int, col: int) -> any type        
+        """ getCellPipelineInfo(row: int, col: int) -> any type
         Provide a way for the spreadsheet to extract vistrail
         information, info, for the cell (row, col)
-        
-        """        
+
+        """
         if not (row,col) in self.pipelineInfo:
             return None
         return self.pipelineInfo[(row,col)]
@@ -338,29 +345,29 @@ class StandardWidgetSheetTabInterface(object):
     def getSelectedLocations(self):
         """ getSelectedLocations() -> list
         Return the selected locations (row, col) of the current sheet
-        
+
         """
         return []
 
     def clearSelection(self):
         """ clearSelection() -> None
         Clear all the selection in the current sheet
-        
+
         """
         pass
-    
+
     def deleteCell(self, row, col):
         """ deleteCell(row, col: int) -> None
         Delete a cell in the sheet
-        
+
         """
         self.setCellByType(row, col, None, None)
         self.setCellPipelineInfo(row, col, None)
-        
+
     def deleteAllCells(self):
         """ deleteAllCells() -> None
         Delete all cells in the sheet
-        
+
         """
         (rowCount, columnCount) = self.getDimension()
         for r in xrange(rowCount):
@@ -368,12 +375,12 @@ class StandardWidgetSheetTabInterface(object):
                 self.deleteCell(r, c)
 
     def takeCell(self, row, col):
-        """ takeCell(row, col) -> QWidget        
+        """ takeCell(row, col) -> QWidget
         Free the cell widget at (row, col) from the tab and return as
         the result of the function. If there is no widget at (row,
         col). This returns None. The ownership of the widget is passed
         to the caller.
-        
+
         """
         cell = self.getCellWidget(row, col)
         if isinstance(cell, QCellContainer):
@@ -386,7 +393,7 @@ class StandardWidgetSheetTabInterface(object):
     def setCellEditingMode(self, r, c, editing=True):
         """ setCellEditingMode(r: int, c: int, editing: bool) -> None
         Turn on/off the editing mode of a single cell
-        
+
         """
         if editing:
             cellWidget = self.getCell(r, c)
@@ -407,11 +414,11 @@ class StandardWidgetSheetTabInterface(object):
                 cellWidget = presenter.releaseCellWidget()
                 self.setCellByWidget(r, c, cellWidget)
                 presenter.hide()
-    
+
     def setEditingMode(self, editing=True):
         """ setEditingMode(editing: bool) -> None
         Turn on/off the editing mode of the tab
-        
+
         """
         # Turn off active cell selection
         self.sheet.clearSelection()
@@ -427,7 +434,7 @@ class StandardWidgetSheetTabInterface(object):
         """ swapCell(row, col: int, newSheet: Sheet,
                      newRow, newCol: int) -> None
         Swap the (row, col) of this sheet to (newRow, newCol) of newSheet
-        
+
         """
         myWidget = self.takeCell(row, col)
         theirWidget = newSheet.takeCell(newRow, newCol)
@@ -442,7 +449,7 @@ class StandardWidgetSheetTabInterface(object):
         """ copyCell(row, col: int, newSheet: Sheet,
                      newRow, newCol: int) -> None
         Copy the (row, col) of this sheet to (newRow, newCol) of newSheet
-        
+
         """
         info = self.getCellPipelineInfo(row, col)
         if info:
@@ -461,10 +468,10 @@ class StandardWidgetSheetTabInterface(object):
     def executePipelineToCell(self, pInfo, row, col, reason=''):
         """ executePipelineToCell(p: tuple, row: int, col: int) -> None
         p: (locator, version, actions, pipeline)
-        
+
         Execute a pipeline and put all of its cell to (row, col). This
         need to be fixed to layout all cells inside the pipeline
-        
+
         """
         pipeline = self.setPipelineToLocateAt(row, col, pInfo[3])
         executePipelineWithProgress(pipeline, 'Execute Cell',
@@ -476,10 +483,10 @@ class StandardWidgetSheetTabInterface(object):
 
     def setPipelineToLocateAt(self, row, col, inPipeline, cellIds=[]):
         """ setPipelineToLocateAt(row: int, col: int, inPipeline: Pipeline,
-                                  cellIds: [ids]) -> Pipeline                                  
+                                  cellIds: [ids]) -> Pipeline
         Modify the pipeline to have its cells (provided by cellIds) to
         be located at (row, col) of this sheet
-        
+
         """
         sheetName = str(self.tabWidget.tabText(self.tabWidget.indexOf(self)))
         # Note that we must increment row/col by 1 to match how the
@@ -490,7 +497,7 @@ class StandardWidgetSheetTabInterface(object):
     def getPipelineInfo(self, row, col):
         """ getPipelineInfo(row: int, col: int) -> tuple
         Return (locator, versionNumber, actions, pipeline, controller) for a cell
-        
+
         """
         info = self.getCellPipelineInfo(row, col)
         if info:
@@ -504,14 +511,14 @@ class StandardWidgetSheetTabInterface(object):
     def exportSheetToImage(self, fileName):
         """ exportSheetToImage() -> None
         Montage all the cell images and export to a file
-        
+
         """
         (rCount, cCount) = self.getDimension()
         if rCount<1 or cCount<1: return
         cellHeights = [self.getCellRect(r, 0).height()
                        for r in xrange(rCount)]
         cellWidths = [self.getCellRect(0, c).width()
-                      for c in xrange(cCount)] 
+                      for c in xrange(cCount)]
         finalImage = QtGui.QImage(sum(cellWidths), sum(cellHeights), QtGui.QImage.Format_ARGB32)
         finalImage.fill(0xFFFFFFFF)
         painter = QtGui.QPainter(finalImage)
@@ -522,14 +529,14 @@ class StandardWidgetSheetTabInterface(object):
                 widget = self.getCell(r, c)
                 if widget:
                     pix = widget.grabWindowPixmap()
-                    cx = (cellWidths[c]-pix.width())/2
-                    cy = (cellHeights[r]-pix.height())/2
+                    cx = (cellWidths[c]-pix.width())//2
+                    cy = (cellHeights[r]-pix.height())//2
                     painter.drawPixmap(x+cx, y+cy, widget.grabWindowPixmap())
                 x += cellWidths[c]
             y += cellHeights[r]
         painter.end()
-         
-        #forcing png format if no extension was provided 
+
+        #forcing png format if no extension was provided
         (_,ext) = os.path.splitext(fileName)
         if ext == '':
             finalImage.save(fileName, 'png')
@@ -540,7 +547,7 @@ class StandardWidgetSheetTabInterface(object):
     def exportSheetToImages(self, dirPath, format='png'):
         """ exportSheetToImage() -> None
         Montage all the cell images and export to a file
-        
+
         """
         (rCount, cCount) = self.getDimension()
         for r in xrange(rCount):
@@ -557,7 +564,7 @@ class StandardWidgetSheetTabInterface(object):
         Set the spanning at location (row, col). This is only a place
         holder. Subclasses should implement this and getSpan for a
         fully functioning spanning feature.
-        
+
         """
         pass
 
@@ -566,23 +573,24 @@ class StandardWidgetSheetTabInterface(object):
         Return the spanning at location (row, col). This is only a
         place holder. Subclasses should implement this and setSpan for
         a fully functioning spanning feature.
-        
+
         """
         return (1, 1)
 
+
 class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
     """
     StandardWidgetSheetTab is a container of StandardWidgetSheet with
     a toolbar on top. This will be added directly to a QTabWidget for
     displaying the spreadsheet.
-    
+
     """
     def __init__(self, tabWidget,row=None , col=None):
         """ StandardWidgetSheet(tabWidget: QTabWidget,
                                 row: int,
                                 col: int) -> StandardWidgetSheet
         Initialize with a toolbar and a sheet widget
-                                
+
         """
         QtGui.QWidget.__init__(self, None)
         StandardWidgetSheetTabInterface.__init__(self)
@@ -607,17 +615,17 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
     def rowSpinBoxChanged(self):
         """ rowSpinBoxChanged() -> None
         Handle the number of row changed
-        
+
         """
         if self.toolBar.rowSpinBox.value()!=self.sheet.rowCount():
             self.sheet.setRowCount(self.toolBar.rowSpinBox.value())
             self.sheet.stretchCells()
             self.setEditingMode(self.tabWidget.editingMode)
-        
+
     def colSpinBoxChanged(self):
         """ colSpinBoxChanged() -> None
         Handle the number of row changed
-        
+
         """
         if self.toolBar.colSpinBox.value()!=self.sheet.columnCount():
             self.sheet.setColumnCount(self.toolBar.colSpinBox.value())
@@ -629,22 +637,22 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
     def getDimension(self):
         """ getDimension() -> tuple
         Get the sheet dimensions
-        
+
         """
         return (self.sheet.rowCount(), self.sheet.columnCount())
-            
+
     def setDimension(self, rc, cc):
         """ setDimension(rc: int, cc: int) -> None
         Set the sheet dimensions
-        
+
         """
         self.toolBar.rowCountSpinBox().setValue(rc)
         self.toolBar.colCountSpinBox().setValue(cc)
-            
+
     def getCellWidget(self, row, col):
         """ getCellWidget(row: int, col: int) -> QWidget
         Get cell at a specific row and column.
-        
+
         """
         return self.sheet.getCell(row, col)
 
@@ -652,7 +660,7 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
         """ getCellRect(row: int, col: int) -> QRect
         Return the rectangle surrounding the cell at location (row, col)
         in parent coordinates
-        
+
         """
         return self.sheet.getCellRect(row, col)
 
@@ -660,15 +668,15 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
         """ getCellGlobalRect(row: int, col: int) -> QRect
         Return the rectangle surrounding the cell at location (row, col)
         in global coordinates
-        
+
         """
         return self.sheet.getCellGlobalRect(row, col)
 
     def showHelpers(self, show, globalPos):
-        """ showHelpers(show: boolean, globalPos: QPoint) -> None        
+        """ showHelpers(show: boolean, globalPos: QPoint) -> None
         Show/hide the helpers (toolbar, resizer) depending on the
         status of show and the global position of the cursor
-        
+
         """
         localPos = self.sheet.viewport().mapFromGlobal(QtGui.QCursor.pos())
         row = self.sheet.rowAt(localPos.y())
@@ -677,11 +685,11 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
         show =  show and (rect.x()+rect.width()-localPos.x()<100 and
                           rect.y()+rect.height()-localPos.y()<100)
         self.sheet.showHelpers(show, row, col)
-        
+
     def getSelectedLocations(self):
         """ getSelectedLocations() -> list
         Return the selected locations (row, col) of the current sheet
-        
+
         """
         indexes = self.sheet.selectedIndexes()
         return [(idx.row(), idx.column()) for idx in indexes]
@@ -689,23 +697,23 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
     def clearSelection(self):
         """ clearSelection() -> None
         Clear all the selection in the current sheet
-        
+
         """
         self.sheet.clearSelection()
-    
+
     def setCellWidget(self, row, col, cellWidget):
         """ setCellWidget(row: int,
-                            col: int,                            
-                            cellWidget: QWidget) -> None                            
+                            col: int,
+                            cellWidget: QWidget) -> None
         Replace the current location (row, col) with a cell widget
-        
+
         """
         self.sheet.setCellByWidget(row, col, cellWidget)
 
     def dragEnterEvent(self, event):
         """ dragEnterEvent(event: QDragEnterEvent) -> None
         Set to accept drops from the version tree
-        
+
         """
         mimeData = event.mimeData()
         if hasattr(mimeData, 'versionId'):
@@ -716,7 +724,7 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
     def dragMoveEvent(self, event):
         """ dragEnterEvent(event: QDragEnterEvent) -> None
         Set to accept drops while moving from the version tree
-        
+
         """
         mimeData = event.mimeData()
         if (hasattr(mimeData, 'versionId') and
@@ -727,7 +735,7 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
 
     def dropEvent(self, event):
         """ Execute the pipeline at the particular location """
-        mimeData = event.mimeData()        
+        mimeData = event.mimeData()
         if (hasattr(mimeData, 'versionId') and
             hasattr(mimeData, 'controller')):
             event.accept()
@@ -755,7 +763,7 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
     def setSpan(self, row, col, rowSpan, colSpan):
         """ setSpan(row, col, rowSpan, colSpan: int) -> None
         Set the spanning at location (row, col).
-        
+
         """
         colSpan = max(colSpan, 1)
         rowSpan = max(rowSpan, 1)
@@ -766,7 +774,7 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
                 for c in xrange(colSpan):
                     if r!=0 or c!=0:
                         self.deleteCell(row+r, col+c)
-                        
+
             # Take the current widget out
             curWidget = self.takeCell(row, col)
 
@@ -782,22 +790,22 @@ class StandardWidgetSheetTab(QtGui.QWidget, StandardWidgetSheetTabInterface):
         Return the spanning at location (row, col). This is only a
         place holder. Subclasses should implement this and setSpan for
         a fully functioning spanning feature.
-        
+
         """
         return (self.sheet.rowSpan(row, col), self.sheet.columnSpan(row, col))
 
-    
-class StandardWidgetTabBarEditor(QtGui.QLineEdit):    
+
+class StandardWidgetTabBarEditor(QtGui.QLineEdit):
     """
     StandardWidgetTabBarEditor overrides QLineEdit to enable canceling
     edit when Esc is pressed
-    
+
     """
     def __init__(self, text='', parent=None):
         """ StandardWidgetTabBarEditor(text: str, parent: QWidget)
                                        -> StandardWidgetTabBarEditor
         Store the original text at during initialization
-        
+
         """
         QtGui.QLineEdit.__init__(self, text, parent)
         self.originalText = text
@@ -805,7 +813,7 @@ class StandardWidgetTabBarEditor(QtGui.QLineEdit):
     def keyPressEvent(self, e):
         """ keyPressEvent(e: QKeyEvent) -> None
         Override keyPressEvent to handle Esc key
-        
+
         """
         if e.key()==QtCore.Qt.Key_Escape:
             e.ignore()
@@ -814,16 +822,17 @@ class StandardWidgetTabBarEditor(QtGui.QLineEdit):
         else:
             QtGui.QLineEdit.keyPressEvent(self, e)
 
+
 class StandardWidgetTabBar(QtGui.QTabBar):
     """
     StandardWidgetTabBar: a customized QTabBar to allow double-click
     to change tab name
-    
+
     """
     def __init__(self, parent=None):
         """ StandardWidgetTabBar(parent: QWidget) -> StandardWidgetTabBar
         Initialize like the original QTabWidget TabBar
-        
+
         """
         QtGui.QTabBar.__init__(self, parent)
         self.setAcceptDrops(True)
@@ -831,7 +840,7 @@ class StandardWidgetTabBar(QtGui.QTabBar):
                           'by dragging the tabs')
         self.setDrawBase(False)
         self.editingIndex = -1
-        self.editor = None        
+        self.editor = None
         self.setFocusPolicy(QtCore.Qt.NoFocus)
         self.connect(self, QtCore.SIGNAL('currentChanged(int)'),
                      self.updateTabText)
@@ -846,18 +855,18 @@ class StandardWidgetTabBar(QtGui.QTabBar):
     def mouseDoubleClickEvent(self, e):
         """ mouseDoubleClickEvent(e: QMouseEvent) -> None
         Handle Double-Click event to start the editor
-        
+
         """
         if e.buttons()!=QtCore.Qt.LeftButton or self.editor: return
-        
+
         # Update the current editing tab widget
         self.editingIndex = self.currentIndex()
-        
+
         # A hack to capture the rect of the triangular tab from commonstyle.cpp
         rect = self.tabRect(self.editingIndex)
         h = rect.height()-2
-        dx = h/3 + 3
-        rect.adjust(dx+1,1,-dx,-1)
+        dx = h//3 + 3
+        rect.adjust(dx+1, 1, -dx, -1)
 
         # Display the editor inplace of the tab text
         text = self.tabText(self.editingIndex)
@@ -875,7 +884,7 @@ class StandardWidgetTabBar(QtGui.QTabBar):
     def updateTabText(self, idx=0):
         """ updateTabText(idx: int) -> None
         Update the tab text after editing has been finished
-        
+
         """
         if self.editingIndex>=0 and self.editor:
             self.setTabText(self.editingIndex, self.editor.text())
@@ -888,19 +897,19 @@ class StandardWidgetTabBar(QtGui.QTabBar):
     def indexAtPos(self, p):
         """ indexAtPos(p: QPoint) -> int Reimplement of the private
         indexAtPos to find the tab index under a point
-        
+
         """
         if self.tabRect(self.currentIndex()).contains(p):
             return self.currentIndex()
         for i in xrange(self.count()):
-            if self.isTabEnabled(i) and self.tabRect(i).contains(p):                
+            if self.isTabEnabled(i) and self.tabRect(i).contains(p):
                 return i
-        return -1;
+        return -1
 
     def mousePressEvent(self, e):
         """ mousePressEvent(e: QMouseEvent) -> None
         Handle mouse press event to see if we should start to drag tabs or not
-        
+
         """
         QtGui.QTabBar.mousePressEvent(self, e)
         if e.buttons()==QtCore.Qt.LeftButton and self.editor==None:
@@ -909,7 +918,7 @@ class StandardWidgetTabBar(QtGui.QTabBar):
     def getGlobalRect(self, index):
         """ getGlobalRect(self, index: int)
         Get the rectangle of a tab in global coordinates
-        
+
         """
         if index<0: return None
         rect = self.tabRect(index)
@@ -919,18 +928,18 @@ class StandardWidgetTabBar(QtGui.QTabBar):
     def highlightTab(self, index):
         """ highlightTab(index: int)
         Highlight the rubber band of a tab
-        
+
         """
         if index==-1:
             self.innerRubberBand.hide()
         else:
             self.innerRubberBand.setGeometry(self.tabRect(index))
             self.innerRubberBand.show()
-            
+
     def mouseMoveEvent(self, e):
         """ mouseMoveEvent(e: QMouseEvent) -> None
         Handle dragging tabs in and out or around
-        
+
         """
         QtGui.QTabBar.mouseMoveEvent(self, e)
         if self.startDragPos:
@@ -941,7 +950,7 @@ class StandardWidgetTabBar(QtGui.QTabBar):
         if self.dragging:
             t = self.indexAtPos(e.pos())
             if t!=-1:
-                if t!=self.targetTab:                    
+                if t!=self.targetTab:
                     self.targetTab = t
                     self.outerRubberBand.hide()
                     self.highlightTab(t)
@@ -961,7 +970,7 @@ class StandardWidgetTabBar(QtGui.QTabBar):
     def mouseReleaseEvent(self, e):
         """ mouseReleaseEvent(e: QMouseEvent) -> None
         Make sure the tab moved at the end
-        
+
         """
         QtGui.QTabBar.mouseReleaseEvent(self, e)
         if self.dragging:
@@ -977,26 +986,26 @@ class StandardWidgetTabBar(QtGui.QTabBar):
             self.targetTab = -1
             self.highlightTab(-1)
             self.outerRubberBand.hide()
-            
+
     def slotIndex(self, pos):
         """ slotIndex(pos: QPoint) -> int
         Return the slot index between the slots at the cursor pos
-        
+
         """
         p = self.mapFromGlobal(pos)
         for i in xrange(self.count()):
             r = self.tabRect(i)
             if self.isTabEnabled(i) and r.contains(p):
-                if p.x()<(r.x()+r.width()/2):
+                if p.x() < (r.x()+r.width()//2):
                     return i
                 else:
                     return i+1
         return -1
-        
+
     def slotGeometry(self, idx):
         """ slotGeometry(idx: int) -> QRect
         Return the geometry between the slots at cursor pos
-        
+
         """
         if idx<0 or self.count()==0: return None
         if idx<self.count():
@@ -1012,7 +1021,7 @@ class StandardWidgetTabBar(QtGui.QTabBar):
     def dragEnterEvent(self, event):
         """ dragEnterEvent(event: QDragEnterEvent) -> None
         Set to accept drops from the other cell info
-        
+
         """
         mimeData = event.mimeData()
         if hasattr(mimeData, 'cellInfo'):
@@ -1023,22 +1032,22 @@ class StandardWidgetTabBar(QtGui.QTabBar):
                 self.setCurrentIndex(idx)
         else:
             event.ignore()
-            
+
     def dragMoveEvent(self, event):
         """ dragMoveEvent(event: QDragMoveEvent) -> None
         Set to accept drops from the other cell info
-        
+
         """
         idx = self.indexAtPos(event.pos())
         if idx>=0:
             self.setCurrentIndex(idx)
-            
-            
+
+
 class StandardTabDockWidget(QtGui.QDockWidget):
     """
     StandardTabDockWidget inherits from QDockWidget to contain a sheet
     widget floating around that can be merge back to tab controller
-    
+
     """
     def __init__(self, title, tabWidget, tabBar, tabController):
         """ StandardTabDockWidget(title: str,
@@ -1047,7 +1056,7 @@ class StandardTabDockWidget(QtGui.QDockWidget):
                                   tabController: StandardWidgetTabController)
                                   -> StandardTabDockWidget
         Initialize the dock widget to override the floating button
-        
+
         """
         QtGui.QDockWidget.__init__(self, title, None,
                                    QtCore.Qt.FramelessWindowHint)
@@ -1071,10 +1080,10 @@ class StandardTabDockWidget(QtGui.QDockWidget):
         self.resize(tabWidget.size())
 
     def findFloatingButton(self):
-        """ findFloatingButton() -> QAbstractButton        
+        """ findFloatingButton() -> QAbstractButton
         Hack to find the private Floating Button. Since there is only
         one button exists, we just need to find QAbstractButton
-        
+
         """
         for c in self.children():
             if isinstance(c, QtGui.QAbstractButton):
@@ -1084,7 +1093,7 @@ class StandardTabDockWidget(QtGui.QDockWidget):
     def eventFilter(self, q, e):
         """ eventFilter(q: QObject, e: QEvent) -> depends on event type
         Event filter the floating button to makes it merge to the tab controller
-        
+
         """
         if q and q==self.floatingButton:
             if (e.type()==QtCore.QEvent.MouseButtonRelease and
@@ -1096,7 +1105,7 @@ class StandardTabDockWidget(QtGui.QDockWidget):
                 return False
         return QtGui.QDockWidget.eventFilter(self, q, e)
 
-    def isTabControllerUnderMouse(self, tb):        
+    def isTabControllerUnderMouse(self, tb):
         """ Check if any of common parent of the tab controller and tb
         is under the mouse """
         tbp = []
@@ -1113,7 +1122,7 @@ class StandardTabDockWidget(QtGui.QDockWidget):
     def event(self, e):
         """ event(e: QEvent) -> depends on event type
         Handle movement of the dock widget to snap to the tab controller
-        
+
         """
         # MOUSE PRESS (QtCore.QEvent.NonClientAreaMouseButtonPress=174)
         if e.type() in [QtCore.QEvent.MouseButtonPress,174]:
@@ -1191,7 +1200,7 @@ class StandardTabDockWidget(QtGui.QDockWidget):
                 self.hide()
                 self.tabController.mergeTab(self, self.tabController.count())
                 return True
-            
+
         return QtGui.QDockWidget.event(self, e)
 
     def closeEvent(self, event):
@@ -1199,5 +1208,6 @@ class StandardTabDockWidget(QtGui.QDockWidget):
         self.tabController.mergeTab(self, self.tabController.count())
         event.accept()
 
+
 spreadsheetRegistry.registerSheet('StandardWidgetSheetTab',
                                   StandardWidgetSheetTab)
