@@ -1,3 +1,40 @@
+###############################################################################
+##
+## Copyright (C) 2014-2015, New York University.
+## Copyright (C) 2013-2014, NYU-Poly.
+## All rights reserved.
+## Contact: contact@vistrails.org
+##
+## This file is part of VisTrails.
+##
+## "Redistribution and use in source and binary forms, with or without
+## modification, are permitted provided that the following conditions are met:
+##
+##  - Redistributions of source code must retain the above copyright notice,
+##    this list of conditions and the following disclaimer.
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
+##    documentation and/or other materials provided with the distribution.
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
+##    this software without specific prior written permission.
+##
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
+##
+###############################################################################
+
+from __future__ import division
+
 import datetime
 from distutils.version import LooseVersion
 import re
@@ -6,6 +43,10 @@ import warnings
 
 from vistrails.core.modules.vistrails_module import Module, ModuleError
 from vistrails.core.bundles import py_import
+from vistrails.core.utils import VistrailsWarning
+
+
+PYTZ_MIN_VER = LooseVersion('2012')
 
 
 class UTC(datetime.tzinfo):
@@ -76,45 +117,45 @@ _decimal_fmt = re.compile(
         '([0-9][0-9])?$')   # minutes
 
 def make_timezone(s):
-        if s == 'local':
-            return LocalTimezone()
+    if s == 'local':
+        return LocalTimezone()
 
-        match = _decimal_fmt.match(s)
-        if match is not None:
-            sign, hours, minutes = match.groups('')
-            sign = -1 if sign == '-' else 1
-            hours = int(hours)
-            minutes = int(minutes) if minutes else 0
-            offset = datetime.timedelta(
-                    minutes=sign * (hours*60 + minutes))
-            name = '%s%02d%02d' % (
-                    '-' if sign == -1 else '+',
-                    hours,
-                    minutes)
-            return FixedOffset(offset, name)
+    match = _decimal_fmt.match(s)
+    if match is not None:
+        sign, hours, minutes = match.groups('')
+        sign = -1 if sign == '-' else 1
+        hours = int(hours)
+        minutes = int(minutes) if minutes else 0
+        offset = datetime.timedelta(
+                minutes=sign * (hours*60 + minutes))
+        name = '%s%02d%02d' % (
+                '-' if sign == -1 else '+',
+                hours,
+                minutes)
+        return FixedOffset(offset, name)
 
+    try:
+        pytz = py_import('pytz', {
+                'pip': 'pytz',
+                'linux-debian': 'python-tz',
+                'linux-ubuntu': 'python-tz',
+                'linux-fedora': 'pytz'})
+    except ImportError: # pragma: no cover
+        raise ValueError("can't understand timezone %r (maybe you should "
+                         "install pytz?)" % s)
+    else:
+        ver = LooseVersion(pytz.__version__)
+        if ver < PYTZ_MIN_VER: # pragma: no cover
+            warnings.warn(
+                    "You are using an old version of pytz (%s). You might "
+                    "run into some issues with daylight saving handling." %
+                    pytz.__version__,
+                    category=VistrailsWarning)
         try:
-            pytz = py_import('pytz', {
-                    'pip': 'pytz',
-                    'linux-debian': 'python-tz',
-                    'linux-ubuntu': 'python-tz',
-                    'linux-fedora': 'pytz'})
-        except ImportError:
-            raise ValueError("can't understand timezone %r (maybe you should "
-                             "install pytz?)" % s)
-        else:
-            ver = LooseVersion(pytz.__version__)
-            if ver < LooseVersion('2012'):
-                warnings.warn(
-                        "You are using an old version of pytz (%s). You might "
-                        "run into some issues with daylight saving handling." %
-                        pytz.__version__,
-                        category=UserWarning)
-            try:
-                return pytz.timezone(s)
-            except KeyError:
-                raise ValueError("can't understand timezone %r (defaulted to "
-                                 "pytz, which gave no answer)" % s)
+            return pytz.timezone(s)
+        except KeyError:
+            raise ValueError("can't understand timezone %r (defaulted to "
+                             "pytz, which gave no answer)" % s)
 
 
 class TimestampsToDates(Module):
@@ -136,10 +177,10 @@ class TimestampsToDates(Module):
         return [datetime.datetime.fromtimestamp(t, utc) for t in timestamps]
 
     def compute(self):
-        timestamps = self.getInputFromPort('timestamps')
+        timestamps = self.get_input('timestamps')
 
         result = self.convert(timestamps)
-        self.setResult('dates', result)
+        self.set_output('dates', result)
 
 
 class StringsToDates(Module):
@@ -185,7 +226,7 @@ class StringsToDates(Module):
                     'linux-debian': 'python-dateutil',
                     'linux-ubuntu': 'python-dateutil',
                     'linux-fedora': 'python-dateutil'})
-            except ImportError:
+            except ImportError: # pragma: no cover
                 raise ValueError("can't read dates without a format without "
                                  "the dateutil package")
             from dateutil import parser
@@ -194,7 +235,11 @@ class StringsToDates(Module):
         else:
             result = [datetime.datetime.strptime(s, fmt) for s in strings]
 
-        if tz:
+        if tz is None:
+            result = [dt.replace(tzinfo=None) for dt in result]
+        elif hasattr(tz, 'localize'):
+            result = [tz.localize(dt) for dt in result]
+        else:
             # Compute the time without daylight saving
             result = [dt.replace(tzinfo=tz) for dt in result]
 
@@ -215,22 +260,20 @@ class StringsToDates(Module):
                         # For dst -> standard (fall): the time will be in dst,
                         #   although it could also have been standard (there is
                         #   noway to know which one was meant)
-        else:
-            result = [dt.replace(tzinfo=None) for dt in result]
 
         return result
 
     def compute(self):
-        tz = self.getInputFromPort('timezone')
+        tz = self.get_input('timezone')
 
-        strings = self.getInputFromPort('strings')
-        fmt = self.getInputFromPort('format')
+        strings = self.get_input('strings')
+        fmt = self.get_input('format')
 
         try:
             result = self.convert(strings, fmt, tz)
         except ValueError, e:
             raise ModuleError(self, e.message)
-        self.setResult('dates', result)
+        self.set_output('dates', result)
 
 
 class DatesToMatplotlib(Module):
@@ -252,12 +295,12 @@ class DatesToMatplotlib(Module):
                     'linux-debian': 'python-matplotlib',
                     'linux-ubuntu': 'python-matplotlib',
                     'linux-fedora': 'python-matplotlib'})
-        except ImportError:
+        except ImportError: # pragma: no cover
             raise ModuleError(self, "matplotlib is not available")
 
-        datetimes = self.getInputFromPort('datetimes')
+        datetimes = self.get_input('datetimes')
         result = self.convert(datetimes)
-        self.setResult('dates', result)
+        self.set_output('dates', result)
 
 
 class TimestampsToMatplotlib(Module):
@@ -282,12 +325,12 @@ class TimestampsToMatplotlib(Module):
                     'linux-debian': 'python-matplotlib',
                     'linux-ubuntu': 'python-matplotlib',
                     'linux-fedora': 'python-matplotlib'})
-        except ImportError:
+        except ImportError: # pragma: no cover
             raise ModuleError(self, "matplotlib is not available")
 
-        timestamps = self.getInputFromPort('timestamps')
+        timestamps = self.get_input('timestamps')
         result = self.convert(timestamps)
-        self.setResult('dates', result)
+        self.set_output('dates', result)
 
 
 class StringsToMatplotlib(Module):
@@ -316,19 +359,19 @@ class StringsToMatplotlib(Module):
                     'linux-debian': 'python-matplotlib',
                     'linux-ubuntu': 'python-matplotlib',
                     'linux-fedora': 'python-matplotlib'})
-        except ImportError:
+        except ImportError: # pragma: no cover
             raise ModuleError(self, "matplotlib is not available")
 
-        tz = self.getInputFromPort('timezone')
+        tz = self.get_input('timezone')
 
-        strings = self.getInputFromPort('strings')
-        fmt = self.getInputFromPort('format')
+        strings = self.get_input('strings')
+        fmt = self.get_input('format')
 
         try:
             result = self.convert(strings, fmt, tz)
         except ValueError, e:
             raise ModuleError(self, e.message)
-        self.setResult('dates', result)
+        self.set_output('dates', result)
 
 
 _modules = {'dates': [
@@ -365,7 +408,7 @@ class TestTimestampToDates(unittest.TestCase):
                  '2013-01-02 19:05:00 UTC +0000'])
         try:
             import pytz
-        except ImportError:
+        except ImportError: # pragma: no cover
             pass
         else:
             self.assertEqual(
@@ -406,7 +449,7 @@ class TestStringsToDates(unittest.TestCase):
         """
         try:
             import dateutil
-        except ImportError:
+        except ImportError: # pragma: no cover
             self.skipTest("dateutil is not available")
 
         dates = ['2013-05-20 9:25',
@@ -455,9 +498,9 @@ class TestStringsToDates(unittest.TestCase):
         """
         try:
             import pytz
-        except ImportError:
+        except ImportError: # pragma: no cover
             self.skipTest("pytz is not available")
-        if LooseVersion(pytz.__version__) < LooseVersion('2012'):
+        if LooseVersion(pytz.__version__) < PYTZ_MIN_VER: # pragma: no cover
             self.skipTest("pytz version is known to cause issues (%s)" %
                           pytz.__version__)
 
@@ -490,7 +533,7 @@ class TestDatesToMatplotlib(unittest.TestCase):
         """
         try:
             import matplotlib
-        except ImportError:
+        except ImportError: # pragma: no cover
             self.skipTest("matplotlib is not available")
 
         from matplotlib.dates import date2num
@@ -533,7 +576,7 @@ class TestTimestampsToMatplotlib(unittest.TestCase):
         """
         try:
             import matplotlib
-        except ImportError:
+        except ImportError: # pragma: no cover
             self.skipTest("matplotlib is not available")
 
         from matplotlib.dates import date2num
@@ -557,7 +600,7 @@ class TestStringsToMatplotlib(unittest.TestCase):
         """
         try:
             import matplotlib
-        except ImportError:
+        except ImportError: # pragma: no cover
             self.skipTest("matplotlib is not available")
 
         from matplotlib.dates import date2num
