@@ -1,46 +1,50 @@
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
 
 ##############################################################################
 # Transfer Function Widget for VTK
+from __future__ import division
+
 from PyQt4 import QtCore, QtGui
-from vistrails.gui.modules.constant_configuration import ConstantWidgetMixin
-from vistrails.core.modules.basic_modules import new_constant, init_constant, Module
+from vistrails.core.modules.vistrails_module import Module
+from vistrails.core.modules.basic_modules import Constant
 from vistrails.core.modules.module_registry import get_module_registry
 from vistrails.core.system import get_elementtree_library
 from vistrails.core.utils.color import ColorByName
+from vistrails.gui.modules.constant_configuration import ConstantWidgetMixin
 import vtk
 import math
 import pickle
@@ -49,8 +53,6 @@ import StringIO
 import unittest
 ElementTree = get_elementtree_library()
 
-from identifiers import identifier as vtk_pkg_identifier
-
 ################################################################################
 # etc
 
@@ -175,7 +177,7 @@ class TransferFunction(object):
             
         return ElementTree.tostring(node)
    
-    @staticmethod         
+    @staticmethod
     def parse(strNode):
         """parse(strNode: str) -> TransferFunction
         Parses a string representing a TransferFunction and returns a 
@@ -210,6 +212,8 @@ class TransferFunction(object):
                                  float(colorNode.get('G','0.0')),
                                  float(colorNode.get('B','0.0')))
                         break
+                else:
+                    assert "'point' node has no 'color' child"
                 tf._pts.append((scalar,opacity,color))
         tf._pts.sort()
         return tf
@@ -261,7 +265,7 @@ class TransferFunctionPoint(QtGui.QGraphicsEllipseItem):
 
     def refresh(self):
         dx = self._fsx * 0.025 / self._sx
-        dy = self._fsy * 0.025/ self._sy
+        dy = self._fsy * 0.025 / self._sy
         # this is the setup
         self.setBrush(QtGui.QBrush(self._color))
         self.setRect(-dx,
@@ -303,8 +307,8 @@ class TransferFunctionPoint(QtGui.QGraphicsEllipseItem):
                 self._left_line.refresh()
             if self._right_line:
                 self._right_line.refresh()
-            if self.scene():
-                self.scene()._tf_poly.setup()
+            if self.parentItem():
+                self.parentItem()._tf_poly.setup()
             self.setToolTip("Double-click to change color\n"
                         "Right-click to remove point\n"
                         "Scalar: %.5f, Opacity: %.5f" % (self._scalar,
@@ -316,15 +320,15 @@ class TransferFunctionPoint(QtGui.QGraphicsEllipseItem):
         if not self._left_line or not self._right_line:
             # Ignore, self is a corner node that can't be removed
             return
-        
+
         # Removes the right line and self, re-ties data structure
         self._left_line._point_right = self._right_line._point_right
         self._left_line._point_right._left_line = self._left_line
-        
+
         # be friends with garbage collector
         self._right_line._point_left = None
         self._right_line._point_right = None
-        self.scene()._tf_poly.setup()
+        self.parentItem()._tf_poly.setup()
         self.scene().removeItem(self._right_line)
         self.scene().removeItem(self)
         self._left_line.refresh()
@@ -339,8 +343,10 @@ class TransferFunctionPoint(QtGui.QGraphicsEllipseItem):
         if self._right_line:
             self._right_line.refresh()
         self.refresh()
-        self.scene()._tf_poly.setup()
-        QtGui.QGraphicsEllipseItem.mouseDoubleClickEvent(self, event)
+        # sometimes the graphicsitem gets recreated, and we need to abort
+        if self.parentItem():
+            self.parentItem()._tf_poly.setup()
+            QtGui.QGraphicsEllipseItem.mouseDoubleClickEvent(self, event)
 
     def mousePressEvent(self, event):
         if event.button() == QtCore.Qt.RightButton:
@@ -348,18 +354,18 @@ class TransferFunctionPoint(QtGui.QGraphicsEllipseItem):
             self.remove_self()
         else:
             QtGui.QGraphicsEllipseItem.mousePressEvent(self, event)
-        
+
     def paint(self, painter, option, widget=None):
         """ paint(painter: QPainter, option: QStyleOptionGraphicsItem,
                   widget: QWidget) -> None
         Peform painting of the point without the ugly default dashed-line black
         square
-        
+
         """
         painter.setBrush(self.brush())
         painter.setPen(self.pen())
         painter.drawEllipse(self.rect())
-        
+
     def add_self_to_transfer_function(self, tf):
         tf.add_point(self._scalar,
                      self._opacity,
@@ -369,16 +375,15 @@ class TransferFunctionPoint(QtGui.QGraphicsEllipseItem):
 
 class TransferFunctionPolygon(QtGui.QGraphicsPolygonItem):
 
-    def __init__(self):
-        QtGui.QGraphicsPolygonItem.__init__(self)
+    def __init__(self, parent=None):
+        QtGui.QGraphicsPolygonItem.__init__(self, parent)
 
     def setup(self):
         # This inspects the scene, finds the left-most point, and
         # then builds the polygon traversing the linked list structure
-        if not self.scene():
+        pt = self.parentItem().get_leftmost_point()
+        if not pt:
             return
-        pt = self.scene().get_leftmost_point()
-        first_pt = pt
         self.setZValue(1.25)
         g = QtGui.QLinearGradient()
         g.setStart(0.0, 0.5)
@@ -468,7 +473,7 @@ class TransferFunctionLine(QtGui.QGraphicsPolygonItem):
         self.setup(self._sx, self._sy)
 
     def mouseDoubleClickEvent(self, event):
-        p = event.scenePos()
+        p = event.pos()
         c_left = self._point_left._color
         c_right = self._point_right._color
         u = ((p.x() - self._point_left._point.x()) /
@@ -476,12 +481,12 @@ class TransferFunctionLine(QtGui.QGraphicsPolygonItem):
         new_c = (u * c_right.redF() + (1-u) * c_left.redF(),
                  u * c_right.greenF() + (1-u) * c_left.greenF(),
                  u * c_right.blueF() + (1-u) * c_left.blueF())
-        new_point = TransferFunctionPoint(p.x()/ self._fsx, p.y()/self._fsy, new_c)
-        new_line = TransferFunctionLine(new_point, self._point_right)
+        new_point = TransferFunctionPoint(p.x()/ self._fsx, p.y()/self._fsy, new_c, self.parentItem())
+        self.parentItem()._tf_items.append(new_point)
+        new_line = TransferFunctionLine(new_point, self._point_right, self.parentItem())
+        self.parentItem()._tf_items.append(new_line)
         new_point._left_line = self
         self._point_right = new_point
-        self.scene().addItem(new_line)
-        self.scene().addItem(new_point)
         new_line.update_scale(self._point_left._sx,
                               self._point_left._sy)
         new_point.update_scale(self._point_left._sx,
@@ -493,25 +498,29 @@ class TransferFunctionLine(QtGui.QGraphicsPolygonItem):
         # This needs to be here, otherwise mouseDoubleClickEvent does
         # not get called.
         event.accept()
-        
 
 ##############################################################################
 # Scene, view, widget
 
-class TransferFunctionScene(QtGui.QGraphicsScene):
-
-    def __init__(self, tf, parent=None):
-        QtGui.QGraphicsScene.__init__(self, parent)
+class QGraphicsTransferFunction(QtGui.QGraphicsWidget, ConstantWidgetMixin):
+    contentsChanged = QtCore.pyqtSignal(tuple)
+    def __init__(self, param, parent=None):
+        QtGui.QGraphicsWidget.__init__(self, parent)
+        ConstantWidgetMixin.__init__(self, param.strValue)
+        self.setAcceptHoverEvents(True)
+        if not param.strValue:
+            self._tf = copy.copy(default_tf)
+        else:
+            self._tf = TransferFunction.parse(param.strValue)
         self._tf_items = []
-        poly = TransferFunctionPolygon()
+        poly = TransferFunctionPolygon(self)
         poly.setup()
         self._tf_poly = poly
-        self.addItem(poly)
-        self.create_tf_items(tf)
+        self.create_tf_items(self._tf)
         self._tf_poly.setup()
         #current scale
         self._sx = 1.0
-        self._sy = 1.0    
+        self._sy = 1.0
         # Add outlines
         line_color = QtGui.QColor(200, 200, 200)
         pen = QtGui.QPen(line_color)
@@ -519,54 +528,51 @@ class TransferFunctionScene(QtGui.QGraphicsScene):
               QtCore.QPointF(GLOBAL_SCALE, 0.0),
               QtCore.QPointF(GLOBAL_SCALE, GLOBAL_SCALE),
               QtCore.QPointF(0.0, GLOBAL_SCALE)]
-        outline = QtGui.QPolygonF(ps)
-        self.addPolygon(outline, pen)
+        polygon = QtGui.QGraphicsPolygonItem(QtGui.QPolygonF(ps), self)
+        polygon.setPen(pen)
 
         for i in xrange(51):
             u = GLOBAL_SCALE * float(i) / 50.0
-            self.addLine(QtCore.QLineF(u, 0.0, u, GLOBAL_SCALE), pen)
-            self.addLine(QtCore.QLineF(0.0, u, GLOBAL_SCALE, u), pen)
+
+            line = QtGui.QGraphicsLineItem(QtCore.QLineF(u, 0.0, u, GLOBAL_SCALE), self)
+            line.setPen(pen)
+            line = QtGui.QGraphicsLineItem(QtCore.QLineF(0.0, u, GLOBAL_SCALE, u), self)
+            line.setPen(pen)
+
+        self.setGeometry(self.boundingRect())
+        # restore y axis inversion
+        self.setTransform(QtGui.QTransform(1, 0, 0, -1, 0, GLOBAL_SCALE))
+        self.setTransformOriginPoint(0, GLOBAL_SCALE)
+        self.reset_transfer_function(self._tf)
+
+    def boundingRect(self):
+        return QtCore.QRectF(0.0, 0.0, GLOBAL_SCALE, GLOBAL_SCALE)
 
     def reset_transfer_function(self, tf):
         self.create_tf_items(tf)
         self.update_scale(self._sx, self._sy)
         self._tf_poly.setup()
-        
-    def removeItem(self, item):
-        if item in self._tf_items:
-            self._tf_items.remove(item)
-        QtGui.QGraphicsScene.removeItem(self, item)
-
-    def addItem(self, item):
-        # Ugly, but hey
-        if isinstance(item, TransferFunctionLine) or \
-           isinstance(item, TransferFunctionPoint):
-            self._tf_items.append(item)
-        QtGui.QGraphicsScene.addItem(self, item)
 
     def create_tf_items(self, tf):
+        if self._tf_items and not self.scene(): # not added to scene yet
+            return
         items = copy.copy(self._tf_items)
         for item in items:
-            self.removeItem(item)
+            self.scene().removeItem(item)
         self._tf_items = []
         if len(tf._pts) == 0:
-            pt_left = TransferFunctionPoint(0.0, 0.0, (0.0, 0.0, 0.0))
-            pt_right = TransferFunctionPoint(1.0, 0.0, (0.0, 0.0, 0.0))
-            line = TransferFunctionLine(pt_left, pt_right)
-            
-            self.addItem(pt_left)
-            self.addItem(pt_right)
-            self.addItem(line)
-            
+            pt_left = TransferFunctionPoint(0.0, 0.0, (0.0, 0.0, 0.0), self)
+            self._tf_items.append(pt_left)
+            pt_right = TransferFunctionPoint(1.0, 0.0, (0.0, 0.0, 0.0), self)
+            self._tf_items.append(pt_right)
+            self._tf_items.append(TransferFunctionLine(pt_left, pt_right, self))
         else:
-            pts = [TransferFunctionPoint(*pt)
+            pts = [TransferFunctionPoint(*pt, parent=self)
                    for pt in tf._pts]
-            lines = [TransferFunctionLine(pt_l, pt_r)
-                     for (pt_l, pt_r) in zip(pts[:-1], pts[1:])]
-            for pt in pts:
-                self.addItem(pt)
-            for line in lines:
-                self.addItem(line)
+            self._tf_items.extend(pts)
+            lns = [TransferFunctionLine(pt_l, pt_r, self)
+                   for (pt_l, pt_r) in zip(pts[:-1], pts[1:])]
+            self._tf_items.extend(lns)
 
     def add_knot(self, scalar, opacity):
         pass
@@ -584,8 +590,7 @@ class TransferFunctionScene(QtGui.QGraphicsScene):
             if hasattr(item, '_left_line') and not item._left_line:
                 pt = item
                 break
-        assert pt
-        return pt        
+        return pt
 
     def get_transfer_function(self):
         result = TransferFunction()
@@ -598,6 +603,29 @@ class TransferFunctionScene(QtGui.QGraphicsScene):
                 break
         return result
 
+    def contents(self):
+        return self.get_transfer_function().serialize()
+
+    def setContents(self, strValue, silent=True):
+        if not strValue:
+            self._tf = copy.copy(default_tf)
+        else:
+            self._tf = TransferFunction.parse(strValue)
+        self.reset_transfer_function(self._tf)
+        if not silent:
+            self.update_parent()    
+
+    def hoverLeaveEvent(self, event):
+        self.update_parent()
+        QtGui.QGraphicsWidget.hoverLeaveEvent(self, event)
+
+class TransferFunctionScene(QtGui.QGraphicsScene):
+
+    def __init__(self, param, parent=None):
+        QtGui.QGraphicsScene.__init__(self, parent)
+        self.tf = QGraphicsTransferFunction(param)
+        self.addItem(self.tf)
+
 class TransferFunctionView(QtGui.QGraphicsView):
     def __init__(self, parent=None):
         QtGui.QGraphicsView.__init__(self, parent)
@@ -608,8 +636,8 @@ class TransferFunctionView(QtGui.QGraphicsView):
     def resizeEvent(self, event):
         self.resetMatrix()
         self.setMatrix(QtGui.QMatrix(event.size().width() / (GLOBAL_SCALE *10.0/9) , 0,
-                                     0, -event.size().height() / (GLOBAL_SCALE*10.0/9), 0, 0))
-        self.scene().update_scale(event.size().width()/(2000.0/9), event.size().height()/(2000.0/9))
+                                     0, event.size().height() / (GLOBAL_SCALE*10.0/9), GLOBAL_SCALE, 0))
+        self.scene().tf.update_scale(event.size().width()/(2000.0/9), event.size().height()/(2000.0/9))
         
     def focusOutEvent(self, event):
         self.parent().update_parent()
@@ -618,17 +646,16 @@ class TransferFunctionView(QtGui.QGraphicsView):
 default_tf = TransferFunction()
 default_tf.add_point(0.0, 0.0, (0.0, 0.0, 0.0))
 default_tf.add_point(1.0, 0.0, (0.0, 0.0, 0.0))
-    
+
 class TransferFunctionWidget(QtGui.QWidget, ConstantWidgetMixin):
+    contentsChanged = QtCore.pyqtSignal(tuple)
+
+    GraphicsItem = QGraphicsTransferFunction
 
     def __init__(self, param, parent=None):
         QtGui.QWidget.__init__(self, parent)
-        ConstantWidgetMixin.__init__(self, param.strValue)
-        if not param.strValue:
-            self._tf = copy.copy(default_tf)
-        else:
-            self._tf = TransferFunction.parse(param.strValue)
-        self._scene = TransferFunctionScene(self._tf, self)
+        self._scene = TransferFunctionScene(param, self)
+        self._scene.tf.update_parent = self.update_parent
         layout = QtGui.QVBoxLayout()
         self.setLayout(layout)
         self._view = TransferFunctionView(self)
@@ -638,7 +665,8 @@ class TransferFunctionWidget(QtGui.QWidget, ConstantWidgetMixin):
         self._view.show()
         self._view.setSizePolicy(QtGui.QSizePolicy.Expanding,
                                  QtGui.QSizePolicy.Expanding)
-        self._view.setMatrix(QtGui.QMatrix(1, 0, 0, -1, 0, 0))
+        # TODO remove this
+        self._view.setMatrix(QtGui.QMatrix(1, 0, 0, -1, GLOBAL_SCALE, 0))
         self.setMinimumSize(260,240)
         caption = QtGui.QLabel("Double-click on the line to add a point")
         font = QtGui.QFont('Arial', 11)
@@ -648,68 +676,77 @@ class TransferFunctionWidget(QtGui.QWidget, ConstantWidgetMixin):
         layout.addWidget(caption)
 
     def contents(self):
-        return self._scene.get_transfer_function().serialize()
-    
+        return self._scene.tf.contents()
+
     def setContents(self, strValue, silent=True):
-        if not strValue:
-            self._tf = copy.copy(default_tf)
-        else:
-            self._tf = TransferFunction.parse(strValue)
-        self._scene.reset_transfer_function(self._tf)
-        if not silent:
-            self.update_parent()    
-            
+        self._scene.tf.setContents(strValue, silent)
+
+    def set_last_contents(self, contents):
+        self._scene.tf._last_contents = contents
+    def get_last_contents(self):
+        return self._scene.tf._last_contents
+    _last_contents = property(get_last_contents, set_last_contents)
+
 ##############################################################################
 # Helper module to adjust range
 
 class vtkScaledTransferFunction(Module):
 
+    # FIXME Add documentation
+    _input_ports = [
+        ['Input', 'vtkAlgorithmOutput'],
+        ['Dataset', 'vtkDataObject'],
+        ['Range', '(basic:Float, basic:Float)'],
+        ['TransferFunction', 'TransferFunction']]
+
+    _output_ports = [
+        ['TransferFunction', 'TransferFunction'],
+        ['vtkPiecewiseFunction', 'vtkPiecewiseFunction'],
+        ['vtkColorTransferFunction', 'vtkColorTransferFunction']]
+
     def compute(self):
         reg = get_module_registry()
-        tf = self.getInputFromPort('TransferFunction')
+        tf = self.get_input('TransferFunction')
         new_tf = copy.copy(tf)
-        if self.hasInputFromPort('Input'):
-            port = self.getInputFromPort('Input')
-            algo = port.vtkInstance.GetProducer()
-            output = algo.GetOutput(port.vtkInstance.GetIndex())
+        if self.has_input('Input'):
+            port = self.get_input('Input')
+            algo = port.GetProducer()
+            output = algo.GetOutput(port.GetIndex())
             (new_tf._min_range, new_tf._max_range) = output.GetScalarRange()
-        elif self.hasInputFromPort('Dataset'):
-            algo = self.getInputFromPort('Dataset').vtkInstance
+        elif self.has_input('Dataset'):
+            algo = self.get_input('Dataset')
             output = algo
             (new_tf._min_range, new_tf._max_range) = output.GetScalarRange()
         else:
-            (new_tf._min_range, new_tf._max_range) = self.getInputFromPort('Range')
-            
-        self.setResult('TransferFunction', new_tf)
+            (new_tf._min_range, new_tf._max_range) = self.get_input('Range')
+
+        self.set_output('TransferFunction', new_tf)
         (of,cf) = new_tf.get_vtk_transfer_functions()
         
-        of_module = reg.get_descriptor_by_name(vtk_pkg_identifier, 
-                                               'vtkPiecewiseFunction').module()
-        of_module.vtkInstance  = of
-        
-        cf_module = reg.get_descriptor_by_name(vtk_pkg_identifier, 
-                                               'vtkColorTransferFunction').module()
-        cf_module.vtkInstance  = cf
-        
-        self.setResult('vtkPicewiseFunction', of_module)
-        self.setResult('vtkColorTransferFunction', cf_module)
+        self.set_output('vtkPicewiseFunction', of)
+        self.set_output('vtkColorTransferFunction', cf)
 
-string_conversion = staticmethod(lambda x: x.serialize())
-conversion = staticmethod(lambda x: TransferFunction.parse(x))
-validation = staticmethod(lambda x: isinstance(x, TransferFunction))
-TransferFunctionConstant = new_constant('TransferFunction',
-                                        conversion,
-                                        default_tf,
-                                        validation,
-                                        TransferFunctionWidget)
-TransferFunctionConstant.translate_to_string = string_conversion
+class TransferFunctionConstant(Constant):
+    default_value = default_tf
+
+    @staticmethod
+    def translate_to_python(x):
+        return TransferFunction.parse(x)
+
+    @staticmethod
+    def translate_to_string(x):
+        return x.serialize()
+
+    @staticmethod
+    def validate(x):
+        return isinstance(x, TransferFunction)
+
+    @staticmethod
+    def get_widget_class():
+        return TransferFunctionWidget
 
 ##############################################################################
 
-def initialize():
-    init_constant(TransferFunctionConstant)
-    
-##############################################################################
 class TestTransferFunction(unittest.TestCase):
     def test_serialization(self):
         tf = TransferFunction()
@@ -731,6 +768,10 @@ class TestTransferFunction(unittest.TestCase):
         assert tf == tf1
         assert tf == tf2
         assert tf1 == tf2
-        
+
+TransferFunctionConstant.__name__ = "TransferFunction"
+
+_modules = [TransferFunctionConstant, vtkScaledTransferFunction]
+
 if __name__ == "__main__":
     unittest.main()
