@@ -1,35 +1,37 @@
 #!/usr/bin/env python
+# pragma: no testimport
 ###############################################################################
 ##
+## Copyright (C) 2014-2015, New York University.
 ## Copyright (C) 2011-2014, NYU-Poly.
-## Copyright (C) 2006-2011, University of Utah. 
+## Copyright (C) 2006-2011, University of Utah.
 ## All rights reserved.
 ## Contact: contact@vistrails.org
 ##
 ## This file is part of VisTrails.
 ##
-## "Redistribution and use in source and binary forms, with or without 
+## "Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##
-##  - Redistributions of source code must retain the above copyright notice, 
+##  - Redistributions of source code must retain the above copyright notice,
 ##    this list of conditions and the following disclaimer.
-##  - Redistributions in binary form must reproduce the above copyright 
-##    notice, this list of conditions and the following disclaimer in the 
+##  - Redistributions in binary form must reproduce the above copyright
+##    notice, this list of conditions and the following disclaimer in the
 ##    documentation and/or other materials provided with the distribution.
-##  - Neither the name of the University of Utah nor the names of its 
-##    contributors may be used to endorse or promote products derived from 
+##  - Neither the name of the New York University nor the names of its
+##    contributors may be used to endorse or promote products derived from
 ##    this software without specific prior written permission.
 ##
-## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
-## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
-## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
+## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 ## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 ##
 ###############################################################################
@@ -43,68 +45,146 @@ any unit tests, as a crude measure of code coverage.
 
 """
 
-# First, import unittest, replacing it with unittest2 if necessary
-import sys
-try:
-    import unittest2
-except ImportError:
-    pass
-else:
-    sys.modules['unittest'] = unittest2
-import unittest
-
 import atexit
 from distutils.version import LooseVersion
 #import doctest
+import locale
 import os
+import sys
 import traceback
-import os.path
-import optparse
 from optparse import OptionParser
 import platform
+import re
 import shutil
 import tempfile
 
-# Makes sure we can import modules as if we were running VisTrails
-# from the root directory
-_this_dir = os.path.dirname(os.path.realpath(__file__))
-root_directory = os.path.realpath(os.path.join(_this_dir,  '..'))
-sys.path.insert(0, os.path.realpath(os.path.join(root_directory, '..')))
+if 'vistrails' not in sys.modules:
+    # Makes sure we can import modules as if we were running VisTrails
+    # from the root directory
+    _this_dir = os.path.dirname(os.path.realpath(__file__))
+    _root_directory = os.path.realpath(os.path.join(_this_dir,  '..'))
+    sys.path.insert(0, os.path.realpath(os.path.join(_root_directory, '..')))
 
 # Use a different temporary directory
 test_temp_dir = tempfile.mkdtemp(prefix='vt_testsuite_')
 tempfile.tempdir = test_temp_dir
-@atexit.register
-def clean_tempdir():
-    nb_dirs = 0
-    nb_files = 0
-    for f in os.listdir(test_temp_dir):
-        if os.path.isdir(f):
-            nb_dirs += 1
-        else:
-            nb_files += 1
-    if nb_dirs > 0 or nb_files > 0:
-        sys.stdout.write("Warning: %d dirs and %d files were left behind in "
-                         "tempdir, cleaning up\n" % (nb_dirs, nb_files))
-    shutil.rmtree(test_temp_dir, ignore_errors=True)
+@apply
+class clean_tempdir(object):
+    def __init__(self):
+        atexit.register(self.clean)
+        self.listdir = os.listdir
+        self.isdir = os.path.isdir
+        self.test_temp_dir = test_temp_dir
+        self.rmtree = shutil.rmtree
+        self.out = sys.stdout.write
+    def clean(self):
+        nb_dirs = 0
+        nb_files = 0
+        for f in self.listdir(self.test_temp_dir):
+            if self.isdir(f):
+                nb_dirs += 1
+            else:
+                nb_files += 1
+        if nb_dirs > 0 or nb_files > 0:
+            self.out("Warning: %d dirs and %d files were left behind in "
+                     "tempdir, cleaning up\n" % (nb_dirs, nb_files))
+        self.rmtree(self.test_temp_dir, ignore_errors=True)
 
+# Parse the command-line
+usage = "Usage: %prog [options] [module1 module2 ...]"
+parser = OptionParser(usage=usage)
+parser.add_option("-V", "--verbose", action="store", type="int",
+                  default=0, dest="verbose",
+                  help="set verboseness level(0--2, default=0, "
+                  "higher means more verbose)")
+parser.add_option("-v", "--vistrails-verbose", action="store", type="int",
+                  default=0, dest="debugLevel",
+                  help="set the debugLevel in VisTrails (0--2, default=0)")
+parser.add_option("-e", "--examples", action="store_true",
+                  default=False,
+                  help="run vistrails examples")
+parser.add_option("-i", "--images", action="store_true",
+                  default=False,
+                  help="perform image comparisons")
+parser.add_option("--installbundles", action='store_true',
+                  default=False,
+                  help=("Attempt to install missing Python packages "
+                        "automatically"))
+parser.add_option("-S", "--startup", action="store", type="str", default=None,
+                  dest="dotVistrails",
+                  help="Set startup file (default is temporary directory)")
+parser.add_option('-L', '--locale', action='store', type='str', default='',
+                  dest='locale',
+                  help="set locale to this string")
+parser.add_option('-D', '--debug', action='store_true',
+                  default=False,
+                  help="start interactive debugger on unexpected error")
+parser.add_option('--no-unbuffered', action='store_false', dest='unbuffered',
+                  default=True,
+                  help="Don't make output stream unbuffered")
+
+(options, test_modules) = parser.parse_args()
+# remove empty strings
+test_modules = filter(len, test_modules)
+verbose = options.verbose
+locale.setlocale(locale.LC_ALL, options.locale or '')
+test_examples = options.examples
+test_images = options.images
+installbundles = options.installbundles
+dotVistrails = options.dotVistrails
+debug_mode = options.debug
+vistrails_verbose = options.debugLevel
+
+# Makes stdout unbuffered, so python -u is not needed
+class Unbuffered(object):
+   def __init__(self, stream):
+       self.stream = stream
+   def write(self, data):
+       self.stream.write(data)
+       self.stream.flush()
+   def __getattr__(self, attr):
+       return getattr(self.stream, attr)
+
+if options.unbuffered:
+    sys.stdout = Unbuffered(sys.stdout)
+    sys.stderr = Unbuffered(sys.stderr)
+
+# Use PyQt API v2
 def setNewPyQtAPI():
     try:
         import sip
         # We now use the new PyQt API - IPython needs it
         sip.setapi('QString', 2)
         sip.setapi('QVariant', 2)
-    except:
+    except Exception:
         print "Could not set PyQt API, is PyQt4 installed?"
 setNewPyQtAPI()
 
+# Log to the console
+import vistrails.core.debug
+vistrails.core.debug.DebugPrint.getInstance().log_to_console()
+
 import vistrails.tests
 import vistrails.core
 import vistrails.core.db.io
 import vistrails.core.db.locator
+from vistrails.core import debug
 import vistrails.gui.application
 from vistrails.core.system import vistrails_root_directory, \
                                   vistrails_examples_directory
+from vistrails.core.packagemanager import get_package_manager
+
+# VisTrails does funny stuff with unittest/unittest2, be sure to load that
+# after vistrails
+import unittest
+
+# reinitializing arguments and options so VisTrails does not try parsing them
+sys.argv = sys.argv[:1]
+vistrails.gui.application.VistrailsApplicationSingleton.use_event_filter = \
+        False
+
+
+root_directory = os.path.realpath(vistrails_root_directory())
 
 ###############################################################################
 # Testing Examples
@@ -147,39 +227,12 @@ def sub_print(s, overline=False):
 
 ###############################################################################
 
-usage = "Usage: %prog [options] [module1 module2 ...]"
-parser = OptionParser(usage=usage)
-parser.add_option("-V", "--verbose", action="store", type="int",
-                  default=0, dest="verbose",
-                  help="set verboseness level(0--2, default=0, "
-                  "higher means more verbose)")
-parser.add_option("-e", "--examples", action="store_true",
-                  default=False,
-                  help="run vistrails examples")
-parser.add_option("-i", "--images", action="store_true",
-                  default=False,
-                  help="perform image comparisons")
-parser.add_option("--installbundles", action='store_true',
-                  default=False,
-                  help=("Attempt to install missing Python packages "
-                        "automatically"))
-parser.add_option("-S", "--startup", action="store", type="str", default=None,
-                  dest="dotVistrails",
-                  help="Set startup file (default is temporary directory)")
-
-(options, args) = parser.parse_args()
-# remove empty strings
-args = filter(len, args)
-verbose = options.verbose
-test_examples = options.examples
-test_images = options.images
-installbundles = options.installbundles
-dotVistrails = options.dotVistrails
-test_modules = None
-if len(args) > 0:
-    test_modules = args
+if len(test_modules) > 0:
+    test_modules = test_modules
 else:
-    test_images = True
+    test_modules = None
+    if os.path.exists(EXAMPLES_PATH):
+        test_images = True
 
 def module_filter(name):
     if test_modules is None:
@@ -190,20 +243,19 @@ def module_filter(name):
     return False
 
 ###############################################################################
-# reinitializing arguments and options so VisTrails does not try parsing them
-sys.argv = sys.argv[:1]
-
 # creates the app so that testing can happen
 
 # We need the windows so we can test events, etc.
 optionsDict = {
-        'interactiveMode': True,
-        'nologger': True,
+        'batch': False,
+        'executionLog': False,
         'singleInstance': False,
-        'fixedSpreadsheetCells': True,
         'installBundles': installbundles,
         'enablePackagesSilently': True,
         'handlerDontAsk': True,
+        'developerDebugger': debug_mode,
+        'debugLevel': vistrails_verbose,
+        'dontUnloadModules': True,
     }
 if dotVistrails:
     optionsDict['dotVistrails'] = dotVistrails
@@ -216,12 +268,17 @@ if v != 0:
         app.finishSession()
     sys.exit(v)
 
+# make sure that fixedCellSize is turned on
+spreadsheet_conf = get_package_manager().get_package_configuration("spreadsheet")
+spreadsheet_conf.fixedCellSize = True
+
 # disable first vistrail
 app = vistrails.gui.application.get_vistrails_application()
 app.builderWindow.auto_view = False
 app.builderWindow.close_all_vistrails(True)
 
 print "Test Suite for VisTrails"
+print "Locale settings: %s" % ', '.join('%s: %s' % (s, locale.setlocale(getattr(locale, s), None)) for s in ('LC_ALL', 'LC_TIME'))
 print "Running on %s" % ', '.join(platform.uname())
 print "Python is %s" % sys.version
 try:
@@ -249,6 +306,8 @@ tests_passed = True
 main_test_suite = unittest.TestSuite()
 test_loader = unittest.TestLoader()
 
+import_skip_regex = re.compile(r'(?i)# *pragma[: ]*no *testimport')
+
 if test_modules:
     sub_print("Trying to import some of the modules")
 else:
@@ -262,6 +321,7 @@ for (p, subdirs, files) in os.walk(root_directory):
         # skip files that don't look like VisTrails python modules
         if not filename.endswith('.py'):
             continue
+        module_file = os.path.join(p, filename)
         module = os.path.join("vistrails", p[len(root_directory)+1:],
                               filename[:-3])
         if (module.startswith(os.sep) or
@@ -277,15 +337,20 @@ for (p, subdirs, files) in os.walk(root_directory):
 
         if not module_filter(module):
             continue
-        if module.startswith('vistrails.tests.run'):
-            continue
         if module.startswith('vistrails.tests.resources'):
             continue
         if ('.system.' in module and not
             module.endswith('__init__')):
             continue
-
-        msg = ("%s %s |" % (" " * (40 - len(module)), module))
+        with open(module_file) as fp:
+            l = fp.readline()
+            if l.startswith('#!'): # shebang
+                l = fp.readline()
+            if import_skip_regex.match(l):
+                if verbose >= 1:
+                    print >>sys.stderr, ("Skipping %s, not an importable "
+                                         "module" % module)
+                continue
 
         m = None
         try:
@@ -293,14 +358,10 @@ for (p, subdirs, files) in os.walk(root_directory):
                 m = __import__(module, globals(), locals(), ['foo'])
             else:
                 m = __import__(module)
-        except vistrails.tests.NotModule:
+        except BaseException:
+            print >>sys.stderr, "ERROR: Could not import module: %s" % module
             if verbose >= 1:
-                print "Skipping %s, not an importable module" % filename
-            continue
-        except:
-            print msg, "ERROR: Could not import module!"
-            if verbose >= 1:
-                traceback.print_exc(file=sys.stdout)
+                traceback.print_exc(file=sys.stderr)
             continue
 
         # Load the unittest TestCases
@@ -320,9 +381,11 @@ for (p, subdirs, files) in os.walk(root_directory):
         main_test_suite.addTests(suite)
 
         if suite.countTestCases() == 0 and verbose >= 1:
-            print msg, "WARNING: %s has no tests!" % filename
+            print >>sys.stderr, "WARNING: module has no tests: %s" % module
         elif verbose >= 2:
-            print msg, "Ok: %d test cases." % suite.countTestCases()
+            print >>sys.stderr, "OK: module as %d test cases: %s" % (
+                    suite.countTestCases(),
+                    module)
 
 sub_print("Imported modules. Running %d tests%s..." % (
           main_test_suite.countTestCases(),
@@ -360,8 +423,13 @@ if compare_use_vtk:
         a = removeAlpha(prev)
         b = removeAlpha(next)
         idiff = vtk.vtkImageDifference()
-        idiff.SetInput(a)
-        idiff.SetImage(b)
+        if LooseVersion(vtk.vtkVersion().GetVTKVersion()) >= \
+           LooseVersion('6.0.0'):
+            idiff.SetInputData(a)
+            idiff.SetImageData(b)
+        else:
+            idiff.SetInput(a)
+            idiff.SetImage(b)
         idiff.Update()
         return idiff.GetThresholdedError()
 else:
@@ -407,7 +475,7 @@ def image_test_generator(vtfile, version):
                     print("   *** Error in %s:%s:%s -- %s" % err)
                     self.fail(str(err))
         except Exception, e:
-            self.fail(str(e))
+            self.fail(debug.format_exception(e))
     return test
 
 class TestVistrailImages(unittest.TestCase):
@@ -423,7 +491,15 @@ if test_images:
 
 ############## RUN TEST SUITE ####################
 
-result = unittest.TextTestRunner(verbosity=max(verbose, 1)).run(main_test_suite)
+class TestResult(unittest.TextTestResult):
+    def addSkip(self, test, reason):
+        self.stream.writeln("skipped '{0}': {1}".format(str(test), reason))
+        super(TestResult, self).addSkip(test, reason)
+
+runner = unittest.TextTestRunner(
+        verbosity=max(verbose, 1),
+        resultclass=TestResult)
+result = runner.run(main_test_suite)
 
 if not result.wasSuccessful():
     tests_passed = False
@@ -453,7 +529,7 @@ if test_examples:
                 errs = vistrails.core.console_mode.run(w_list, update_vistrail=False)
                 summary[vtfile] = errs
         except Exception, e:
-            errs.append((vtfile,"None", "None", str(e)))
+            errs.append((vtfile,"None", "None", debug.format_exception(e)))
             summary[vtfile] = errs
         nvtfiles += 1
 
